@@ -502,7 +502,6 @@ static void test_custom_default_button(void)
         }
     }
 
-    todo_wine
     ok(add_button_has_been_pressed, "The Add button has not been pressed!\n");
 
     DestroyWindow(hdlg);
@@ -650,7 +649,7 @@ static LRESULT CALLBACK sheet_callback_messages_proc (HWND hwnd, UINT msg, WPARA
 
 static int CALLBACK sheet_callback_messages(HWND hwnd, UINT msg, LPARAM lParam)
 {
-    save_message(hwnd, msg, (WPARAM)NULL, lParam, RECEIVER_SHEET_CALLBACK);
+    save_message(hwnd, msg, 0, lParam, RECEIVER_SHEET_CALLBACK);
 
     switch (msg)
     {
