@@ -1937,7 +1937,7 @@ TOOLTIPS_MouseMessage (TOOLTIPS_INFO *infoPtr)
 
 
 static LRESULT
-TOOLTIPS_NCCreate (HWND hwnd, const CREATESTRUCTW *lpcs)
+TOOLTIPS_NCCreate (HWND hwnd)
 {
     DWORD dwStyle = GetWindowLongW (hwnd, GWL_STYLE);
     DWORD dwExStyle = GetWindowLongW (hwnd, GWL_EXSTYLE);
@@ -2051,7 +2051,7 @@ TOOLTIPS_OnWMGetText (const TOOLTIPS_INFO *infoPtr, WPARAM size, LPWSTR pszText)
 {
     LRESULT res;
 
-    if(!infoPtr->szTipText || !size)
+    if(!size)
         return 0;
 
     res = min(strlenW(infoPtr->szTipText)+1, size);
@@ -2292,7 +2292,7 @@ TOOLTIPS_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	    return TOOLTIPS_MouseMessage (infoPtr);
 
 	case WM_NCCREATE:
-	    return TOOLTIPS_NCCreate (hwnd, (LPCREATESTRUCTW)lParam);
+	    return TOOLTIPS_NCCreate (hwnd);
 
 	case WM_NCHITTEST:
 	    return TOOLTIPS_NCHitTest (infoPtr, wParam, lParam);
