@@ -18,6 +18,10 @@
 #ifndef __CRYPTUIRES_H_
 #define __CRYPTUIRES_H_
 
+#include <windef.h>
+#include <winuser.h>
+#include <commctrl.h>
+
 #define IDS_CERTIFICATE 1000
 #define IDS_CERTIFICATEINFORMATION 1001
 #define IDS_CERT_INFO_BAD_SIG 1002
@@ -112,6 +116,7 @@
 #define IDS_WARN_REMOVE_DEFAULT 1092
 #define IDS_WARN_REMOVE_PLURAL_DEFAULT 1093
 #define IDS_CERT_MGR 1094
+#define IDS_FRIENDLY_NAME_NONE 1095
 
 #define IDS_PURPOSE_SERVER_AUTH 1100
 #define IDS_PURPOSE_CLIENT_AUTH 1101
@@ -160,6 +165,13 @@
 #define IDS_NO 1217
 #define IDS_EXPORT_SUCCEEDED 1218
 #define IDS_EXPORT_FAILED 1219
+#define IDS_EXPORT_PRIVATE_KEY_TITLE 1220
+#define IDS_EXPORT_PRIVATE_KEY_SUBTITLE 1221
+#define IDS_EXPORT_PASSWORD_TITLE 1222
+#define IDS_EXPORT_PASSWORD_SUBTITLE 1223
+#define IDS_EXPORT_PASSWORD_MISMATCH 1224
+#define IDS_EXPORT_PRIVATE_KEY_UNAVAILABLE 1225
+#define IDS_EXPORT_PRIVATE_KEY_NON_EXPORTABLE 1226
 
 #define IDD_GENERAL 100
 #define IDD_DETAIL 101
@@ -175,9 +187,11 @@
 #define IDD_CERT_MGR 111
 #define IDD_CERT_MGR_ADVANCED 112
 #define IDD_EXPORT_WELCOME 113
-#define IDD_EXPORT_FORMAT 114
-#define IDD_EXPORT_FILE 115
-#define IDD_EXPORT_FINISH 116
+#define IDD_EXPORT_PRIVATE_KEY 114
+#define IDD_EXPORT_PASSWORD 115
+#define IDD_EXPORT_FORMAT 116
+#define IDD_EXPORT_FILE 117
+#define IDD_EXPORT_FINISH 118
 
 #define IDB_SMALL_ICONS 200
 #define IDB_CERT 201
@@ -253,5 +267,10 @@
 #define IDC_EXPORT_FILENAME 2909
 #define IDC_EXPORT_BROWSE_FILE 2910
 #define IDC_EXPORT_SETTINGS 2911
+#define IDC_EXPORT_PRIVATE_KEY_YES 2912
+#define IDC_EXPORT_PRIVATE_KEY_NO 2913
+#define IDC_EXPORT_PRIVATE_KEY_UNAVAILABLE 2914
+#define IDC_EXPORT_PASSWORD 2915
+#define IDC_EXPORT_PASSWORD_CONFIRM 2916
 
 #endif /* ndef __CRYPTUIRES_H_ */
