@@ -721,7 +721,7 @@ HRESULT WINAPI DllCanUnloadNow(void)
  */
 HRESULT WINAPI DllRegisterServer(void)
 {
-    return __wine_register_resources( instance, NULL );
+    return __wine_register_resources( instance );
 }
 
 /***********************************************************************
@@ -729,7 +729,7 @@ HRESULT WINAPI DllRegisterServer(void)
  */
 HRESULT WINAPI DllUnregisterServer(void)
 {
-    return __wine_unregister_resources( instance, NULL );
+    return __wine_unregister_resources( instance );
 }
 
 /*******************************************************************************
@@ -752,14 +752,19 @@ DestroyCallback(IDirectDrawSurface7 *surf,
                 void *context)
 {
     IDirectDrawSurfaceImpl *Impl = impl_from_IDirectDrawSurface7(surf);
-    ULONG ref, ref3, ref2;
+    ULONG ref7, ref4, ref3, ref2, ref1, iface_count;
 
-    ref = IDirectDrawSurface7_Release(surf);  /* For the EnumSurfaces */
+    ref7 = IDirectDrawSurface7_Release(surf);  /* For the EnumSurfaces */
+    IDirectDrawSurface4_AddRef(&Impl->IDirectDrawSurface4_iface);
+    ref4 = IDirectDrawSurface4_Release(&Impl->IDirectDrawSurface4_iface);
     IDirectDrawSurface3_AddRef(&Impl->IDirectDrawSurface3_iface);
     ref3 = IDirectDrawSurface3_Release(&Impl->IDirectDrawSurface3_iface);
     IDirectDrawSurface2_AddRef(&Impl->IDirectDrawSurface2_iface);
     ref2 = IDirectDrawSurface2_Release(&Impl->IDirectDrawSurface2_iface);
-    WARN("Surface %p has an reference counts of %d 3: %d 2: %d\n", Impl, ref, ref3, ref2);
+    IDirectDrawSurface_AddRef(&Impl->IDirectDrawSurface_iface);
+    ref1 = IDirectDrawSurface_Release(&Impl->IDirectDrawSurface_iface);
+    WARN("Surface %p has an reference counts of 7: %u 4: %u 3: %u 2: %u 1: %u\n",
+            Impl, ref7, ref4, ref3, ref2, ref1);
 
     /* Skip surfaces which are attached somewhere or which are
      * part of a complex compound. They will get released when destroying
@@ -769,9 +774,8 @@ DestroyCallback(IDirectDrawSurface7 *surf,
         return DDENUMRET_OK;
 
     /* Destroy the surface */
-    while (ref) ref = IDirectDrawSurface7_Release(surf);
-    while (ref3) ref3 = IDirectDrawSurface3_Release(&Impl->IDirectDrawSurface3_iface);
-    while (ref2) ref2 = IDirectDrawSurface2_Release(&Impl->IDirectDrawSurface2_iface);
+    iface_count = ddraw_surface_release_iface(Impl);
+    while (iface_count) iface_count = ddraw_surface_release_iface(Impl);
 
     return DDENUMRET_OK;
 }
