@@ -115,58 +115,67 @@ static BOOL check_hook_thread(void);
 static CRITICAL_SECTION dinput_hook_crit;
 static struct list direct_input_list = LIST_INIT( direct_input_list );
 
-/******************************************************************************
- *	DirectInputCreateEx (DINPUT.@)
- */
-HRESULT WINAPI DirectInputCreateEx(
-	HINSTANCE hinst, DWORD dwVersion, REFIID riid, LPVOID *ppDI,
-	LPUNKNOWN punkOuter) 
+static HRESULT initialize_directinput_instance(IDirectInputImpl *This, DWORD dwVersion);
+static void uninitialize_directinput_instance(IDirectInputImpl *This);
+
+static HRESULT create_directinput_instance(REFIID riid, LPVOID *ppDI, IDirectInputImpl **out)
 {
-    IDirectInputImpl* This;
+    IDirectInputImpl *This = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(IDirectInputImpl) );
+    HRESULT hr;
 
-    TRACE("(%p,%04x,%s,%p,%p)\n", hinst, dwVersion, debugstr_guid(riid), ppDI, punkOuter);
-
-    if (IsEqualGUID( &IID_IUnknown,       riid ) ||
-        IsEqualGUID( &IID_IDirectInputA,  riid ) ||
-        IsEqualGUID( &IID_IDirectInput2A, riid ) ||
-        IsEqualGUID( &IID_IDirectInput7A, riid ) ||
-        IsEqualGUID( &IID_IDirectInputW,  riid ) ||
-        IsEqualGUID( &IID_IDirectInput2W, riid ) ||
-        IsEqualGUID( &IID_IDirectInput7W, riid ) ||
-        IsEqualGUID( &IID_IDirectInput8A, riid ) ||
-        IsEqualGUID( &IID_IDirectInput8W, riid ))
-    {
-        if (!(This = HeapAlloc( GetProcessHeap(), 0, sizeof(IDirectInputImpl) )))
-            return DIERR_OUTOFMEMORY;
-    }
-    else
-        return DIERR_OLDDIRECTINPUTVERSION;
+    if (!This)
+        return E_OUTOFMEMORY;
 
     This->IDirectInput7A_iface.lpVtbl = &ddi7avt;
     This->IDirectInput7W_iface.lpVtbl = &ddi7wvt;
     This->IDirectInput8A_iface.lpVtbl = &ddi8avt;
     This->IDirectInput8W_iface.lpVtbl = &ddi8wvt;
-    This->ref         = 0;
-    This->dwVersion   = dwVersion;
-    This->evsequence  = 1;
 
-    InitializeCriticalSection(&This->crit);
-    This->crit.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": IDirectInputImpl*->crit");
-
-    list_init( &This->devices_list );
-
-    /* Add self to the list of the IDirectInputs */
-    EnterCriticalSection( &dinput_hook_crit );
-    list_add_head( &direct_input_list, &This->entry );
-    LeaveCriticalSection( &dinput_hook_crit );
-
-    if (!check_hook_thread())
+    hr = IDirectInput_QueryInterface( &This->IDirectInput7A_iface, riid, ppDI );
+    if (FAILED(hr))
     {
-        IUnknown_Release( &This->IDirectInput7A_iface );
-        return DIERR_GENERIC;
+        HeapFree( GetProcessHeap(), 0, This );
+        return hr;
+    }
+
+    if (out) *out = This;
+    return DI_OK;
+}
+
+/******************************************************************************
+ *	DirectInputCreateEx (DINPUT.@)
+ */
+HRESULT WINAPI DirectInputCreateEx(
+	HINSTANCE hinst, DWORD dwVersion, REFIID riid, LPVOID *ppDI,
+	LPUNKNOWN punkOuter)
+{
+    IDirectInputImpl *This;
+    HRESULT hr;
+
+    TRACE("(%p,%04x,%s,%p,%p)\n", hinst, dwVersion, debugstr_guid(riid), ppDI, punkOuter);
+
+    if (IsEqualGUID( &IID_IDirectInputA,  riid ) ||
+        IsEqualGUID( &IID_IDirectInput2A, riid ) ||
+        IsEqualGUID( &IID_IDirectInput7A, riid ) ||
+        IsEqualGUID( &IID_IDirectInputW,  riid ) ||
+        IsEqualGUID( &IID_IDirectInput2W, riid ) ||
+        IsEqualGUID( &IID_IDirectInput7W, riid ))
+    {
+        hr = create_directinput_instance(riid, ppDI, &This);
+        if (FAILED(hr))
+            return hr;
+    }
+    else
+        return DIERR_NOINTERFACE;
+
+    hr = IDirectInput_Initialize( &This->IDirectInput7A_iface, hinst, dwVersion );
+    if (FAILED(hr))
+    {
+        IDirectInput_Release( &This->IDirectInput7A_iface );
+        *ppDI = NULL;
+        return hr;
     }
 
-    IDirectInput_QueryInterface( &This->IDirectInput7A_iface, riid, ppDI );
     return DI_OK;
 }
 
@@ -224,7 +233,7 @@ static void _dump_EnumDevices_dwFlags(DWORD dwFlags) {
     TRACE("\n");
 }
 
-void _dump_diactionformatA(LPDIACTIONFORMATA lpdiActionFormat) {
+static void _dump_diactionformatA(LPDIACTIONFORMATA lpdiActionFormat) {
     unsigned int i;
 
     FIXME("diaf.dwSize = %d\n", lpdiActionFormat->dwSize);
@@ -254,6 +263,92 @@ void _dump_diactionformatA(LPDIACTIONFORMATA lpdiActionFormat) {
     }
 }
 
+void _copy_diactionformatAtoW(LPDIACTIONFORMATW to, LPDIACTIONFORMATA from)
+{
+    int i;
+
+    to->dwSize = sizeof(DIACTIONFORMATW);
+    to->dwActionSize = sizeof(DIACTIONW);
+    to->dwDataSize = from->dwDataSize;
+    to->dwNumActions = from->dwNumActions;
+    to->guidActionMap = from->guidActionMap;
+    to->dwGenre = from->dwGenre;
+    to->dwBufferSize = from->dwBufferSize;
+    to->lAxisMin = from->lAxisMin;
+    to->lAxisMax = from->lAxisMax;
+    to->dwCRC = from->dwCRC;
+    to->ftTimeStamp = from->ftTimeStamp;
+
+    for (i=0; i < to->dwNumActions; i++)
+    {
+        to->rgoAction[i].uAppData = from->rgoAction[i].uAppData;
+        to->rgoAction[i].dwSemantic = from->rgoAction[i].dwSemantic;
+        to->rgoAction[i].dwFlags = from->rgoAction[i].dwFlags;
+        to->rgoAction[i].guidInstance = from->rgoAction[i].guidInstance;
+        to->rgoAction[i].dwObjID = from->rgoAction[i].dwObjID;
+        to->rgoAction[i].dwHow = from->rgoAction[i].dwHow;
+    }
+}
+
+void _copy_diactionformatWtoA(LPDIACTIONFORMATA to, LPDIACTIONFORMATW from)
+{
+    int i;
+
+    to->dwSize = sizeof(DIACTIONFORMATA);
+    to->dwActionSize = sizeof(DIACTIONA);
+    to->dwDataSize = from->dwDataSize;
+    to->dwNumActions = from->dwNumActions;
+    to->guidActionMap = from->guidActionMap;
+    to->dwGenre = from->dwGenre;
+    to->dwBufferSize = from->dwBufferSize;
+    to->lAxisMin = from->lAxisMin;
+    to->lAxisMax = from->lAxisMax;
+    to->dwCRC = from->dwCRC;
+    to->ftTimeStamp = from->ftTimeStamp;
+
+    for (i=0; i < to->dwNumActions; i++)
+    {
+        to->rgoAction[i].uAppData = from->rgoAction[i].uAppData;
+        to->rgoAction[i].dwSemantic = from->rgoAction[i].dwSemantic;
+        to->rgoAction[i].dwFlags = from->rgoAction[i].dwFlags;
+        to->rgoAction[i].guidInstance = from->rgoAction[i].guidInstance;
+        to->rgoAction[i].dwObjID = from->rgoAction[i].dwObjID;
+        to->rgoAction[i].dwHow = from->rgoAction[i].dwHow;
+    }
+}
+
+/* _diactionformat_priority
+ *
+ *  Given a DIACTIONFORMAT structure and a DI genre, returns the enumeration
+ *  priority. Joysticks should pass the game genre, and mouse or keyboard their
+ *  respective DI*_MASK
+ */
+static DWORD _diactionformat_priorityA(LPDIACTIONFORMATA lpdiaf, DWORD genre)
+{
+    int i;
+    DWORD priorityFlags = 0;
+
+    /* If there's at least one action for the device it's priority 1 */
+    for(i=0; i < lpdiaf->dwActionSize; i++)
+        if ((lpdiaf->rgoAction[i].dwSemantic & genre) == genre)
+            priorityFlags |= DIEDBS_MAPPEDPRI1;
+
+    return priorityFlags;
+}
+
+static DWORD _diactionformat_priorityW(LPDIACTIONFORMATW lpdiaf, DWORD genre)
+{
+    int i;
+    DWORD priorityFlags = 0;
+
+    /* If there's at least one action for the device it's priority 1 */
+    for(i=0; i < lpdiaf->dwActionSize; i++)
+        if ((lpdiaf->rgoAction[i].dwSemantic & genre) == genre)
+            priorityFlags |= DIEDBS_MAPPEDPRI1;
+
+    return priorityFlags;
+}
+
 /******************************************************************************
  *	IDirectInputA_EnumDevices
  */
@@ -271,6 +366,12 @@ static HRESULT WINAPI IDirectInputAImpl_EnumDevices(
 	  lpCallback, pvRef, dwFlags);
     _dump_EnumDevices_dwFlags(dwFlags);
 
+    if (!lpCallback)
+        return DIERR_INVALIDPARAM;
+
+    if (!This->initialized)
+        return DIERR_NOTINITIALIZED;
+
     for (i = 0; i < NB_DINPUT_DEVICES; i++) {
         if (!dinput_devices[i]->enum_deviceA) continue;
         for (j = 0, r = -1; r != 0; j++) {
@@ -302,6 +403,12 @@ static HRESULT WINAPI IDirectInputWImpl_EnumDevices(
 	  lpCallback, pvRef, dwFlags);
     _dump_EnumDevices_dwFlags(dwFlags);
 
+    if (!lpCallback)
+        return DIERR_INVALIDPARAM;
+
+    if (!This->initialized)
+        return DIERR_NOTINITIALIZED;
+
     for (i = 0; i < NB_DINPUT_DEVICES; i++) {
         if (!dinput_devices[i]->enum_deviceW) continue;
         for (j = 0, r = -1; r != 0; j++) {
@@ -339,20 +446,13 @@ static ULONG WINAPI IDirectInputAImpl_Release(LPDIRECTINPUT7A iface)
 
     TRACE( "(%p) releasing from %d\n", This, ref + 1 );
 
-    if (ref) return ref;
+    if (ref == 0)
+    {
+        uninitialize_directinput_instance( This );
+        HeapFree( GetProcessHeap(), 0, This );
+    }
 
-    /* Remove self from the list of the IDirectInputs */
-    EnterCriticalSection( &dinput_hook_crit );
-    list_remove( &This->entry );
-    LeaveCriticalSection( &dinput_hook_crit );
-
-    check_hook_thread();
-
-    This->crit.DebugInfo->Spare[0] = 0;
-    DeleteCriticalSection( &This->crit );
-    HeapFree( GetProcessHeap(), 0, This );
-
-    return 0;
+    return ref;
 }
 
 static ULONG WINAPI IDirectInputWImpl_Release(LPDIRECTINPUT7W iface)
@@ -367,6 +467,9 @@ static HRESULT WINAPI IDirectInputAImpl_QueryInterface(LPDIRECTINPUT7A iface, RE
 
     TRACE( "(%p)->(%s,%p)\n", This, debugstr_guid(riid), ppobj );
 
+    if (!riid || !ppobj)
+        return E_POINTER;
+
     if (IsEqualGUID( &IID_IUnknown, riid ) ||
         IsEqualGUID( &IID_IDirectInputA,  riid ) ||
         IsEqualGUID( &IID_IDirectInput2A, riid ) ||
@@ -405,7 +508,8 @@ static HRESULT WINAPI IDirectInputAImpl_QueryInterface(LPDIRECTINPUT7A iface, RE
     }
 
     FIXME( "Unsupported interface: %s\n", debugstr_guid(riid));
-    return E_FAIL;
+    *ppobj = NULL;
+    return E_NOINTERFACE;
 }
 
 static HRESULT WINAPI IDirectInputWImpl_QueryInterface(LPDIRECTINPUT7W iface, REFIID riid, LPVOID *ppobj)
@@ -414,14 +518,83 @@ static HRESULT WINAPI IDirectInputWImpl_QueryInterface(LPDIRECTINPUT7W iface, RE
     return IDirectInputAImpl_QueryInterface( &This->IDirectInput7A_iface, riid, ppobj );
 }
 
-static HRESULT WINAPI IDirectInputAImpl_Initialize(LPDIRECTINPUT7A iface, HINSTANCE hinst, DWORD x) {
-	TRACE("(this=%p,%p,%x)\n",iface, hinst, x);
-	
-	/* Initialize can return: DIERR_BETADIRECTINPUTVERSION, DIERR_OLDDIRECTINPUTVERSION and DI_OK.
-	 * Since we already initialized the device, return DI_OK. In the past we returned DIERR_ALREADYINITIALIZED
-	 * which broke applications like Tomb Raider Legend because it isn't a legal return value.
-	 */
-	return DI_OK;
+static HRESULT initialize_directinput_instance(IDirectInputImpl *This, DWORD dwVersion)
+{
+    if (!This->initialized)
+    {
+        This->dwVersion = dwVersion;
+        This->evsequence = 1;
+
+        InitializeCriticalSection( &This->crit );
+        This->crit.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": IDirectInputImpl*->crit");
+
+        list_init( &This->devices_list );
+
+        /* Add self to the list of the IDirectInputs */
+        EnterCriticalSection( &dinput_hook_crit );
+        list_add_head( &direct_input_list, &This->entry );
+        LeaveCriticalSection( &dinput_hook_crit );
+
+        This->initialized = TRUE;
+
+        if (!check_hook_thread())
+        {
+            uninitialize_directinput_instance( This );
+            return DIERR_GENERIC;
+        }
+    }
+
+    return DI_OK;
+}
+
+static void uninitialize_directinput_instance(IDirectInputImpl *This)
+{
+    if (This->initialized)
+    {
+        /* Remove self from the list of the IDirectInputs */
+        EnterCriticalSection( &dinput_hook_crit );
+        list_remove( &This->entry );
+        LeaveCriticalSection( &dinput_hook_crit );
+
+        check_hook_thread();
+
+        This->crit.DebugInfo->Spare[0] = 0;
+        DeleteCriticalSection( &This->crit );
+
+        This->initialized = FALSE;
+    }
+}
+
+enum directinput_versions
+{
+    DIRECTINPUT_VERSION_300 = 0x0300,
+    DIRECTINPUT_VERSION_500 = 0x0500,
+    DIRECTINPUT_VERSION_50A = 0x050A,
+    DIRECTINPUT_VERSION_5B2 = 0x05B2,
+    DIRECTINPUT_VERSION_602 = 0x0602,
+    DIRECTINPUT_VERSION_61A = 0x061A,
+    DIRECTINPUT_VERSION_700 = 0x0700,
+};
+
+static HRESULT WINAPI IDirectInputAImpl_Initialize(LPDIRECTINPUT7A iface, HINSTANCE hinst, DWORD version)
+{
+    IDirectInputImpl *This = impl_from_IDirectInput7A( iface );
+
+    TRACE("(%p)->(%p, 0x%04x)\n", iface, hinst, version);
+
+    if (!hinst)
+        return DIERR_INVALIDPARAM;
+    else if (version == 0)
+        return DIERR_NOTINITIALIZED;
+    else if (version > DIRECTINPUT_VERSION_700)
+        return DIERR_OLDDIRECTINPUTVERSION;
+    else if (version != DIRECTINPUT_VERSION_300 && version != DIRECTINPUT_VERSION_500 &&
+             version != DIRECTINPUT_VERSION_50A && version != DIRECTINPUT_VERSION_5B2 &&
+             version != DIRECTINPUT_VERSION_602 && version != DIRECTINPUT_VERSION_61A &&
+             version != DIRECTINPUT_VERSION_700 && version != DIRECTINPUT_VERSION)
+        return DIERR_BETADIRECTINPUTVERSION;
+
+    return initialize_directinput_instance(This, version);
 }
 
 static HRESULT WINAPI IDirectInputWImpl_Initialize(LPDIRECTINPUT7W iface, HINSTANCE hinst, DWORD x)
@@ -438,6 +611,9 @@ static HRESULT WINAPI IDirectInputAImpl_GetDeviceStatus(LPDIRECTINPUT7A iface, R
 
     TRACE( "(%p)->(%s)\n", This, debugstr_guid(rguid) );
 
+    if (!This->initialized)
+        return DIERR_NOTINITIALIZED;
+
     hr = IDirectInput_CreateDevice( iface, rguid, &device, NULL );
     if (hr != DI_OK) return DI_NOTATTACHED;
 
@@ -456,9 +632,25 @@ static HRESULT WINAPI IDirectInputAImpl_RunControlPanel(LPDIRECTINPUT7A iface,
 							HWND hwndOwner,
 							DWORD dwFlags)
 {
+    WCHAR control_exeW[] = {'c','o','n','t','r','o','l','.','e','x','e',0};
+    STARTUPINFOW si = {0};
+    PROCESS_INFORMATION pi;
+
     IDirectInputImpl *This = impl_from_IDirectInput7A( iface );
 
-    FIXME( "(%p)->(%p,%08x): stub\n", This, hwndOwner, dwFlags );
+    TRACE( "(%p)->(%p, %08x)\n", This, hwndOwner, dwFlags );
+
+    if (hwndOwner && !IsWindow(hwndOwner))
+        return E_HANDLE;
+
+    if (dwFlags)
+        return DIERR_INVALIDPARAM;
+
+    if (!This->initialized)
+        return DIERR_NOTINITIALIZED;
+
+    if (!CreateProcessW(NULL, control_exeW, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
+        return HRESULT_FROM_WIN32(GetLastError());
 
     return DI_OK;
 }
@@ -489,61 +681,51 @@ static HRESULT WINAPI IDirectInput2WImpl_FindDevice(LPDIRECTINPUT7W iface, REFGU
     return DI_OK;
 }
 
-static HRESULT WINAPI IDirectInput7AImpl_CreateDeviceEx(LPDIRECTINPUT7A iface, REFGUID rguid,
-							REFIID riid, LPVOID* pvOut, LPUNKNOWN lpUnknownOuter)
+static HRESULT create_device(IDirectInputImpl *This, REFGUID rguid, REFIID riid, LPVOID *pvOut, BOOL unicode)
 {
-  IDirectInputImpl *This = impl_from_IDirectInput7A( iface );
-  HRESULT ret_value = DIERR_DEVICENOTREG;
-  unsigned int i;
+    unsigned int i;
 
-  TRACE("(%p)->(%s, %s, %p, %p)\n", This, debugstr_guid(rguid), debugstr_guid(riid), pvOut, lpUnknownOuter);
+    if (pvOut)
+        *pvOut = NULL;
 
-  if (!rguid || !pvOut) return E_POINTER;
+    if (!rguid || !pvOut)
+        return E_POINTER;
 
-  /* Loop on all the devices to see if anyone matches the given GUID */
-  for (i = 0; i < NB_DINPUT_DEVICES; i++) {
-    HRESULT ret;
+    if (!This->initialized)
+        return DIERR_NOTINITIALIZED;
 
-    if (!dinput_devices[i]->create_deviceA) continue;
-    if ((ret = dinput_devices[i]->create_deviceA(This, rguid, riid, (LPDIRECTINPUTDEVICEA*) pvOut)) == DI_OK)
-      return DI_OK;
+    /* Loop on all the devices to see if anyone matches the given GUID */
+    for (i = 0; i < NB_DINPUT_DEVICES; i++)
+    {
+        HRESULT ret;
 
-    if (ret == DIERR_NOINTERFACE)
-      ret_value = DIERR_NOINTERFACE;
-  }
+        if (!dinput_devices[i]->create_device) continue;
+        if ((ret = dinput_devices[i]->create_device(This, rguid, riid, pvOut, unicode)) == DI_OK)
+            return DI_OK;
+    }
 
-  if (ret_value == DIERR_NOINTERFACE)
-  {
     WARN("invalid device GUID %s\n", debugstr_guid(rguid));
-  }
+    return DIERR_DEVICENOTREG;
+}
 
-  return ret_value;
+static HRESULT WINAPI IDirectInput7AImpl_CreateDeviceEx(LPDIRECTINPUT7A iface, REFGUID rguid,
+                                                        REFIID riid, LPVOID* pvOut, LPUNKNOWN lpUnknownOuter)
+{
+    IDirectInputImpl *This = impl_from_IDirectInput7A( iface );
+
+    TRACE("(%p)->(%s, %s, %p, %p)\n", This, debugstr_guid(rguid), debugstr_guid(riid), pvOut, lpUnknownOuter);
+
+    return create_device(This, rguid, riid, pvOut, FALSE);
 }
 
 static HRESULT WINAPI IDirectInput7WImpl_CreateDeviceEx(LPDIRECTINPUT7W iface, REFGUID rguid,
-							REFIID riid, LPVOID* pvOut, LPUNKNOWN lpUnknownOuter)
+                                                        REFIID riid, LPVOID* pvOut, LPUNKNOWN lpUnknownOuter)
 {
-  IDirectInputImpl *This = impl_from_IDirectInput7W( iface );
-  HRESULT ret_value = DIERR_DEVICENOTREG;
-  unsigned int i;
+    IDirectInputImpl *This = impl_from_IDirectInput7W( iface );
 
-  TRACE("(%p)->(%s, %s, %p, %p)\n", This, debugstr_guid(rguid), debugstr_guid(riid), pvOut, lpUnknownOuter);
+    TRACE("(%p)->(%s, %s, %p, %p)\n", This, debugstr_guid(rguid), debugstr_guid(riid), pvOut, lpUnknownOuter);
 
-  if (!rguid || !pvOut) return E_POINTER;
-
-  /* Loop on all the devices to see if anyone matches the given GUID */
-  for (i = 0; i < NB_DINPUT_DEVICES; i++) {
-    HRESULT ret;
-
-    if (!dinput_devices[i]->create_deviceW) continue;
-    if ((ret = dinput_devices[i]->create_deviceW(This, rguid, riid, (LPDIRECTINPUTDEVICEW*) pvOut)) == DI_OK)
-      return DI_OK;
-
-    if (ret == DIERR_NOINTERFACE)
-      ret_value = DIERR_NOINTERFACE;
-  }
-
-  return ret_value;
+    return create_device(This, rguid, riid, pvOut, TRUE);
 }
 
 static HRESULT WINAPI IDirectInputAImpl_CreateDevice(LPDIRECTINPUT7A iface, REFGUID rguid,
@@ -650,16 +832,28 @@ static HRESULT WINAPI IDirectInput8WImpl_RunControlPanel(LPDIRECTINPUT8W iface,
     return IDirectInputAImpl_RunControlPanel( &This->IDirectInput7A_iface, hwndOwner, dwFlags );
 }
 
-static HRESULT WINAPI IDirectInput8AImpl_Initialize(LPDIRECTINPUT8A iface, HINSTANCE hinst, DWORD x)
+static HRESULT WINAPI IDirectInput8AImpl_Initialize(LPDIRECTINPUT8A iface, HINSTANCE hinst, DWORD version)
 {
     IDirectInputImpl *This = impl_from_IDirectInput8A( iface );
-    return IDirectInputAImpl_Initialize( &This->IDirectInput7A_iface, hinst, x );
+
+    TRACE("(%p)->(%p, 0x%04x)\n", iface, hinst, version);
+
+    if (!hinst)
+        return DIERR_INVALIDPARAM;
+    else if (version == 0)
+        return DIERR_NOTINITIALIZED;
+    else if (version < DIRECTINPUT_VERSION)
+        return DIERR_BETADIRECTINPUTVERSION;
+    else if (version > DIRECTINPUT_VERSION)
+        return DIERR_OLDDIRECTINPUTVERSION;
+
+    return initialize_directinput_instance(This, version);
 }
 
-static HRESULT WINAPI IDirectInput8WImpl_Initialize(LPDIRECTINPUT8W iface, HINSTANCE hinst, DWORD x)
+static HRESULT WINAPI IDirectInput8WImpl_Initialize(LPDIRECTINPUT8W iface, HINSTANCE hinst, DWORD version)
 {
     IDirectInputImpl *This = impl_from_IDirectInput8W( iface );
-    return IDirectInputAImpl_Initialize( &This->IDirectInput7A_iface, hinst, x );
+    return IDirectInput8AImpl_Initialize( &This->IDirectInput8A_iface, hinst, version );
 }
 
 static HRESULT WINAPI IDirectInput8AImpl_FindDevice(LPDIRECTINPUT8A iface, REFGUID rguid, LPCSTR pszName, LPGUID pguidInstance)
@@ -680,9 +874,16 @@ static HRESULT WINAPI IDirectInput8AImpl_EnumDevicesBySemantics(
       LPVOID pvRef, DWORD dwFlags
 )
 {
+    static REFGUID guids[2] = { &GUID_SysKeyboard, &GUID_SysMouse };
+    static const DWORD actionMasks[] = { DIKEYBOARD_MASK, DIMOUSE_MASK };
     IDirectInputImpl *This = impl_from_IDirectInput8A( iface );
+    DIDEVICEINSTANCEA didevi;
+    LPDIRECTINPUTDEVICE8A lpdid;
+    DWORD callbackFlags;
+    int i, j;
 
-    FIXME("(this=%p,%s,%p,%p,%p,%04x): stub\n", This, ptszUserName, lpdiActionFormat,
+
+    FIXME("(this=%p,%s,%p,%p,%p,%04x): semi-stub\n", This, ptszUserName, lpdiActionFormat,
           lpCallback, pvRef, dwFlags);
 #define X(x) if (dwFlags & x) FIXME("\tdwFlags |= "#x"\n");
 	X(DIEDBSFL_ATTACHEDONLY)
@@ -695,6 +896,46 @@ static HRESULT WINAPI IDirectInput8AImpl_EnumDevicesBySemantics(
 
     _dump_diactionformatA(lpdiActionFormat);
 
+    didevi.dwSize = sizeof(didevi);
+
+    /* Enumerate all the joysticks */
+    for (i = 0; i < NB_DINPUT_DEVICES; i++)
+    {
+        BOOL enumSuccess;
+
+        if (!dinput_devices[i]->enum_deviceA) continue;
+
+        for (j = 0, enumSuccess = -1; enumSuccess != 0; j++)
+        {
+            TRACE(" - checking device %u ('%s')\n", i, dinput_devices[i]->name);
+
+            callbackFlags = _diactionformat_priorityA(lpdiActionFormat, lpdiActionFormat->dwGenre);
+            /* Default behavior is to enumerate attached game controllers */
+            enumSuccess = dinput_devices[i]->enum_deviceA(DI8DEVCLASS_GAMECTRL, DIEDFL_ATTACHEDONLY | dwFlags, &didevi, This->dwVersion, j);
+            if (enumSuccess)
+            {
+                IDirectInput_CreateDevice(iface, &didevi.guidInstance, &lpdid, NULL);
+
+                if (lpCallback(&didevi, lpdid, callbackFlags, 0, pvRef) == DIENUM_STOP)
+                    return DI_OK;
+            }
+        }
+    }
+
+    if (dwFlags & DIEDBSFL_FORCEFEEDBACK) return DI_OK;
+
+    /* Enumerate keyboard and mouse */
+    for(i=0; i < sizeof(guids)/sizeof(guids[0]); i++)
+    {
+        callbackFlags = _diactionformat_priorityA(lpdiActionFormat, actionMasks[i]);
+
+        IDirectInput_CreateDevice(iface, guids[i], &lpdid, NULL);
+        IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
+
+        if (lpCallback(&didevi, lpdid, callbackFlags, sizeof(guids)/sizeof(guids[0]) - (i+1), pvRef) == DIENUM_STOP)
+            return DI_OK;
+    }
+
     return DI_OK;
 }
 
@@ -704,11 +945,58 @@ static HRESULT WINAPI IDirectInput8WImpl_EnumDevicesBySemantics(
       LPVOID pvRef, DWORD dwFlags
 )
 {
-      IDirectInputImpl *This = impl_from_IDirectInput8W( iface );
+    static REFGUID guids[2] = { &GUID_SysKeyboard, &GUID_SysMouse };
+    static const DWORD actionMasks[] = { DIKEYBOARD_MASK, DIMOUSE_MASK };
+    IDirectInputImpl *This = impl_from_IDirectInput8W(iface);
+    DIDEVICEINSTANCEW didevi;
+    LPDIRECTINPUTDEVICE8W lpdid;
+    DWORD callbackFlags;
+    int i, j;
 
-      FIXME("(this=%p,%s,%p,%p,%p,%04x): stub\n", This, debugstr_w(ptszUserName), lpdiActionFormat,
-            lpCallback, pvRef, dwFlags);
-      return 0;
+    FIXME("(this=%p,%s,%p,%p,%p,%04x): semi-stub\n", This, debugstr_w(ptszUserName), lpdiActionFormat,
+          lpCallback, pvRef, dwFlags);
+
+    didevi.dwSize = sizeof(didevi);
+
+    /* Enumerate all the joysticks */
+    for (i = 0; i < NB_DINPUT_DEVICES; i++)
+    {
+        BOOL enumSuccess;
+
+        if (!dinput_devices[i]->enum_deviceW) continue;
+
+        for (j = 0, enumSuccess = -1; enumSuccess != 0; j++)
+        {
+            TRACE(" - checking device %u ('%s')\n", i, dinput_devices[i]->name);
+
+            callbackFlags = _diactionformat_priorityW(lpdiActionFormat, lpdiActionFormat->dwGenre);
+            /* Default behavior is to enumerate attached game controllers */
+            enumSuccess = dinput_devices[i]->enum_deviceW(DI8DEVCLASS_GAMECTRL, DIEDFL_ATTACHEDONLY | dwFlags, &didevi, This->dwVersion, j);
+            if (enumSuccess)
+            {
+                IDirectInput_CreateDevice(iface, &didevi.guidInstance, &lpdid, NULL);
+
+                if (lpCallback(&didevi, lpdid, callbackFlags, 0, pvRef) == DIENUM_STOP)
+                    return DI_OK;
+            }
+        }
+    }
+
+    if (dwFlags & DIEDBSFL_FORCEFEEDBACK) return DI_OK;
+
+    /* Enumerate keyboard and mouse */
+    for(i=0; i < sizeof(guids)/sizeof(guids[0]); i++)
+    {
+        callbackFlags = _diactionformat_priorityW(lpdiActionFormat, actionMasks[i]);
+
+        IDirectInput_CreateDevice(iface, guids[i], &lpdid, NULL);
+        IDirectInputDevice_GetDeviceInfo(lpdid, &didevi);
+
+        if (lpCallback(&didevi, lpdid, callbackFlags, sizeof(guids)/sizeof(guids[0]) - (i+1), pvRef) == DIENUM_STOP)
+            return DI_OK;
+    }
+
+    return DI_OK;
 }
 
 static HRESULT WINAPI IDirectInput8AImpl_ConfigureDevices(
@@ -834,11 +1122,8 @@ static HRESULT WINAPI DICF_CreateInstance(
 	     IsEqualGUID( &IID_IDirectInput2A, riid ) ||
 	     IsEqualGUID( &IID_IDirectInput2W, riid ) ||
 	     IsEqualGUID( &IID_IDirectInput7A, riid ) ||
-	     IsEqualGUID( &IID_IDirectInput7W, riid ) ||
-	     IsEqualGUID( &IID_IDirectInput8A, riid ) ||
-	     IsEqualGUID( &IID_IDirectInput8W, riid ) ) {
-		/* FIXME: reuse already created dinput if present? */
-		return DirectInputCreateEx(0,0,riid,ppobj,pOuter);
+	     IsEqualGUID( &IID_IDirectInput7W, riid ) ) {
+		return create_directinput_instance(riid, ppobj, NULL);
 	}
 
 	FIXME("(%p,%p,%s,%p) Interface not found!\n",This,pOuter,debugstr_guid(riid),ppobj);	
@@ -889,7 +1174,7 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
  */
 HRESULT WINAPI DllRegisterServer(void)
 {
-    return __wine_register_resources( DINPUT_instance, NULL );
+    return __wine_register_resources( DINPUT_instance );
 }
 
 /***********************************************************************
@@ -897,7 +1182,7 @@ HRESULT WINAPI DllRegisterServer(void)
  */
 HRESULT WINAPI DllUnregisterServer(void)
 {
-    return __wine_unregister_resources( DINPUT_instance, NULL );
+    return __wine_unregister_resources( DINPUT_instance );
 }
 
 /******************************************************************************
