@@ -620,7 +620,7 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetCurrentPosition(
 	This->sec_mixpos = newpos;
 
 	/* at this point, do not attempt to reset buffers, mess with primary mix position,
-           or anything like that to reduce latancy. The data already prebuffered cannot be changed */
+           or anything like that to reduce latency. The data already prebuffered cannot be changed */
 
 	/* position HW buffer if applicable, else just start mixing from new location instead */
 	if (This->hwbuf) {
@@ -791,13 +791,13 @@ static HRESULT WINAPI IDirectSoundBufferImpl_AcquireResources(
 	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	DWORD u;
 
-	FIXME("(%p,%08u,%u,%p): stub\n",This,dwFlags,dwEffectsCount,pdwResultCodes);
+	FIXME("(%p,%08u,%u,%p): stub, faking success\n",This,dwFlags,dwEffectsCount,pdwResultCodes);
 
 	if (pdwResultCodes)
 		for (u=0; u<dwEffectsCount; u++) pdwResultCodes[u] = DSFXR_UNKNOWN;
 
 	WARN("control unavailable\n");
-	return DSERR_CONTROLUNAVAIL;
+	return DS_OK;
 }
 
 static HRESULT WINAPI IDirectSoundBufferImpl_GetObjectInPath(
@@ -1177,7 +1177,7 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
 {
     IDirectSoundBufferImpl *dsb;
     HRESULT hres = DS_OK;
-    TRACE("(%p,%p,%p)\n", device, pdsb, pdsb);
+    TRACE("(%p,%p,%p)\n", device, ppdsb, pdsb);
 
     dsb = HeapAlloc(GetProcessHeap(),0,sizeof(*dsb));
     if (dsb == NULL) {
@@ -1213,6 +1213,9 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
     dsb->ref = 0;
     dsb->state = STATE_STOPPED;
     dsb->buf_mixpos = dsb->sec_mixpos = 0;
+    dsb->notify = NULL;
+    dsb->notifies = NULL;
+    dsb->nrofnotifies = 0;
     dsb->device = device;
     dsb->ds3db = NULL;
     dsb->iks = NULL; /* FIXME? */
