@@ -635,7 +635,7 @@ static HRESULT GAMEUX_loadGameStatistics(struct GAMEUX_STATS *pStats,
     }
     else if(hr == S_FALSE && openType == GAMESTATS_OPEN_OPENORCREATE) /* file does not exist */
     {
-        /* create new statitics, not yet connected with file */
+        /* create new statistics, not yet connected with file */
         ZeroMemory(pStats->categories, sizeof(pStats->categories));
         *pOpenResult = GAMESTATS_OPEN_CREATED;
         hr = S_OK;
@@ -819,12 +819,13 @@ static HRESULT WINAPI GameStatisticsImpl_GetCategoryTitle(
 
     TRACE("%p, %d, %p\n", This, categoryIndex, pTitle);
 
+    if(!pTitle)
+        return E_INVALIDARG;
     *pTitle = NULL;
 
-    if(!pTitle || categoryIndex >= MAX_CATEGORIES)
+    if (categoryIndex >= MAX_CATEGORIES)
         hr = E_INVALIDARG;
 
-
     if(SUCCEEDED(hr))
     {
         nLength = lstrlenW(This->stats.categories[categoryIndex].sName);
