@@ -50,81 +50,145 @@ static const struct gdi_obj_funcs bitmap_funcs =
  *           null driver fallback implementations
  */
 
-LONG CDECL nulldrv_GetBitmapBits( HBITMAP bitmap, void *bits, LONG size )
+DWORD nulldrv_GetImage( PHYSDEV dev, HBITMAP hbitmap, BITMAPINFO *info,
+                        struct gdi_image_bits *bits, struct bitblt_coords *src )
 {
-    BITMAPOBJ *bmp = GDI_GetObjPtr( bitmap, OBJ_BITMAP );
+    BITMAPOBJ *bmp;
+    int height, width_bytes;
 
-    if (bmp->bitmap.bmBits) memcpy( bits, bmp->bitmap.bmBits, size );
-    else memset( bits, 0, size );
-    GDI_ReleaseObj( bitmap );
-    return size;
-}
+    if (!hbitmap) return ERROR_NOT_SUPPORTED;
+    if (!(bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return ERROR_INVALID_HANDLE;
 
-LONG CDECL nulldrv_SetBitmapBits( HBITMAP bitmap, const void *bits, LONG size )
-{
-    BITMAPOBJ *bmp = GDI_GetObjPtr( bitmap, OBJ_BITMAP );
-
-    if (!bmp->bitmap.bmBits)
+    info->bmiHeader.biSize          = sizeof(info->bmiHeader);
+    info->bmiHeader.biPlanes        = 1;
+    info->bmiHeader.biBitCount      = bmp->bitmap.bmBitsPixel;
+    info->bmiHeader.biCompression   = BI_RGB;
+    info->bmiHeader.biXPelsPerMeter = 0;
+    info->bmiHeader.biYPelsPerMeter = 0;
+    info->bmiHeader.biClrUsed       = 0;
+    info->bmiHeader.biClrImportant  = 0;
+    if (bmp->bitmap.bmBitsPixel == 16)
     {
-        LONG total = bmp->bitmap.bmHeight * bmp->bitmap.bmWidthBytes;  /* alloc enough for entire bitmap */
-        if (!(bmp->bitmap.bmBits = HeapAlloc( GetProcessHeap(), 0, total )))
+        DWORD *masks = (DWORD *)info->bmiColors;
+        masks[0] = 0x7c00;
+        masks[1] = 0x03e0;
+        masks[2] = 0x001f;
+        info->bmiHeader.biCompression = BI_BITFIELDS;
+    }
+    if (!bits) goto done;
+
+    height = src->visrect.bottom - src->visrect.top;
+    width_bytes = get_dib_stride( bmp->bitmap.bmWidth, bmp->bitmap.bmBitsPixel );
+    info->bmiHeader.biWidth     = bmp->bitmap.bmWidth;
+    info->bmiHeader.biHeight    = -height;
+    info->bmiHeader.biSizeImage = height * width_bytes;
+
+    /* make the source rectangle relative to the returned bits */
+    src->y -= src->visrect.top;
+    offset_rect( &src->visrect, 0, -src->visrect.top );
+
+    if (bmp->bitmap.bmBits && bmp->bitmap.bmWidthBytes == width_bytes)
+    {
+        bits->ptr = (char *)bmp->bitmap.bmBits + src->visrect.top * width_bytes;
+        bits->is_copy = FALSE;
+        bits->free = NULL;
+    }
+    else
+    {
+        bits->ptr = HeapAlloc( GetProcessHeap(), 0, info->bmiHeader.biSizeImage );
+        bits->is_copy = TRUE;
+        bits->free = free_heap_bits;
+        if (bmp->bitmap.bmBits)
         {
-            GDI_ReleaseObj( bitmap );
-            return 0;
+            /* fixup the line alignment */
+            char *src = bmp->bitmap.bmBits, *dst = bits->ptr;
+            for ( ; height > 0; height--, src += bmp->bitmap.bmWidthBytes, dst += width_bytes)
+            {
+                memcpy( dst, src, bmp->bitmap.bmWidthBytes );
+                memset( dst + bmp->bitmap.bmWidthBytes, 0, width_bytes - bmp->bitmap.bmWidthBytes );
+            }
         }
-        if (size < total) memset( (char *)bmp->bitmap.bmBits + size, 0, total - size );
+        else memset( bits->ptr, 0, info->bmiHeader.biSizeImage );
     }
-    memcpy( bmp->bitmap.bmBits, bits, size );
-    GDI_ReleaseObj( bitmap );
-    return size;
+done:
+    GDI_ReleaseObj( hbitmap );
+    return ERROR_SUCCESS;
 }
 
-INT CDECL nulldrv_GetDIBits( PHYSDEV dev, HBITMAP bitmap, UINT start, UINT lines, LPVOID bits,
-                             BITMAPINFO *info, UINT coloruse )
+DWORD nulldrv_PutImage( PHYSDEV dev, HBITMAP hbitmap, HRGN clip, BITMAPINFO *info,
+                        const struct gdi_image_bits *bits, struct bitblt_coords *src,
+                        struct bitblt_coords *dst, DWORD rop )
 {
-    /* FIXME: transfer bits from bmp->bitmap.bmBits */
-    return 0;
-}
+    BITMAPOBJ *bmp;
 
-INT CDECL nulldrv_SetDIBits( PHYSDEV dev, HBITMAP bitmap, UINT start, UINT lines,
-                             const void *bits, const BITMAPINFO *info, UINT coloruse )
-{
-    /* FIXME: transfer bits to bmp->bitmap.bmBits */
-    return 0;
-}
+    if (!hbitmap) return ERROR_SUCCESS;
+    if (!(bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return ERROR_INVALID_HANDLE;
 
-/***********************************************************************
- *           BITMAP_GetWidthBytes
- *
- * Return number of bytes taken by a scanline of 16-bit aligned Windows DDB
- * data.
- */
-INT BITMAP_GetWidthBytes( INT bmWidth, INT bpp )
-{
-    switch(bpp)
+    if (info->bmiHeader.biPlanes != 1) goto update_format;
+    if (info->bmiHeader.biBitCount != bmp->bitmap.bmBitsPixel) goto update_format;
+    /* FIXME: check color masks */
+
+    if (bits)
     {
-    case 1:
-	return 2 * ((bmWidth+15) >> 4);
+        int i, width_bytes = get_dib_stride( info->bmiHeader.biWidth, info->bmiHeader.biBitCount );
+        unsigned char *dst_bits, *src_bits;
 
-    case 24:
-	bmWidth *= 3; /* fall through */
-    case 8:
-	return bmWidth + (bmWidth & 1);
+        if ((src->width != dst->width) || (src->height != dst->height))
+        {
+            GDI_ReleaseObj( hbitmap );
+            return ERROR_TRANSFORM_NOT_SUPPORTED;
+        }
+        if (src->visrect.left > 0 || src->visrect.right < bmp->bitmap.bmWidth ||
+            dst->visrect.left > 0 || dst->visrect.right < bmp->bitmap.bmWidth)
+        {
+            FIXME( "setting partial rows not supported\n" );
+            GDI_ReleaseObj( hbitmap );
+            return ERROR_NOT_SUPPORTED;
+        }
+        if (clip)
+        {
+            FIXME( "clip region not supported\n" );
+            GDI_ReleaseObj( hbitmap );
+            return ERROR_NOT_SUPPORTED;
+        }
 
-    case 32:
-	return bmWidth * 4;
+        if (!bmp->bitmap.bmBits &&
+            !(bmp->bitmap.bmBits = HeapAlloc( GetProcessHeap(), 0,
+                                              bmp->bitmap.bmHeight * bmp->bitmap.bmWidthBytes )))
+        {
+            GDI_ReleaseObj( hbitmap );
+            return ERROR_OUTOFMEMORY;
+        }
+        src_bits = bits->ptr;
+        if (info->bmiHeader.biHeight > 0)
+        {
+            src_bits += (info->bmiHeader.biHeight - 1 - src->visrect.top) * width_bytes;
+            width_bytes = -width_bytes;
+        }
+        else
+            src_bits += src->visrect.top * width_bytes;
 
-    case 16:
-    case 15:
-	return bmWidth * 2;
-
-    case 4:
-	return 2 * ((bmWidth+3) >> 2);
-
-    default:
-	WARN("Unknown depth %d, please report.\n", bpp );
+        dst_bits = (unsigned char *)bmp->bitmap.bmBits + dst->visrect.top * bmp->bitmap.bmWidthBytes;
+        if (width_bytes != bmp->bitmap.bmWidthBytes)
+        {
+            for (i = 0; i < dst->visrect.bottom - dst->visrect.top; i++)
+            {
+                memcpy( dst_bits, src_bits, min( abs(width_bytes), bmp->bitmap.bmWidthBytes ));
+                src_bits += width_bytes;
+                dst_bits += bmp->bitmap.bmWidthBytes;
+            }
+        }
+        else memcpy( dst_bits, src_bits, width_bytes * (dst->visrect.bottom - dst->visrect.top) );
     }
-    return -1;
+    GDI_ReleaseObj( hbitmap );
+    return ERROR_SUCCESS;
+
+update_format:
+    info->bmiHeader.biPlanes   = 1;
+    info->bmiHeader.biBitCount = bmp->bitmap.bmBitsPixel;
+    if (info->bmiHeader.biHeight > 0) info->bmiHeader.biHeight = -info->bmiHeader.biHeight;
+    GDI_ReleaseObj( hbitmap );
+    return ERROR_BAD_FORMAT;
 }
 
 
@@ -152,7 +216,7 @@ HBITMAP WINAPI CreateBitmap( INT width, INT height, UINT planes,
     bm.bmType = 0;
     bm.bmWidth = width;
     bm.bmHeight = height;
-    bm.bmWidthBytes = BITMAP_GetWidthBytes( width, bpp );
+    bm.bmWidthBytes = get_bitmap_stride( width, bpp );
     bm.bmPlanes = planes;
     bm.bmBitsPixel = bpp;
     bm.bmBits = (LPVOID)bits;
@@ -319,7 +383,7 @@ HBITMAP WINAPI CreateBitmapIndirect( const BITMAP *bmp )
     }
 
     /* Windows ignores the provided bm.bmWidthBytes */
-    bm.bmWidthBytes = BITMAP_GetWidthBytes( bm.bmWidth, bm.bmBitsPixel );
+    bm.bmWidthBytes = get_bitmap_stride( bm.bmWidth, bm.bmBitsPixel );
     /* XP doesn't allow to create bitmaps larger than 128 Mb */
     if (bm.bmHeight > 128 * 1024 * 1024 / bm.bmWidthBytes)
     {
@@ -359,6 +423,26 @@ HBITMAP WINAPI CreateBitmapIndirect( const BITMAP *bmp )
 }
 
 
+/* convenience wrapper for GetImage to retrieve the full contents of a bitmap */
+BOOL get_bitmap_image( HBITMAP hbitmap, BITMAPINFO *info, struct gdi_image_bits *bits )
+{
+    struct bitblt_coords src;
+    BOOL ret = FALSE;
+    BITMAPOBJ *bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP );
+
+    if (bmp)
+    {
+        src.visrect.left   = src.x = 0;
+        src.visrect.top    = src.y = 0;
+        src.visrect.right  = src.width = bmp->bitmap.bmWidth;
+        src.visrect.bottom = src.height = bmp->bitmap.bmHeight;
+        ret = !bmp->funcs->pGetImage( NULL, hbitmap, info, bits, &src );
+        GDI_ReleaseObj( hbitmap );
+    }
+    return ret;
+}
+
+
 /***********************************************************************
  * GetBitmapBits [GDI32.@]
  *
@@ -373,83 +457,58 @@ LONG WINAPI GetBitmapBits(
     LONG count,        /* [in]  Number of bytes to copy */
     LPVOID bits)       /* [out] Pointer to buffer to receive bits */
 {
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *info = (BITMAPINFO *)buffer;
+    struct gdi_image_bits src_bits;
+    struct bitblt_coords src;
+    int dst_stride, max, ret;
     BITMAPOBJ *bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP );
-    LONG height, ret;
 
     if (!bmp) return 0;
 
-    if (bmp->dib)  /* simply copy the bits from the DIB */
+    if (bmp->dib) dst_stride = get_bitmap_stride( bmp->dib->dsBmih.biWidth, bmp->dib->dsBmih.biBitCount );
+    else dst_stride = get_bitmap_stride( bmp->bitmap.bmWidth, bmp->bitmap.bmBitsPixel );
+
+    ret = max = dst_stride * bmp->bitmap.bmHeight;
+    if (!bits) goto done;
+    if (count > max) count = max;
+    ret = count;
+
+    src.visrect.left = 0;
+    src.visrect.right = bmp->bitmap.bmWidth;
+    src.visrect.top = 0;
+    src.visrect.bottom = (count + dst_stride - 1) / dst_stride;
+    src.x = src.y = 0;
+    src.width = src.visrect.right - src.visrect.left;
+    src.height = src.visrect.bottom - src.visrect.top;
+
+    if (!bmp->funcs->pGetImage( NULL, hbitmap, info, &src_bits, &src ))
     {
-        DIBSECTION *dib = bmp->dib;
-        const char *src = dib->dsBm.bmBits;
-        INT width_bytes = BITMAP_GetWidthBytes(dib->dsBm.bmWidth, dib->dsBm.bmBitsPixel);
-        LONG max = width_bytes * bmp->bitmap.bmHeight;
+        const char *src_ptr = src_bits.ptr;
+        int src_stride = get_dib_stride( info->bmiHeader.biWidth, info->bmiHeader.biBitCount );
 
-        if (!bits)
+        /* GetBitmapBits returns 16-bit aligned data */
+
+        if (info->bmiHeader.biHeight > 0)
         {
-            ret = max;
-            goto done;
+            src_ptr += (info->bmiHeader.biHeight - 1) * src_stride;
+            src_stride = -src_stride;
         }
+        src_ptr += src.visrect.top * src_stride;
 
-        if (count > max) count = max;
-        ret = count;
-
-        /* GetBitmapBits returns not 32-bit aligned data */
-
-        if (bmp->dib->dsBmih.biHeight >= 0)  /* not top-down, need to flip contents vertically */
+        if (src_stride == dst_stride) memcpy( bits, src_ptr, count );
+        else while (count > 0)
         {
-            src += dib->dsBm.bmWidthBytes * dib->dsBm.bmHeight;
-            while (count > 0)
-            {
-                src -= dib->dsBm.bmWidthBytes;
-                memcpy( bits, src, min( count, width_bytes ) );
-                bits = (char *)bits + width_bytes;
-                count -= width_bytes;
-            }
+            memcpy( bits, src_ptr, min( count, dst_stride ) );
+            src_ptr += src_stride;
+            bits = (char *)bits + dst_stride;
+            count -= dst_stride;
         }
-        else
-        {
-            while (count > 0)
-            {
-                memcpy( bits, src, min( count, width_bytes ) );
-                src += dib->dsBm.bmWidthBytes;
-                bits = (char *)bits + width_bytes;
-                count -= width_bytes;
-            }
-        }
-        goto done;
+        if (src_bits.free) src_bits.free( &src_bits );
     }
+    else ret = 0;
 
-    /* If the bits vector is null, the function should return the read size */
-    if(bits == NULL)
-    {
-        ret = bmp->bitmap.bmWidthBytes * bmp->bitmap.bmHeight;
-        goto done;
-    }
-
-    if (count < 0) {
-	WARN("(%d): Negative number of bytes passed???\n", count );
-	count = -count;
-    }
-
-    /* Only get entire lines */
-    height = count / bmp->bitmap.bmWidthBytes;
-    if (height > bmp->bitmap.bmHeight) height = bmp->bitmap.bmHeight;
-    count = height * bmp->bitmap.bmWidthBytes;
-    if (count == 0)
-      {
-	WARN("Less than one entire line requested\n");
-        ret = 0;
-        goto done;
-      }
-
-
-    TRACE("(%p, %d, %p) %dx%d %d colors fetched height: %d\n",
-          hbitmap, count, bits, bmp->bitmap.bmWidth, bmp->bitmap.bmHeight,
-          1 << bmp->bitmap.bmBitsPixel, height );
-
-    ret = bmp->funcs->pGetBitmapBits( hbitmap, bits, count );
- done:
+done:
     GDI_ReleaseObj( hbitmap );
     return ret;
 }
@@ -469,8 +528,13 @@ LONG WINAPI SetBitmapBits(
     LONG count,        /* [in] Number of bytes in bitmap array */
     LPCVOID bits)      /* [in] Address of array with bitmap bits */
 {
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *info = (BITMAPINFO *)buffer;
     BITMAPOBJ *bmp;
-    LONG height, ret;
+    DWORD err;
+    int i, src_stride, dst_stride;
+    struct bitblt_coords src, dst;
+    struct gdi_image_bits src_bits;
 
     if (!bits) return 0;
 
@@ -482,43 +546,71 @@ LONG WINAPI SetBitmapBits(
 	count = -count;
     }
 
-    if (bmp->dib)  /* simply copy the bits into the DIB */
+    if (bmp->dib) src_stride = get_bitmap_stride( bmp->dib->dsBmih.biWidth, bmp->dib->dsBmih.biBitCount );
+    else src_stride = get_bitmap_stride( bmp->bitmap.bmWidth, bmp->bitmap.bmBitsPixel );
+
+    dst_stride = get_dib_stride( bmp->bitmap.bmWidth, bmp->bitmap.bmBitsPixel );
+
+    src.visrect.left   = src.x = 0;
+    src.visrect.top    = src.y = 0;
+    src.visrect.right  = src.width = bmp->bitmap.bmWidth;
+    src.visrect.bottom = src.height = min( count / src_stride, bmp->bitmap.bmHeight );
+    dst = src;
+
+    /* Only set entire lines */
+    count = src.height * src_stride;
+
+    TRACE("(%p, %d, %p) %dx%d %d bpp fetched height: %d\n",
+          hbitmap, count, bits, bmp->bitmap.bmWidth, bmp->bitmap.bmHeight,
+          bmp->bitmap.bmBitsPixel, src.height );
+
+    if (src_stride == dst_stride)
     {
-        DIBSECTION *dib = bmp->dib;
-        char *dest = dib->dsBm.bmBits;
-        LONG max = dib->dsBm.bmWidthBytes * dib->dsBm.bmHeight;
-        if (count > max) count = max;
-        ret = count;
-
-        if (bmp->dib->dsBmih.biHeight >= 0)  /* not top-down, need to flip contents vertically */
+        src_bits.ptr = (void *)bits;
+        src_bits.is_copy = FALSE;
+        src_bits.free = NULL;
+    }
+    else
+    {
+        if (!(src_bits.ptr = HeapAlloc( GetProcessHeap(), 0, dst.height * dst_stride )))
         {
-            dest += dib->dsBm.bmWidthBytes * dib->dsBm.bmHeight;
-            while (count > 0)
-            {
-                dest -= dib->dsBm.bmWidthBytes;
-                memcpy( dest, bits, min( count, dib->dsBm.bmWidthBytes ) );
-                bits = (const char *)bits + dib->dsBm.bmWidthBytes;
-                count -= dib->dsBm.bmWidthBytes;
-            }
+            GDI_ReleaseObj( hbitmap );
+            return 0;
         }
-        else memcpy( dest, bits, count );
-
-        GDI_ReleaseObj( hbitmap );
-        return ret;
+        src_bits.is_copy = TRUE;
+        src_bits.free = free_heap_bits;
+        for (i = 0; i < dst.height; i++)
+            memcpy( (char *)src_bits.ptr + i * dst_stride, (char *)bits + i * src_stride, src_stride );
     }
 
-    /* Only get entire lines */
-    height = count / bmp->bitmap.bmWidthBytes;
-    if (height > bmp->bitmap.bmHeight) height = bmp->bitmap.bmHeight;
-    count = height * bmp->bitmap.bmWidthBytes;
+    /* query the color info */
+    info->bmiHeader.biSize          = sizeof(info->bmiHeader);
+    info->bmiHeader.biPlanes        = 1;
+    info->bmiHeader.biBitCount      = bmp->bitmap.bmBitsPixel;
+    info->bmiHeader.biCompression   = BI_RGB;
+    info->bmiHeader.biXPelsPerMeter = 0;
+    info->bmiHeader.biYPelsPerMeter = 0;
+    info->bmiHeader.biClrUsed       = 0;
+    info->bmiHeader.biClrImportant  = 0;
+    info->bmiHeader.biWidth         = 0;
+    info->bmiHeader.biHeight        = 0;
+    info->bmiHeader.biSizeImage     = 0;
+    err = bmp->funcs->pPutImage( NULL, hbitmap, 0, info, NULL, NULL, NULL, SRCCOPY );
 
-    TRACE("(%p, %d, %p) %dx%d %d colors fetched height: %d\n",
-          hbitmap, count, bits, bmp->bitmap.bmWidth, bmp->bitmap.bmHeight,
-          1 << bmp->bitmap.bmBitsPixel, height );
+    if (!err || err == ERROR_BAD_FORMAT)
+    {
+        info->bmiHeader.biPlanes        = 1;
+        info->bmiHeader.biBitCount      = bmp->bitmap.bmBitsPixel;
+        info->bmiHeader.biWidth         = bmp->bitmap.bmWidth;
+        info->bmiHeader.biHeight        = -dst.height;
+        info->bmiHeader.biSizeImage     = dst.height * dst_stride;
+        err = bmp->funcs->pPutImage( NULL, hbitmap, 0, info, &src_bits, &src, &dst, SRCCOPY );
+    }
+    if (err) count = 0;
 
-    ret = bmp->funcs->pSetBitmapBits( hbitmap, bits, count );
+    if (src_bits.free) src_bits.free( &src_bits );
     GDI_ReleaseObj( hbitmap );
-    return ret;
+    return count;
 }
 
 /**********************************************************************
@@ -575,6 +667,12 @@ HBITMAP BITMAP_CopyBitmap(HBITMAP hbitmap)
 }
 
 
+static void set_initial_bitmap_bits( HBITMAP hbitmap, BITMAPOBJ *bmp )
+{
+    if (!bmp->bitmap.bmBits) return;
+    SetBitmapBits( hbitmap, bmp->bitmap.bmHeight * bmp->bitmap.bmWidthBytes, bmp->bitmap.bmBits );
+}
+
 /***********************************************************************
  *           BITMAP_SetOwnerDC
  *
@@ -592,15 +690,19 @@ BOOL BITMAP_SetOwnerDC( HBITMAP hbitmap, PHYSDEV physdev )
 
     if (!(bitmap = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return FALSE;
 
-    if (bitmap->funcs != physdev->funcs)
+    if (!bitmap->dib && bitmap->funcs != physdev->funcs)
     {
         /* we can only change from the null driver to some other driver */
         if (bitmap->funcs == &null_driver)
         {
             if (physdev->funcs->pCreateBitmap)
             {
-                ret = physdev->funcs->pCreateBitmap( physdev, hbitmap, bitmap->bitmap.bmBits );
-                if (ret) bitmap->funcs = physdev->funcs;
+                ret = physdev->funcs->pCreateBitmap( physdev, hbitmap );
+                if (ret)
+                {
+                    bitmap->funcs = physdev->funcs;
+                    set_initial_bitmap_bits( hbitmap, bitmap );
+                }
             }
             else
             {
@@ -627,7 +729,7 @@ static HGDIOBJ BITMAP_SelectObject( HGDIOBJ handle, HDC hdc )
     HGDIOBJ ret;
     BITMAPOBJ *bitmap;
     DC *dc;
-    PHYSDEV physdev;
+    PHYSDEV physdev = NULL, old_physdev = NULL;
 
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
@@ -653,7 +755,17 @@ static HGDIOBJ BITMAP_SelectObject( HGDIOBJ handle, HDC hdc )
         goto done;
     }
 
-    physdev = GET_DC_PHYSDEV( dc, pSelectBitmap );
+    old_physdev = GET_DC_PHYSDEV( dc, pSelectBitmap );
+    if(old_physdev == &dc->dibdrv.dev)
+        pop_dc_driver( dc, old_physdev );
+
+    if(bitmap->dib)
+    {
+        physdev = &dc->dibdrv.dev;
+        push_dc_driver( dc, physdev, physdev->funcs );
+    }
+    else
+        physdev = GET_DC_PHYSDEV( dc, pSelectBitmap );
 
     if (!BITMAP_SetOwnerDC( handle, physdev ))
     {
@@ -682,6 +794,11 @@ static HGDIOBJ BITMAP_SelectObject( HGDIOBJ handle, HDC hdc )
     }
 
  done:
+    if(!ret)
+    {
+        if(physdev == &dc->dibdrv.dev) pop_dc_driver( dc, physdev );
+        if(old_physdev == &dc->dibdrv.dev) push_dc_driver( dc, old_physdev, old_physdev->funcs );
+    }
     release_dc_ptr( dc );
     return ret;
 }
