@@ -62,9 +62,7 @@ static HGLOBAL dib_copy(const BITMAPINFO *info, UINT coloruse)
     if (info->bmiHeader.biCompression != BI_RGB && info->bmiHeader.biCompression != BI_BITFIELDS)
         size = info->bmiHeader.biSizeImage;
     else
-        size = DIB_GetDIBImageBytes(info->bmiHeader.biWidth,
-                                    info->bmiHeader.biHeight,
-                                    info->bmiHeader.biBitCount);
+        size = get_dib_image_size(info);
     size += bitmap_info_size( info, coloruse );
 
     if (!(hmem = GlobalAlloc( GMEM_MOVEABLE, size )))
@@ -331,9 +329,9 @@ HBRUSH WINAPI CreateSolidBrush( COLORREF color )
  * Set the brush origin for a device context.
  *
  * PARAMS
- *  hdc    [I] Device context to set the brush origin for 
+ *  hdc    [I] Device context to set the brush origin for
  *  x      [I] New x origin
- *  y      [I] Ney y origin
+ *  y      [I] New y origin
  *  oldorg [O] If non NULL, destination for previously set brush origin.
  *
  * RETURNS
@@ -390,7 +388,15 @@ static HGDIOBJ BRUSH_SelectObject( HGDIOBJ handle, HDC hdc )
         PHYSDEV physdev = GET_DC_PHYSDEV( dc, pSelectBrush );
 
         if (brush->logbrush.lbStyle == BS_PATTERN)
-            BITMAP_SetOwnerDC( (HBITMAP)brush->logbrush.lbHatch, physdev );
+        {
+            PHYSDEV pattern_dev = physdev;
+            /* FIXME: This will go away once the dib driver implements
+               pattern brushes */
+            if(pattern_dev == &dc->dibdrv.dev)
+                pattern_dev = GET_NEXT_PHYSDEV( physdev, pSelectBrush );
+
+            BITMAP_SetOwnerDC( (HBITMAP)brush->logbrush.lbHatch, pattern_dev );
+        }
 
         GDI_inc_ref_count( handle );
         GDI_ReleaseObj( handle );
