@@ -57,11 +57,11 @@ WINE_DEFAULT_DEBUG_CHANNEL(dib);
  *
  */
 
-/* A = (P & A1) | (~P & A2) */
-#define ZERO {0, 0}
-#define ONE {0xffffffff, 0xffffffff}
-#define P {0xffffffff, 0}
-#define NOT_P {0, 0xffffffff}
+/* A = (P & A1) ^ A2 */
+#define ZERO  { 0u,  0u}
+#define ONE   { 0u, ~0u}
+#define P     {~0u,  0u}
+#define NOT_P {~0u, ~0u}
 
 static const DWORD rop2_and_array[16][2] =
 {
@@ -71,7 +71,7 @@ static const DWORD rop2_and_array[16][2] =
     ZERO, NOT_P, NOT_P, ZERO
 };
 
-/* X = (P & X1) | (~P & X2) */
+/* X = (P & X1) ^ X2 */
 static const DWORD rop2_xor_array[16][2] =
 {
     ZERO, NOT_P, ZERO, NOT_P,
@@ -85,11 +85,22 @@ static const DWORD rop2_xor_array[16][2] =
 #undef ONE
 #undef ZERO
 
-void calc_and_xor_masks(INT rop, DWORD color, DWORD *and, DWORD *xor)
+void get_rop_codes(INT rop, struct rop_codes *codes)
 {
     /* NB The ROP2 codes start at one and the arrays are zero-based */
-    *and = (color & rop2_and_array[rop-1][0]) | ((~color) & rop2_and_array[rop-1][1]);
-    *xor = (color & rop2_xor_array[rop-1][0]) | ((~color) & rop2_xor_array[rop-1][1]);
+    codes->a1 = rop2_and_array[rop-1][0];
+    codes->a2 = rop2_and_array[rop-1][1];
+    codes->x1 = rop2_xor_array[rop-1][0];
+    codes->x2 = rop2_xor_array[rop-1][1];
+}
+
+void calc_and_xor_masks(INT rop, DWORD color, DWORD *and, DWORD *xor)
+{
+    struct rop_codes codes;
+    get_rop_codes( rop, &codes );
+
+    *and = (color & codes.a1) ^ codes.a2;
+    *xor = (color & codes.x1) ^ codes.x2;
 }
 
 static inline RGBQUAD rgbquad_from_colorref(COLORREF c)
@@ -926,7 +937,7 @@ static const dash_pattern dash_patterns[5] =
 /***********************************************************************
  *           dibdrv_SelectPen
  */
-HPEN CDECL dibdrv_SelectPen( PHYSDEV dev, HPEN hpen )
+HPEN dibdrv_SelectPen( PHYSDEV dev, HPEN hpen )
 {
     PHYSDEV next = GET_NEXT_PHYSDEV( dev, pSelectPen );
     dibdrv_physdev *pdev = get_dibdrv_pdev(dev);
@@ -1003,7 +1014,7 @@ HPEN CDECL dibdrv_SelectPen( PHYSDEV dev, HPEN hpen )
 /***********************************************************************
  *           dibdrv_SetDCPenColor
  */
-COLORREF CDECL dibdrv_SetDCPenColor( PHYSDEV dev, COLORREF color )
+COLORREF dibdrv_SetDCPenColor( PHYSDEV dev, COLORREF color )
 {
     PHYSDEV next = GET_NEXT_PHYSDEV( dev, pSetDCPenColor );
     dibdrv_physdev *pdev = get_dibdrv_pdev(dev);
@@ -1072,7 +1083,7 @@ static void free_pattern_brush_bits( dibdrv_physdev *pdev )
 void free_pattern_brush( dibdrv_physdev *pdev )
 {
     free_pattern_brush_bits( pdev );
-    free_dib_info( &pdev->brush_dib, TRUE );
+    free_dib_info( &pdev->brush_dib );
 }
 
 static BOOL create_pattern_brush_bits(dibdrv_physdev *pdev)
@@ -1133,7 +1144,7 @@ static BOOL create_hatch_brush_bits(dibdrv_physdev *pdev)
     copy_dib_color_info(&pdev->brush_dib, &pdev->dib);
     pdev->brush_dib.width  = 8;
     pdev->brush_dib.height = 8;
-    pdev->brush_dib.stride = ((pdev->brush_dib.width * pdev->brush_dib.bit_count + 31) >> 3) & ~3;
+    pdev->brush_dib.stride = get_dib_stride( pdev->brush_dib.width, pdev->brush_dib.bit_count );
 
     size = pdev->brush_dib.height * pdev->brush_dib.stride;
 
@@ -1245,7 +1256,7 @@ void update_brush_rop( dibdrv_physdev *pdev, INT rop )
 /***********************************************************************
  *           dibdrv_SelectBrush
  */
-HBRUSH CDECL dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush )
+HBRUSH dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush )
 {
     PHYSDEV next = GET_NEXT_PHYSDEV( dev, pSelectBrush );
     dibdrv_physdev *pdev = get_dibdrv_pdev(dev);
@@ -1285,17 +1296,31 @@ HBRUSH CDECL dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush )
         dib_info orig_dib;
         WORD usage = LOWORD(logbrush.lbColor);
         HPALETTE pal = (usage == DIB_PAL_COLORS) ? GetCurrentObject(dev->hdc, OBJ_PAL) : NULL;
+        RECT rect;
 
         if(!bi) return NULL;
         if(init_dib_info_from_packed(&orig_dib, bi, usage, pal))
         {
             copy_dib_color_info(&pdev->brush_dib, &pdev->dib);
-            if(convert_dib(&pdev->brush_dib, &orig_dib))
+
+            pdev->brush_dib.height = orig_dib.height;
+            pdev->brush_dib.width  = orig_dib.width;
+            pdev->brush_dib.stride = get_dib_stride( pdev->brush_dib.width, pdev->brush_dib.bit_count );
+            pdev->brush_dib.ptr_to_free = HeapAlloc( GetProcessHeap(), 0, pdev->brush_dib.height * pdev->brush_dib.stride );
+            pdev->brush_dib.bits = pdev->brush_dib.ptr_to_free;
+
+            rect.left = rect.top = 0;
+            rect.right = orig_dib.width;
+            rect.bottom = orig_dib.height;
+
+            if(pdev->brush_dib.funcs->convert_to(&pdev->brush_dib, &orig_dib, &rect))
             {
                 pdev->brush_rects = pattern_brush;
                 pdev->defer &= ~DEFER_BRUSH;
             }
-            free_dib_info(&orig_dib, FALSE);
+            else
+                free_dib_info(&pdev->brush_dib);
+            free_dib_info(&orig_dib);
         }
         GlobalUnlock((HGLOBAL)logbrush.lbHatch);
         break;
@@ -1323,7 +1348,7 @@ HBRUSH CDECL dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush )
 /***********************************************************************
  *           dibdrv_SetDCBrushColor
  */
-COLORREF CDECL dibdrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
+COLORREF dibdrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
 {
     PHYSDEV next = GET_NEXT_PHYSDEV( dev, pSetDCBrushColor );
     dibdrv_physdev *pdev = get_dibdrv_pdev(dev);
