@@ -22,10 +22,12 @@
 #define __WINE_GDI_PRIVATE_H
 
 #include <math.h>
+#include <stdlib.h>
 #include <stdarg.h>
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
+#include "wine/gdi_driver.h"
 
 /* Metafile defines */
 #define META_EOF 0x0000
@@ -72,18 +74,17 @@ typedef struct tagGDIOBJHDR
 
 /* Device functions for the Wine driver interface */
 
-typedef struct gdi_physdev
+enum dib_info_flags
 {
-    const struct tagDC_FUNCS *funcs;
-    struct gdi_physdev       *next;
-    HDC                       hdc;
-} *PHYSDEV;
+    private_color_table = 1
+};
 
 typedef struct
 {
     int bit_count, width, height;
     int stride; /* stride in bytes.  Will be -ve for bottom-up dibs (see bits). */
     void *bits; /* points to the top-left corner of the dib. */
+    void *ptr_to_free;
 
     DWORD red_mask, green_mask, blue_mask;
     int red_shift, green_shift, blue_shift;
@@ -92,13 +93,15 @@ typedef struct
     RGBQUAD *color_table;
     DWORD color_table_size;
 
+    enum dib_info_flags flags;
+
     const struct primitive_funcs *funcs;
 } dib_info;
 
 typedef struct
 {
     DWORD count;
-    DWORD dashes[16]; /* 16 is the maximium number for a PS_USERSTYLE pen */
+    DWORD dashes[16]; /* 16 is the maximum number for a PS_USERSTYLE pen */
     DWORD total_len;  /* total length of the dashes, should be multiplied by 2 if there are an odd number of dash lengths */
 } dash_pattern;
 
@@ -154,144 +157,7 @@ typedef struct dibdrv_physdev
 #define DEFER_PEN        2
 #define DEFER_BRUSH      4
 
-typedef struct tagDC_FUNCS
-{
-    INT      (CDECL *pAbortDoc)(PHYSDEV);
-    BOOL     (CDECL *pAbortPath)(PHYSDEV);
-    BOOL     (CDECL *pAlphaBlend)(PHYSDEV,INT,INT,INT,INT,PHYSDEV,INT,INT,INT,INT,BLENDFUNCTION);
-    BOOL     (CDECL *pAngleArc)(PHYSDEV,INT,INT,DWORD,FLOAT,FLOAT);
-    BOOL     (CDECL *pArc)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT);
-    BOOL     (CDECL *pArcTo)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT);
-    BOOL     (CDECL *pBeginPath)(PHYSDEV);
-    INT      (CDECL *pChoosePixelFormat)(PHYSDEV,const PIXELFORMATDESCRIPTOR *);
-    BOOL     (CDECL *pChord)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT);
-    BOOL     (CDECL *pCloseFigure)(PHYSDEV);
-    BOOL     (CDECL *pCreateBitmap)(PHYSDEV,HBITMAP,LPVOID);
-    BOOL     (CDECL *pCreateDC)(HDC,PHYSDEV *,LPCWSTR,LPCWSTR,LPCWSTR,const DEVMODEW*);
-    HBITMAP  (CDECL *pCreateDIBSection)(PHYSDEV,HBITMAP,const BITMAPINFO *,UINT);
-    BOOL     (CDECL *pDeleteBitmap)(HBITMAP);
-    BOOL     (CDECL *pDeleteDC)(PHYSDEV);
-    BOOL     (CDECL *pDeleteObject)(PHYSDEV,HGDIOBJ);
-    INT      (CDECL *pDescribePixelFormat)(PHYSDEV,INT,UINT,PIXELFORMATDESCRIPTOR *);
-    DWORD    (CDECL *pDeviceCapabilities)(LPSTR,LPCSTR,LPCSTR,WORD,LPSTR,LPDEVMODEA);
-    BOOL     (CDECL *pEllipse)(PHYSDEV,INT,INT,INT,INT);
-    INT      (CDECL *pEndDoc)(PHYSDEV);
-    INT      (CDECL *pEndPage)(PHYSDEV);
-    BOOL     (CDECL *pEndPath)(PHYSDEV);
-    BOOL     (CDECL *pEnumDeviceFonts)(PHYSDEV,LPLOGFONTW,FONTENUMPROCW,LPARAM);
-    INT      (CDECL *pEnumICMProfiles)(PHYSDEV,ICMENUMPROCW,LPARAM);
-    INT      (CDECL *pExcludeClipRect)(PHYSDEV,INT,INT,INT,INT);
-    INT      (CDECL *pExtDeviceMode)(LPSTR,HWND,LPDEVMODEA,LPSTR,LPSTR,LPDEVMODEA,LPSTR,DWORD);
-    INT      (CDECL *pExtEscape)(PHYSDEV,INT,INT,LPCVOID,INT,LPVOID);
-    BOOL     (CDECL *pExtFloodFill)(PHYSDEV,INT,INT,COLORREF,UINT);
-    INT      (CDECL *pExtSelectClipRgn)(PHYSDEV,HRGN,INT);
-    BOOL     (CDECL *pExtTextOut)(PHYSDEV,INT,INT,UINT,const RECT*,LPCWSTR,UINT,const INT*);
-    BOOL     (CDECL *pFillPath)(PHYSDEV);
-    BOOL     (CDECL *pFillRgn)(PHYSDEV,HRGN,HBRUSH);
-    BOOL     (CDECL *pFlattenPath)(PHYSDEV);
-    BOOL     (CDECL *pFrameRgn)(PHYSDEV,HRGN,HBRUSH,INT,INT);
-    BOOL     (CDECL *pGdiComment)(PHYSDEV,UINT,CONST BYTE*);
-    LONG     (CDECL *pGetBitmapBits)(HBITMAP,void*,LONG);
-    BOOL     (CDECL *pGetCharWidth)(PHYSDEV,UINT,UINT,LPINT);
-    INT      (CDECL *pGetDIBits)(PHYSDEV,HBITMAP,UINT,UINT,LPVOID,BITMAPINFO*,UINT);
-    INT      (CDECL *pGetDeviceCaps)(PHYSDEV,INT);
-    BOOL     (CDECL *pGetDeviceGammaRamp)(PHYSDEV,LPVOID);
-    BOOL     (CDECL *pGetICMProfile)(PHYSDEV,LPDWORD,LPWSTR);
-    COLORREF (CDECL *pGetNearestColor)(PHYSDEV,COLORREF);
-    COLORREF (CDECL *pGetPixel)(PHYSDEV,INT,INT);
-    INT      (CDECL *pGetPixelFormat)(PHYSDEV);
-    UINT     (CDECL *pGetSystemPaletteEntries)(PHYSDEV,UINT,UINT,LPPALETTEENTRY);
-    BOOL     (CDECL *pGetTextExtentExPoint)(PHYSDEV,LPCWSTR,INT,INT,LPINT,LPINT,LPSIZE);
-    BOOL     (CDECL *pGetTextMetrics)(PHYSDEV,TEXTMETRICW*);
-    INT      (CDECL *pIntersectClipRect)(PHYSDEV,INT,INT,INT,INT);
-    BOOL     (CDECL *pInvertRgn)(PHYSDEV,HRGN);
-    BOOL     (CDECL *pLineTo)(PHYSDEV,INT,INT);
-    BOOL     (CDECL *pModifyWorldTransform)(PHYSDEV,const XFORM*,DWORD);
-    BOOL     (CDECL *pMoveTo)(PHYSDEV,INT,INT);
-    INT      (CDECL *pOffsetClipRgn)(PHYSDEV,INT,INT);
-    BOOL     (CDECL *pOffsetViewportOrgEx)(PHYSDEV,INT,INT,POINT*);
-    BOOL     (CDECL *pOffsetWindowOrgEx)(PHYSDEV,INT,INT,POINT*);
-    BOOL     (CDECL *pPaintRgn)(PHYSDEV,HRGN);
-    BOOL     (CDECL *pPatBlt)(PHYSDEV,INT,INT,INT,INT,DWORD);
-    BOOL     (CDECL *pPie)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT);
-    BOOL     (CDECL *pPolyBezier)(PHYSDEV,const POINT*,DWORD);
-    BOOL     (CDECL *pPolyBezierTo)(PHYSDEV,const POINT*,DWORD);
-    BOOL     (CDECL *pPolyDraw)(PHYSDEV,const POINT*,const BYTE *,DWORD);
-    BOOL     (CDECL *pPolyPolygon)(PHYSDEV,const POINT*,const INT*,UINT);
-    BOOL     (CDECL *pPolyPolyline)(PHYSDEV,const POINT*,const DWORD*,DWORD);
-    BOOL     (CDECL *pPolygon)(PHYSDEV,const POINT*,INT);
-    BOOL     (CDECL *pPolyline)(PHYSDEV,const POINT*,INT);
-    BOOL     (CDECL *pPolylineTo)(PHYSDEV,const POINT*,INT);
-    UINT     (CDECL *pRealizeDefaultPalette)(PHYSDEV);
-    UINT     (CDECL *pRealizePalette)(PHYSDEV,HPALETTE,BOOL);
-    BOOL     (CDECL *pRectangle)(PHYSDEV,INT,INT,INT,INT);
-    HDC      (CDECL *pResetDC)(PHYSDEV,const DEVMODEW*);
-    BOOL     (CDECL *pRestoreDC)(PHYSDEV,INT);
-    BOOL     (CDECL *pRoundRect)(PHYSDEV,INT,INT,INT,INT,INT,INT);
-    INT      (CDECL *pSaveDC)(PHYSDEV);
-    BOOL     (CDECL *pScaleViewportExtEx)(PHYSDEV,INT,INT,INT,INT,SIZE*);
-    BOOL     (CDECL *pScaleWindowExtEx)(PHYSDEV,INT,INT,INT,INT,SIZE*);
-    HBITMAP  (CDECL *pSelectBitmap)(PHYSDEV,HBITMAP);
-    HBRUSH   (CDECL *pSelectBrush)(PHYSDEV,HBRUSH);
-    BOOL     (CDECL *pSelectClipPath)(PHYSDEV,INT);
-    HFONT    (CDECL *pSelectFont)(PHYSDEV,HFONT,HANDLE);
-    HPALETTE (CDECL *pSelectPalette)(PHYSDEV,HPALETTE,BOOL);
-    HPEN     (CDECL *pSelectPen)(PHYSDEV,HPEN);
-    INT      (CDECL *pSetArcDirection)(PHYSDEV,INT);
-    LONG     (CDECL *pSetBitmapBits)(HBITMAP,const void*,LONG);
-    COLORREF (CDECL *pSetBkColor)(PHYSDEV,COLORREF);
-    INT      (CDECL *pSetBkMode)(PHYSDEV,INT);
-    COLORREF (CDECL *pSetDCBrushColor)(PHYSDEV, COLORREF);
-    COLORREF (CDECL *pSetDCPenColor)(PHYSDEV, COLORREF);
-    UINT     (CDECL *pSetDIBColorTable)(PHYSDEV,UINT,UINT,const RGBQUAD*);
-    INT      (CDECL *pSetDIBits)(PHYSDEV,HBITMAP,UINT,UINT,LPCVOID,const BITMAPINFO*,UINT);
-    INT      (CDECL *pSetDIBitsToDevice)(PHYSDEV,INT,INT,DWORD,DWORD,INT,INT,UINT,UINT,LPCVOID,
-                                           const BITMAPINFO*,UINT);
-    VOID     (CDECL *pSetDeviceClipping)(PHYSDEV,HRGN,HRGN);
-    BOOL     (CDECL *pSetDeviceGammaRamp)(PHYSDEV,LPVOID);
-    DWORD    (CDECL *pSetLayout)(PHYSDEV,DWORD);
-    INT      (CDECL *pSetMapMode)(PHYSDEV,INT);
-    DWORD    (CDECL *pSetMapperFlags)(PHYSDEV,DWORD);
-    COLORREF (CDECL *pSetPixel)(PHYSDEV,INT,INT,COLORREF);
-    BOOL     (CDECL *pSetPixelFormat)(PHYSDEV,INT,const PIXELFORMATDESCRIPTOR *);
-    INT      (CDECL *pSetPolyFillMode)(PHYSDEV,INT);
-    INT      (CDECL *pSetROP2)(PHYSDEV,INT);
-    INT      (CDECL *pSetRelAbs)(PHYSDEV,INT);
-    INT      (CDECL *pSetStretchBltMode)(PHYSDEV,INT);
-    UINT     (CDECL *pSetTextAlign)(PHYSDEV,UINT);
-    INT      (CDECL *pSetTextCharacterExtra)(PHYSDEV,INT);
-    COLORREF (CDECL *pSetTextColor)(PHYSDEV,COLORREF);
-    BOOL     (CDECL *pSetTextJustification)(PHYSDEV,INT,INT);
-    BOOL     (CDECL *pSetViewportExtEx)(PHYSDEV,INT,INT,SIZE*);
-    BOOL     (CDECL *pSetViewportOrgEx)(PHYSDEV,INT,INT,POINT*);
-    BOOL     (CDECL *pSetWindowExtEx)(PHYSDEV,INT,INT,SIZE*);
-    BOOL     (CDECL *pSetWindowOrgEx)(PHYSDEV,INT,INT,POINT*);
-    BOOL     (CDECL *pSetWorldTransform)(PHYSDEV,const XFORM*);
-    INT      (CDECL *pStartDoc)(PHYSDEV,const DOCINFOW*);
-    INT      (CDECL *pStartPage)(PHYSDEV);
-    BOOL     (CDECL *pStretchBlt)(PHYSDEV,INT,INT,INT,INT,PHYSDEV,INT,INT,INT,INT,DWORD);
-    INT      (CDECL *pStretchDIBits)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT,const void *,
-                                       const BITMAPINFO*,UINT,DWORD);
-    BOOL     (CDECL *pStrokeAndFillPath)(PHYSDEV);
-    BOOL     (CDECL *pStrokePath)(PHYSDEV);
-    BOOL     (CDECL *pSwapBuffers)(PHYSDEV);
-    BOOL     (CDECL *pUnrealizePalette)(HPALETTE);
-    BOOL     (CDECL *pWidenPath)(PHYSDEV);
-
-    /* OpenGL32 */
-    BOOL     (CDECL *pwglCopyContext)(HGLRC, HGLRC, UINT);
-    HGLRC    (CDECL *pwglCreateContext)(PHYSDEV);
-    HGLRC    (CDECL *pwglCreateContextAttribsARB)(PHYSDEV, HGLRC, const int*);
-    BOOL     (CDECL *pwglDeleteContext)(HGLRC);
-    PROC     (CDECL *pwglGetProcAddress)(LPCSTR);
-    HDC      (CDECL *pwglGetPbufferDCARB)(PHYSDEV, void*);
-    BOOL     (CDECL *pwglMakeCurrent)(PHYSDEV, HGLRC);
-    BOOL     (CDECL *pwglMakeContextCurrentARB)(PHYSDEV, PHYSDEV, HGLRC);
-    BOOL     (CDECL *pwglSetPixelFormatWINE)(PHYSDEV,INT,const PIXELFORMATDESCRIPTOR *);
-    BOOL     (CDECL *pwglShareLists)(HGLRC hglrc1, HGLRC hglrc2);
-    BOOL     (CDECL *pwglUseFontBitmapsA)(PHYSDEV, DWORD, DWORD, DWORD);
-    BOOL     (CDECL *pwglUseFontBitmapsW)(PHYSDEV, DWORD, DWORD, DWORD);
-} DC_FUNCTIONS;
+typedef struct gdi_dc_funcs DC_FUNCTIONS;
 
 /* It should not be necessary to access the contents of the GdiPath
  * structure directly; if you find that the exported functions don't
@@ -408,6 +274,9 @@ static inline INT GDI_ROUND(double val)
    return (int)floor(val + 0.5);
 }
 
+#define GET_DC_PHYSDEV(dc,func) \
+    get_physdev_entry_point( (dc)->physDev, FIELD_OFFSET(struct gdi_dc_funcs,func))
+
 /* bitmap object */
 
 typedef struct tagBITMAPOBJ
@@ -440,11 +309,12 @@ extern BOOL BIDI_Reorder( HDC hDC, LPCWSTR lpString, INT uCount, DWORD dwFlags,
                           LPWSTR lpOutString, INT uCountOut, UINT *lpOrder, WORD **lpGlyphs, INT* cGlyphs ) DECLSPEC_HIDDEN;
 
 /* bitmap.c */
+extern BOOL get_bitmap_image( HBITMAP hbitmap, BITMAPINFO *info, struct gdi_image_bits *bits ) DECLSPEC_HIDDEN;
 extern HBITMAP BITMAP_CopyBitmap( HBITMAP hbitmap ) DECLSPEC_HIDDEN;
 extern BOOL BITMAP_SetOwnerDC( HBITMAP hbitmap, PHYSDEV physdev ) DECLSPEC_HIDDEN;
-extern INT BITMAP_GetWidthBytes( INT bmWidth, INT bpp ) DECLSPEC_HIDDEN;
 
 /* clipping.c */
+extern int get_clip_box( DC *dc, RECT *rect ) DECLSPEC_HIDDEN;
 extern void CLIPPING_UpdateGCRegion( DC * dc ) DECLSPEC_HIDDEN;
 
 /* Return the total clip region (if any) */
@@ -467,11 +337,10 @@ extern void DC_InitDC( DC * dc ) DECLSPEC_HIDDEN;
 extern void DC_UpdateXforms( DC * dc ) DECLSPEC_HIDDEN;
 
 /* dib.c */
-extern int DIB_GetDIBWidthBytes( int width, int depth ) DECLSPEC_HIDDEN;
-extern int DIB_GetDIBImageBytes( int width, int height, int depth ) DECLSPEC_HIDDEN;
 extern int bitmap_info_size( const BITMAPINFO * info, WORD coloruse ) DECLSPEC_HIDDEN;
-extern int DIB_GetBitmapInfo( const BITMAPINFOHEADER *header, LONG *width,
-                              LONG *height, WORD *planes, WORD *bpp, DWORD *compr, DWORD *size ) DECLSPEC_HIDDEN;
+extern DWORD convert_bitmapinfo( const BITMAPINFO *src_info, void *src_bits, struct bitblt_coords *src,
+                                 const BITMAPINFO *dst_info, void *dst_bits ) DECLSPEC_HIDDEN;
+
 
 /* driver.c */
 extern const DC_FUNCTIONS null_driver DECLSPEC_HIDDEN;
@@ -480,15 +349,6 @@ extern const DC_FUNCTIONS *DRIVER_get_display_driver(void) DECLSPEC_HIDDEN;
 extern const DC_FUNCTIONS *DRIVER_load_driver( LPCWSTR name ) DECLSPEC_HIDDEN;
 extern BOOL DRIVER_GetDriverName( LPCWSTR device, LPWSTR driver, DWORD size ) DECLSPEC_HIDDEN;
 
-static inline PHYSDEV get_physdev_entry_point( PHYSDEV dev, size_t offset )
-{
-    while (!((void **)dev->funcs)[offset / sizeof(void *)]) dev = dev->next;
-    return dev;
-}
-
-#define GET_DC_PHYSDEV(dc,func) get_physdev_entry_point( (dc)->physDev, FIELD_OFFSET(DC_FUNCTIONS,func))
-#define GET_NEXT_PHYSDEV(dev,func) get_physdev_entry_point( (dev)->next, FIELD_OFFSET(DC_FUNCTIONS,func))
-
 /* enhmetafile.c */
 extern HENHMETAFILE EMF_Create_HENHMETAFILE(ENHMETAHEADER *emh, BOOL on_disk ) DECLSPEC_HIDDEN;
 
@@ -607,52 +467,53 @@ static inline void release_wine_region(HRGN rgn)
 }
 
 /* null driver entry points */
-extern BOOL CDECL nulldrv_AbortPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_AngleArc( PHYSDEV dev, INT x, INT y, DWORD radius, FLOAT start, FLOAT sweep ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_ArcTo( PHYSDEV dev, INT left, INT top, INT right, INT bottom, INT xstart, INT ystart, INT xend, INT yend ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_BeginPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_CloseFigure( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_EndPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_ExcludeClipRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_ExtSelectClipRgn( PHYSDEV dev, HRGN rgn, INT mode ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_FillPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_FillRgn( PHYSDEV dev, HRGN rgn, HBRUSH brush ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_FlattenPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_FrameRgn( PHYSDEV dev, HRGN rgn, HBRUSH brush, INT width, INT height ) DECLSPEC_HIDDEN;
-extern LONG CDECL nulldrv_GetBitmapBits( HBITMAP bitmap, void *bits, LONG size ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_GetDIBits( PHYSDEV dev, HBITMAP bitmap, UINT start, UINT lines, LPVOID bits, BITMAPINFO *info, UINT coloruse ) DECLSPEC_HIDDEN;
-extern COLORREF CDECL nulldrv_GetNearestColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_IntersectClipRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_InvertRgn( PHYSDEV dev, HRGN rgn ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_ModifyWorldTransform( PHYSDEV dev, const XFORM *xform, DWORD mode ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_OffsetClipRgn( PHYSDEV dev, INT x, INT y ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_OffsetViewportOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_OffsetWindowOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_PolyBezier( PHYSDEV dev, const POINT *points, DWORD count ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_PolyBezierTo( PHYSDEV dev, const POINT *points, DWORD count ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_PolyDraw( PHYSDEV dev, const POINT *points, const BYTE *types, DWORD count ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_PolylineTo( PHYSDEV dev, const POINT *points, INT count ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_RestoreDC( PHYSDEV dev, INT level ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_SaveDC( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_ScaleViewportExtEx( PHYSDEV dev, INT x_num, INT x_denom, INT y_num, INT y_denom, SIZE *size ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_ScaleWindowExtEx( PHYSDEV dev, INT x_num, INT x_denom, INT y_num, INT y_denom, SIZE *size ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_SelectClipPath( PHYSDEV dev, INT mode ) DECLSPEC_HIDDEN;
-extern LONG CDECL nulldrv_SetBitmapBits( HBITMAP bitmap, const void *bits, LONG size ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_SetDIBits( PHYSDEV dev, HBITMAP bitmap, UINT start, UINT lines, const void *bits, const BITMAPINFO *info, UINT coloruse ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_SetMapMode( PHYSDEV dev, INT mode ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_SetViewportExtEx( PHYSDEV dev, INT cx, INT cy, SIZE *size ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_SetViewportOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_SetWindowExtEx( PHYSDEV dev, INT cx, INT cy, SIZE *size ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_SetWindowOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_SetWorldTransform( PHYSDEV dev, const XFORM *xform ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_StretchBlt( PHYSDEV dst_dev, INT xDst, INT yDst, INT widthDst, INT heightDst,
-                                      PHYSDEV src_dev, INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, DWORD rop ) DECLSPEC_HIDDEN;
-extern INT  CDECL nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst, INT heightDst,
-                                         INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, const void *bits,
-                                         const BITMAPINFO *info, UINT coloruse, DWORD rop ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_StrokeAndFillPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_StrokePath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL CDECL nulldrv_WidenPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_AbortPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_AngleArc( PHYSDEV dev, INT x, INT y, DWORD radius, FLOAT start, FLOAT sweep ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_ArcTo( PHYSDEV dev, INT left, INT top, INT right, INT bottom, INT xstart, INT ystart, INT xend, INT yend ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_BeginPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_CloseFigure( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_EndPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_ExcludeClipRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_ExtSelectClipRgn( PHYSDEV dev, HRGN rgn, INT mode ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_FillPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_FillRgn( PHYSDEV dev, HRGN rgn, HBRUSH brush ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_FlattenPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_FrameRgn( PHYSDEV dev, HRGN rgn, HBRUSH brush, INT width, INT height ) DECLSPEC_HIDDEN;
+extern LONG nulldrv_GetBitmapBits( HBITMAP bitmap, void *bits, LONG size ) DECLSPEC_HIDDEN;
+extern DWORD nulldrv_GetImage( PHYSDEV dev, HBITMAP hbitmap, BITMAPINFO *info, struct gdi_image_bits *bits, struct bitblt_coords *src ) DECLSPEC_HIDDEN;
+extern COLORREF nulldrv_GetNearestColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_IntersectClipRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_InvertRgn( PHYSDEV dev, HRGN rgn ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_ModifyWorldTransform( PHYSDEV dev, const XFORM *xform, DWORD mode ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_OffsetClipRgn( PHYSDEV dev, INT x, INT y ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_OffsetViewportOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_OffsetWindowOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_PolyBezier( PHYSDEV dev, const POINT *points, DWORD count ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_PolyBezierTo( PHYSDEV dev, const POINT *points, DWORD count ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_PolyDraw( PHYSDEV dev, const POINT *points, const BYTE *types, DWORD count ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_PolylineTo( PHYSDEV dev, const POINT *points, INT count ) DECLSPEC_HIDDEN;
+extern DWORD nulldrv_PutImage( PHYSDEV dev, HBITMAP hbitmap, HRGN clip, BITMAPINFO *info,
+                               const struct gdi_image_bits *bits, struct bitblt_coords *src,
+                               struct bitblt_coords *dst, DWORD rop ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_RestoreDC( PHYSDEV dev, INT level ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_SaveDC( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_ScaleViewportExtEx( PHYSDEV dev, INT x_num, INT x_denom, INT y_num, INT y_denom, SIZE *size ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_ScaleWindowExtEx( PHYSDEV dev, INT x_num, INT x_denom, INT y_num, INT y_denom, SIZE *size ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_SelectClipPath( PHYSDEV dev, INT mode ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_SetMapMode( PHYSDEV dev, INT mode ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_SetViewportExtEx( PHYSDEV dev, INT cx, INT cy, SIZE *size ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_SetViewportOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_SetWindowExtEx( PHYSDEV dev, INT cx, INT cy, SIZE *size ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_SetWindowOrgEx( PHYSDEV dev, INT x, INT y, POINT *pt ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_SetWorldTransform( PHYSDEV dev, const XFORM *xform ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
+                                PHYSDEV src_dev, struct bitblt_coords *src, DWORD rop ) DECLSPEC_HIDDEN;
+extern INT  nulldrv_StretchDIBits( PHYSDEV dev, INT xDst, INT yDst, INT widthDst, INT heightDst,
+                                   INT xSrc, INT ySrc, INT widthSrc, INT heightSrc, const void *bits,
+                                   const BITMAPINFO *info, UINT coloruse, DWORD rop ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_StrokeAndFillPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_StrokePath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL nulldrv_WidenPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
 
 static inline DC *get_nulldrv_dc( PHYSDEV dev )
 {
@@ -666,4 +527,50 @@ BOOL WINAPI FontIsLinked(HDC);
 
 BOOL WINAPI SetVirtualResolution(HDC hdc, DWORD horz_res, DWORD vert_res, DWORD horz_size, DWORD vert_size);
 
+static inline BOOL is_rect_empty( const RECT *rect )
+{
+    return (rect->left >= rect->right || rect->top >= rect->bottom);
+}
+
+static inline BOOL intersect_rect( RECT *dst, const RECT *src1, const RECT *src2 )
+{
+    dst->left   = max( src1->left, src2->left );
+    dst->top    = max( src1->top, src2->top );
+    dst->right  = min( src1->right, src2->right );
+    dst->bottom = min( src1->bottom, src2->bottom );
+    return !is_rect_empty( dst );
+}
+
+static inline void offset_rect( RECT *rect, int offset_x, int offset_y )
+{
+    rect->left   += offset_x;
+    rect->top    += offset_y;
+    rect->right  += offset_x;
+    rect->bottom += offset_y;
+}
+
+static inline int get_bitmap_stride( int width, int bpp )
+{
+    return ((width * bpp + 15) >> 3) & ~1;
+}
+
+static inline int get_dib_stride( int width, int bpp )
+{
+    return ((width * bpp + 31) >> 3) & ~3;
+}
+
+static inline int get_dib_image_size( const BITMAPINFO *info )
+{
+    return get_dib_stride( info->bmiHeader.biWidth, info->bmiHeader.biBitCount )
+        * abs( info->bmiHeader.biHeight );
+}
+
+static inline int get_dib_num_of_colors( const BITMAPINFO *info )
+{
+    if (info->bmiHeader.biClrUsed) return min( info->bmiHeader.biClrUsed, 256 );
+    return info->bmiHeader.biBitCount > 8 ? 0 : 1 << info->bmiHeader.biBitCount;
+}
+
+extern void free_heap_bits( struct gdi_image_bits *bits ) DECLSPEC_HIDDEN;
+
 #endif /* __WINE_GDI_PRIVATE_H */
