@@ -406,7 +406,23 @@ BOOL WINAPI ReadFile( HANDLE hFile, LPVOID buffer, DWORD bytesToRead,
     if (!bytesToRead) return TRUE;
 
     if (is_console_handle(hFile))
-        return ReadConsoleA(hFile, buffer, bytesToRead, bytesRead, NULL);
+    {
+        DWORD conread, mode;
+        if (!ReadConsoleA(hFile, buffer, bytesToRead, &conread, NULL) ||
+            !GetConsoleMode(hFile, &mode))
+            return FALSE;
+        /* ctrl-Z (26) means end of file on window (if at beginning of buffer)
+         * but Unix uses ctrl-D (4), and ctrl-Z is a bad idea on Unix :-/
+         * So map both ctrl-D ctrl-Z to EOF.
+         */
+        if ((mode & ENABLE_PROCESSED_INPUT) && conread > 0 &&
+            (((char*)buffer)[0] == 26 || ((char*)buffer)[0] == 4))
+        {
+            conread = 0;
+        }
+        if (bytesRead) *bytesRead = conread;
+        return TRUE;
+    }
 
     if (overlapped != NULL)
     {
@@ -1192,7 +1208,7 @@ BOOL WINAPI UnlockFileEx( HANDLE hFile, DWORD reserved, DWORD count_low, DWORD c
  */
 UINT WINAPI SetHandleCount( UINT count )
 {
-    return min( 256, count );
+    return count;
 }
 
 
@@ -1256,9 +1272,10 @@ HANDLE WINAPI CreateFileW( LPCWSTR filename, DWORD access, DWORD sharing,
         return INVALID_HANDLE_VALUE;
     }
 
-    TRACE("%s %s%s%s%s%s%s creation %d attributes 0x%x\n", debugstr_w(filename),
+    TRACE("%s %s%s%s%s%s%s%s creation %d attributes 0x%x\n", debugstr_w(filename),
           (access & GENERIC_READ)?"GENERIC_READ ":"",
           (access & GENERIC_WRITE)?"GENERIC_WRITE ":"",
+          (access & GENERIC_EXECUTE)?"GENERIC_EXECUTE ":"",
           (!access)?"QUERY_ACCESS ":"",
           (sharing & FILE_SHARE_READ)?"FILE_SHARE_READ ":"",
           (sharing & FILE_SHARE_WRITE)?"FILE_SHARE_WRITE ":"",
@@ -1305,10 +1322,10 @@ HANDLE WINAPI CreateFileW( LPCWSTR filename, DWORD access, DWORD sharing,
             switch (access & (GENERIC_READ|GENERIC_WRITE))
             {
             case GENERIC_READ:
-                ret = OpenConsoleW(coninW, access, (sa && sa->bInheritHandle), creation);
+                ret = OpenConsoleW(coninW, access, (sa && sa->bInheritHandle), OPEN_EXISTING);
                 goto done;
             case GENERIC_WRITE:
-                ret = OpenConsoleW(conoutW, access, (sa && sa->bInheritHandle), creation);
+                ret = OpenConsoleW(conoutW, access, (sa && sa->bInheritHandle), OPEN_EXISTING);
                 goto done;
             default:
                 SetLastError( ERROR_FILE_NOT_FOUND );
@@ -1510,6 +1527,10 @@ BOOL WINAPI ReplaceFileW(LPCWSTR lpReplacedFileName, LPCWSTR lpReplacementFileNa
     IO_STATUS_BLOCK io;
     OBJECT_ATTRIBUTES attr;
 
+    TRACE("%s %s %s 0x%08x %p %p\n", debugstr_w(lpReplacedFileName),
+          debugstr_w(lpReplacementFileName), debugstr_w(lpBackupFileName),
+          dwReplaceFlags, lpExclude, lpReserved);
+
     if (dwReplaceFlags)
         FIXME("Ignoring flags %x\n", dwReplaceFlags);
 
@@ -2519,3 +2540,68 @@ error:  /* We get here if there was an error opening the file */
     WARN("(%s): return = HFILE_ERROR error= %d\n", name,ofs->nErrCode );
     return HFILE_ERROR;
 }
+
+/***********************************************************************
+ *           K32EnumDeviceDrivers (KERNEL32.@)
+ */
+BOOL WINAPI K32EnumDeviceDrivers(void **image_base, DWORD cb, DWORD *needed)
+{
+    FIXME("(%p, %d, %p): stub\n", image_base, cb, needed);
+
+    if (needed)
+        *needed = 0;
+
+    return TRUE;
+}
+
+/***********************************************************************
+ *          K32GetDeviceDriverBaseNameA (KERNEL32.@)
+ */
+DWORD WINAPI K32GetDeviceDriverBaseNameA(void *image_base, LPSTR base_name, DWORD size)
+{
+    FIXME("(%p, %p, %d): stub\n", image_base, base_name, size);
+
+    if (base_name && size)
+        base_name[0] = '\0';
+
+    return 0;
+}
+
+/***********************************************************************
+ *           K32GetDeviceDriverBaseNameW (KERNEL32.@)
+ */
+DWORD WINAPI K32GetDeviceDriverBaseNameW(void *image_base, LPWSTR base_name, DWORD size)
+{
+    FIXME("(%p, %p, %d): stub\n", image_base, base_name, size);
+
+    if (base_name && size)
+        base_name[0] = '\0';
+
+    return 0;
+}
+
+/***********************************************************************
+ *           K32GetDeviceDriverFileNameA (KERNEL32.@)
+ */
+DWORD WINAPI K32GetDeviceDriverFileNameA(void *image_base, LPSTR file_name, DWORD size)
+{
+    FIXME("(%p, %p, %d): stub\n", image_base, file_name, size);
+
+    if (file_name && size)
+        file_name[0] = '\0';
+
+    return 0;
+}
+
+/***********************************************************************
+ *           K32GetDeviceDriverFileNameW (KERNEL32.@)
+ */
+DWORD WINAPI K32GetDeviceDriverFileNameW(void *image_base, LPWSTR file_name, DWORD size)
+{
+    FIXME("(%p, %p, %d): stub\n", image_base, file_name, size);
+
+    if (file_name && size)
+        file_name[0] = '\0';
+
+    return 0;
+}
