@@ -88,34 +88,42 @@ TID_LIST
 #undef XDIID
 };
 
+static HRESULT load_typelib(void)
+{
+    HRESULT hres;
+    ITypeLib *tl;
+
+    hres = LoadRegTypeLib(&LIBID_MSHTML, 4, 0, LOCALE_SYSTEM_DEFAULT, &tl);
+    if(FAILED(hres)) {
+        ERR("LoadRegTypeLib failed: %08x\n", hres);
+        return hres;
+    }
+
+    if(InterlockedCompareExchangePointer((void**)&typelib, tl, NULL))
+        ITypeLib_Release(tl);
+    return hres;
+}
+
 static HRESULT get_typeinfo(tid_t tid, ITypeInfo **typeinfo)
 {
     HRESULT hres;
 
-    if(!typelib) {
-        ITypeLib *tl;
-
-        hres = LoadRegTypeLib(&LIBID_MSHTML, 4, 0, LOCALE_SYSTEM_DEFAULT, &tl);
-        if(FAILED(hres)) {
-            ERR("LoadRegTypeLib failed: %08x\n", hres);
-            return hres;
-        }
-
-        if(InterlockedCompareExchangePointer((void**)&typelib, tl, NULL))
-            ITypeLib_Release(tl);
-    }
+    if (!typelib)
+        hres = load_typelib();
+    if (!typelib)
+        return hres;
 
     if(!typeinfos[tid]) {
-        ITypeInfo *typeinfo;
+        ITypeInfo *ti;
 
-        hres = ITypeLib_GetTypeInfoOfGuid(typelib, tid_ids[tid], &typeinfo);
+        hres = ITypeLib_GetTypeInfoOfGuid(typelib, tid_ids[tid], &ti);
         if(FAILED(hres)) {
             ERR("GetTypeInfoOfGuid(%s) failed: %08x\n", debugstr_guid(tid_ids[tid]), hres);
             return hres;
         }
 
-        if(InterlockedCompareExchangePointer((void**)(typeinfos+tid), typeinfo, NULL))
-            ITypeInfo_Release(typeinfo);
+        if(InterlockedCompareExchangePointer((void**)(typeinfos+tid), ti, NULL))
+            ITypeInfo_Release(ti);
     }
 
     *typeinfo = typeinfos[tid];
@@ -149,6 +157,21 @@ void release_typelib(void)
     ITypeLib_Release(typelib);
 }
 
+HRESULT get_htmldoc_classinfo(ITypeInfo **typeinfo)
+{
+    HRESULT hres;
+
+    if (!typelib)
+        hres = load_typelib();
+    if (!typelib)
+        return hres;
+
+    hres = ITypeLib_GetTypeInfoOfGuid(typelib, &CLSID_HTMLDocument, typeinfo);
+    if(FAILED(hres))
+        ERR("GetTypeInfoOfGuid failed: %08x\n", hres);
+    return hres;
+}
+
 static void add_func_info(dispex_data_t *data, DWORD *size, tid_t tid, const FUNCDESC *desc, ITypeInfo *dti)
 {
     HRESULT hres;
@@ -225,21 +248,19 @@ static dispex_data_t *preprocess_dispex_data(DispatchEx *This)
 
     if(!data->func_cnt) {
         heap_free(data->funcs);
+        data->name_table = NULL;
         data->funcs = NULL;
-    }else if(data->func_cnt != size) {
-        data->funcs = heap_realloc(data->funcs, data->func_cnt * sizeof(func_info_t));
+        return data;
     }
 
+
+    data->funcs = heap_realloc(data->funcs, data->func_cnt * sizeof(func_info_t));
     qsort(data->funcs, data->func_cnt, sizeof(func_info_t), dispid_cmp);
 
-    if(data->funcs) {
-        data->name_table = heap_alloc(data->func_cnt * sizeof(func_info_t*));
-        for(i=0; i < data->func_cnt; i++)
-            data->name_table[i] = data->funcs+i;
-        qsort(data->name_table, data->func_cnt, sizeof(func_info_t*), func_name_cmp);
-    }else {
-        data->name_table = NULL;
-    }
+    data->name_table = heap_alloc(data->func_cnt * sizeof(func_info_t*));
+    for(i=0; i < data->func_cnt; i++)
+        data->name_table[i] = data->funcs+i;
+    qsort(data->name_table, data->func_cnt, sizeof(func_info_t*), func_name_cmp);
 
     return data;
 }
@@ -324,30 +345,6 @@ static dispex_data_t *get_dispex_data(DispatchEx *This)
     return This->data->data;
 }
 
-HRESULT call_disp_func(IDispatch *disp, DISPPARAMS *dp)
-{
-    EXCEPINFO ei;
-    IDispatchEx *dispex;
-    VARIANT res;
-    HRESULT hres;
-
-    VariantInit(&res);
-    memset(&ei, 0, sizeof(ei));
-
-    hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
-    if(SUCCEEDED(hres)) {
-        hres = IDispatchEx_InvokeEx(dispex, 0, GetUserDefaultLCID(), DISPATCH_METHOD, dp, &res, &ei, NULL);
-        IDispatchEx_Release(dispex);
-    }else {
-        TRACE("Could not get IDispatchEx interface: %08x\n", hres);
-        hres = IDispatch_Invoke(disp, 0, &IID_NULL, GetUserDefaultLCID(), DISPATCH_METHOD,
-                dp, &res, &ei, NULL);
-    }
-
-    VariantClear(&res);
-    return hres;
-}
-
 static inline BOOL is_custom_dispid(DISPID id)
 {
     return MSHTML_DISPID_CUSTOM_MIN <= id && id <= MSHTML_DISPID_CUSTOM_MAX;
@@ -358,6 +355,17 @@ static inline BOOL is_dynamic_dispid(DISPID id)
     return DISPID_DYNPROP_0 <= id && id <= DISPID_DYNPROP_MAX;
 }
 
+static HRESULT variant_copy(VARIANT *dest, VARIANT *src)
+{
+    if(V_VT(src) == VT_BSTR && !V_BSTR(src)) {
+        V_VT(dest) = VT_BSTR;
+        V_BSTR(dest) = NULL;
+        return S_OK;
+    }
+
+    return VariantCopy(dest, src);
+}
+
 static inline dispex_dynamic_data_t *get_dynamic_data(DispatchEx *This, BOOL alloc)
 {
     return !alloc || This->dynamic_data
@@ -962,7 +970,7 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
         case DISPATCH_PROPERTYGET:
             if(prop->flags & DYNPROP_DELETED)
                 return DISP_E_UNKNOWNNAME;
-            return VariantCopy(pvarRes, &prop->var);
+            return variant_copy(pvarRes, &prop->var);
         case DISPATCH_PROPERTYPUT|DISPATCH_PROPERTYPUTREF:
         case DISPATCH_PROPERTYPUT:
             if(pdp->cArgs != 1 || (pdp->cNamedArgs == 1 && *pdp->rgdispidNamedArgs != DISPID_PROPERTYPUT)
@@ -973,7 +981,7 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
 
             TRACE("put %s\n", debugstr_variant(pdp->rgvarg));
             VariantClear(&prop->var);
-            hres = VariantCopy(&prop->var, pdp->rgvarg);
+            hres = variant_copy(&prop->var, pdp->rgvarg);
             if(FAILED(hres))
                 return hres;
 
