@@ -29,17 +29,19 @@
 #include "shlwapi.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
 #include "htmlstyle.h"
 
+WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
+
 static const WCHAR aW[]        = {'A',0};
 static const WCHAR bodyW[]     = {'B','O','D','Y',0};
 static const WCHAR embedW[]    = {'E','M','B','E','D',0};
 static const WCHAR formW[]     = {'F','O','R','M',0};
 static const WCHAR frameW[]    = {'F','R','A','M','E',0};
+static const WCHAR headW[]     = {'H','E','A','D',0};
 static const WCHAR iframeW[]   = {'I','F','R','A','M','E',0};
 static const WCHAR imgW[]      = {'I','M','G',0};
 static const WCHAR inputW[]    = {'I','N','P','U','T',0};
@@ -50,6 +52,7 @@ static const WCHAR selectW[]   = {'S','E','L','E','C','T',0};
 static const WCHAR styleW[]    = {'S','T','Y','L','E',0};
 static const WCHAR tableW[]    = {'T','A','B','L','E',0};
 static const WCHAR textareaW[] = {'T','E','X','T','A','R','E','A',0};
+static const WCHAR title_tagW[]= {'T','I','T','L','E',0};
 static const WCHAR trW[]       = {'T','R',0};
 
 typedef struct {
@@ -63,6 +66,7 @@ static const tag_desc_t tag_descs[] = {
     {embedW,     HTMLEmbedElement_Create},
     {formW,      HTMLFormElement_Create},
     {frameW,     HTMLFrameElement_Create},
+    {headW,      HTMLHeadElement_Create},
     {iframeW,    HTMLIFrame_Create},
     {imgW,       HTMLImgElement_Create},
     {inputW,     HTMLInputElement_Create},
@@ -73,6 +77,7 @@ static const tag_desc_t tag_descs[] = {
     {styleW,     HTMLStyleElement_Create},
     {tableW,     HTMLTable_Create},
     {textareaW,  HTMLTextAreaElement_Create},
+    {title_tagW, HTMLTitleElement_Create},
     {trW,        HTMLTableRow_Create}
 };
 
@@ -96,6 +101,65 @@ static const tag_desc_t *get_tag_desc(const WCHAR *tag_name)
     return NULL;
 }
 
+HRESULT replace_node_by_html(nsIDOMHTMLDocument *nsdoc, nsIDOMNode *nsnode, const WCHAR *html)
+{
+    nsIDOMDocumentFragment *nsfragment;
+    nsIDOMDocumentRange *nsdocrange;
+    nsIDOMNSRange *nsrange;
+    nsIDOMNode *nsparent;
+    nsIDOMRange *range;
+    nsAString html_str;
+    nsresult nsres;
+    HRESULT hres = S_OK;
+
+    nsres = nsIDOMHTMLDocument_QueryInterface(nsdoc, &IID_nsIDOMDocumentRange, (void**)&nsdocrange);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    nsres = nsIDOMDocumentRange_CreateRange(nsdocrange, &range);
+    nsIDOMDocumentRange_Release(nsdocrange);
+    if(NS_FAILED(nsres)) {
+        ERR("CreateRange failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMRange_QueryInterface(range, &IID_nsIDOMNSRange, (void**)&nsrange);
+    nsIDOMRange_Release(range);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMNSRange: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsAString_InitDepend(&html_str, html);
+    nsIDOMNSRange_CreateContextualFragment(nsrange, &html_str, &nsfragment);
+    nsIDOMNSRange_Release(nsrange);
+    nsAString_Finish(&html_str);
+    if(NS_FAILED(nsres)) {
+        ERR("CreateContextualFragment failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMNode_GetParentNode(nsnode, &nsparent);
+    if(NS_SUCCEEDED(nsres) && nsparent) {
+        nsIDOMNode *nstmp;
+
+        nsres = nsIDOMNode_ReplaceChild(nsparent, (nsIDOMNode*)nsfragment, nsnode, &nstmp);
+        nsIDOMNode_Release(nsparent);
+        if(NS_FAILED(nsres)) {
+            ERR("ReplaceChild failed: %08x\n", nsres);
+            hres = E_FAIL;
+        }else if(nstmp) {
+            nsIDOMNode_Release(nstmp);
+        }
+    }else {
+        ERR("GetParentNode failed: %08x\n", nsres);
+        hres = E_FAIL;
+    }
+
+    nsIDOMDocumentFragment_Release(nsfragment);
+    return hres;
+}
+
 typedef struct
 {
     DispatchEx dispex;
@@ -111,9 +175,6 @@ static inline HTMLFiltersCollection *impl_from_IHTMLFiltersCollection(IHTMLFilte
 
 static IHTMLFiltersCollection *HTMLFiltersCollection_Create(void);
 
-
-WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
-
 static inline HTMLElement *impl_from_IHTMLElement(IHTMLElement *iface)
 {
     return CONTAINING_RECORD(iface, HTMLElement, IHTMLElement_iface);
@@ -441,7 +502,7 @@ static HRESULT WINAPI HTMLElement_get_style(IHTMLElement *iface, IHTMLStyle **p)
         nsres = nsIDOMHTMLElement_QueryInterface(This->nselem, &IID_nsIDOMElementCSSInlineStyle,
                 (void**)&nselemstyle);
         if(NS_FAILED(nsres)) {
-            ERR("Coud not get nsIDOMCSSStyleDeclaration interface: %08x\n", nsres);
+            ERR("Could not get nsIDOMCSSStyleDeclaration interface: %08x\n", nsres);
             return E_FAIL;
         }
 
@@ -819,8 +880,27 @@ static HRESULT WINAPI HTMLElement_get_lang(IHTMLElement *iface, BSTR *p)
 static HRESULT WINAPI HTMLElement_get_offsetLeft(IHTMLElement *iface, LONG *p)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsIDOMNSHTMLElement *nselem;
+    PRInt32 off_left = 0;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLElement_QueryInterface(This->nselem, &IID_nsIDOMNSHTMLElement, (void**)&nselem);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMNSHTMLElement: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMNSHTMLElement_GetOffsetLeft(nselem, &off_left);
+    nsIDOMNSHTMLElement_Release(nselem);
+    if(NS_FAILED(nsres)) {
+        ERR("GetOffsetLeft failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    *p = off_left;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLElement_get_offsetTop(IHTMLElement *iface, LONG *p)
@@ -904,8 +984,41 @@ static HRESULT WINAPI HTMLElement_get_offsetHeight(IHTMLElement *iface, LONG *p)
 static HRESULT WINAPI HTMLElement_get_offsetParent(IHTMLElement *iface, IHTMLElement **p)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsIDOMNSHTMLElement *nselem;
+    nsIDOMElement *nsparent;
+    nsresult nsres;
+    HRESULT hres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLElement_QueryInterface(This->nselem, &IID_nsIDOMNSHTMLElement, (void**)&nselem);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMNSHTMLElement: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMNSHTMLElement_GetOffsetParent(nselem, &nsparent);
+    nsIDOMNSHTMLElement_Release(nselem);
+    if(NS_FAILED(nsres)) {
+        ERR("GetOffsetParent failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    if(nsparent) {
+        HTMLDOMNode *node;
+
+        hres = get_node(This->node.doc, (nsIDOMNode*)nsparent, TRUE, &node);
+        nsIDOMElement_Release(nsparent);
+        if(FAILED(hres))
+            return hres;
+
+        hres = IHTMLDOMNode_QueryInterface(&node->IHTMLDOMNode_iface, &IID_IHTMLElement, (void**)p);
+    }else {
+        *p = NULL;
+        hres = S_OK;
+    }
+
+    return hres;
 }
 
 static HRESULT WINAPI HTMLElement_put_innerHTML(IHTMLElement *iface, BSTR v)
@@ -1034,63 +1147,10 @@ static HRESULT WINAPI HTMLElement_get_innerText(IHTMLElement *iface, BSTR *p)
 static HRESULT WINAPI HTMLElement_put_outerHTML(IHTMLElement *iface, BSTR v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    nsIDOMDocumentFragment *nsfragment;
-    nsIDOMDocumentRange *nsdocrange;
-    nsIDOMNSRange *nsrange;
-    nsIDOMNode *nsparent;
-    nsIDOMRange *range;
-    nsAString html_str;
-    nsresult nsres;
-    HRESULT hres = S_OK;
 
     TRACE("(%p)->(%s)\n", This, debugstr_w(v));
 
-    nsres = nsIDOMHTMLDocument_QueryInterface(This->node.doc->nsdoc, &IID_nsIDOMDocumentRange, (void**)&nsdocrange);
-    if(NS_FAILED(nsres))
-        return E_FAIL;
-
-    nsres = nsIDOMDocumentRange_CreateRange(nsdocrange, &range);
-    nsIDOMDocumentRange_Release(nsdocrange);
-    if(NS_FAILED(nsres)) {
-        ERR("CreateRange failed: %08x\n", nsres);
-        return E_FAIL;
-    }
-
-    nsres = nsIDOMRange_QueryInterface(range, &IID_nsIDOMNSRange, (void**)&nsrange);
-    nsIDOMRange_Release(range);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMNSRange: %08x\n", nsres);
-        return E_FAIL;
-    }
-
-    nsAString_InitDepend(&html_str, v);
-    nsIDOMNSRange_CreateContextualFragment(nsrange, &html_str, &nsfragment);
-    nsIDOMNSRange_Release(nsrange);
-    nsAString_Finish(&html_str);
-    if(NS_FAILED(nsres)) {
-        ERR("CreateContextualFragment failed: %08x\n", nsres);
-        return E_FAIL;
-    }
-
-    nsres = nsIDOMNode_GetParentNode(This->node.nsnode, &nsparent);
-    if(NS_SUCCEEDED(nsres) && nsparent) {
-        nsIDOMNode *nstmp;
-
-        nsres = nsIDOMNode_ReplaceChild(nsparent, (nsIDOMNode*)nsfragment, This->node.nsnode, &nstmp);
-        nsIDOMNode_Release(nsparent);
-        if(NS_FAILED(nsres)) {
-            ERR("ReplaceChild failed: %08x\n", nsres);
-            hres = E_FAIL;
-        }else if(nstmp) {
-            nsIDOMNode_Release(nstmp);
-        }
-    }else {
-        ERR("GetParentNode failed: %08x\n", nsres);
-        hres = E_FAIL;
-    }
-
-    nsIDOMDocumentFragment_Release(nsfragment);
-    return hres;
+    return replace_node_by_html(This->node.doc->nsdoc, This->node.nsnode, v);
 }
 
 static HRESULT WINAPI HTMLElement_get_outerHTML(IHTMLElement *iface, BSTR *p)
@@ -1310,7 +1370,7 @@ static HRESULT WINAPI HTMLElement_click(IHTMLElement *iface)
 
     TRACE("(%p)\n", This);
 
-    return call_event(&This->node, EVENTID_CLICK);
+    return call_fire_event(&This->node, EVENTID_CLICK);
 }
 
 static HRESULT WINAPI HTMLElement_get_filters(IHTMLElement *iface,
