@@ -141,6 +141,7 @@ typedef struct {
 #define EVENT_FORWARDBODY        0x0004
 #define EVENT_NODEHANDLER        0x0008
 #define EVENT_CANCELABLE         0x0010
+#define EVENT_HASDEFAULTHANDLERS 0x0020
 
 static const event_info_t event_info[] = {
     {beforeunloadW,      onbeforeunloadW,      EVENTT_NONE,   DISPID_EVMETH_ONBEFOREUNLOAD,
@@ -150,7 +151,7 @@ static const event_info_t event_info[] = {
     {changeW,            onchangeW,            EVENTT_HTML,   DISPID_EVMETH_ONCHANGE,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {clickW,             onclickW,             EVENTT_MOUSE,  DISPID_EVMETH_ONCLICK,
-        EVENT_DEFAULTLISTENER|EVENT_BUBBLE|EVENT_CANCELABLE},
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE|EVENT_CANCELABLE|EVENT_HASDEFAULTHANDLERS},
     {contextmenuW,       oncontextmenuW,       EVENTT_MOUSE,  DISPID_EVMETH_ONCONTEXTMENU,
         EVENT_BUBBLE|EVENT_CANCELABLE},
     {dblclickW,          ondblclickW,          EVENTT_MOUSE,  DISPID_EVMETH_ONDBLCLICK,
@@ -434,8 +435,17 @@ static HRESULT WINAPI HTMLEventObj_get_shiftKey(IHTMLEventObj *iface, VARIANT_BO
 static HRESULT WINAPI HTMLEventObj_put_returnValue(IHTMLEventObj *iface, VARIANT v)
 {
     HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
-    FIXME("(%p)->()\n", This);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
+
+    if(V_VT(&v) != VT_BOOL) {
+        FIXME("unsupported vt %d\n", V_VT(&v));
+        return DISP_E_BADVARTYPE;
+    }
+
+    if(!V_BOOL(&v))
+        This->prevent_default = TRUE;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLEventObj_get_returnValue(IHTMLEventObj *iface, VARIANT *p)
@@ -971,8 +981,10 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     HTMLEventObj *event_obj = NULL;
     IHTMLEventObj *prev_event;
     nsIDOMNode *parent, *nsnode;
+    BOOL prevent_default = FALSE;
     HTMLDOMNode *node;
     PRUint16 node_type;
+    nsresult nsres;
     HRESULT hres;
 
     TRACE("(%p) %s\n", doc, debugstr_w(event_info[eid].name));
@@ -1044,14 +1056,42 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     if(nsnode)
         nsIDOMNode_Release(nsnode);
 
+    if(event_obj && event_obj->prevent_default)
+        prevent_default = TRUE;
     doc->basedoc.window->event = prev_event;
-
-    if(event_obj) {
-        if(event_obj->prevent_default && nsevent) {
-            TRACE("calling PreventDefault\n");
-            nsIDOMEvent_PreventDefault(nsevent);
-        }
+    if(event_obj)
         IHTMLEventObj_Release(&event_obj->IHTMLEventObj_iface);
+
+    if(!prevent_default && (event_info[eid].flags & EVENT_HASDEFAULTHANDLERS)) {
+        nsIDOMNode_AddRef(target);
+        nsnode = target;
+
+        do {
+            hres = get_node(doc, nsnode, TRUE, &node);
+            if(FAILED(hres))
+                break;
+
+            if(node && node->vtbl->handle_event) {
+                hres = node->vtbl->handle_event(node, eid, &prevent_default);
+                if(FAILED(hres) || prevent_default)
+                    break;
+            }
+
+            nsres = nsIDOMNode_GetParentNode(nsnode, &parent);
+            if(NS_FAILED(nsres))
+                break;
+
+            nsIDOMNode_Release(nsnode);
+            nsnode = parent;
+        } while(nsnode);
+
+        if(nsnode)
+            nsIDOMNode_Release(nsnode);
+    }
+
+    if(prevent_default && nsevent) {
+        TRACE("calling PreventDefault\n");
+        nsIDOMEvent_PreventDefault(nsevent);
     }
 }
 
@@ -1079,14 +1119,14 @@ HRESULT dispatch_event(HTMLDOMNode *node, const WCHAR *event_name, VARIANT *even
     return S_OK;
 }
 
-HRESULT call_event(HTMLDOMNode *node, eventid_t eid)
+HRESULT call_fire_event(HTMLDOMNode *node, eventid_t eid)
 {
     HRESULT hres;
 
-    if(node->vtbl->call_event) {
+    if(node->vtbl->fire_event) {
         BOOL handled = FALSE;
 
-        hres = node->vtbl->call_event(node, eid, &handled);
+        hres = node->vtbl->fire_event(node, eid, &handled);
         if(handled)
             return hres;
     }
@@ -1143,12 +1183,6 @@ static HRESULT ensure_nsevent_handler(HTMLDocumentNode *doc, event_target_t *eve
     if(!(event_info[eid].flags & EVENT_DEFAULTLISTENER))
         return S_OK;
 
-    if(!doc->event_vector) {
-        doc->event_vector = heap_alloc_zero(EVENTID_LAST*sizeof(BOOL));
-        if(!doc->event_vector)
-            return E_OUTOFMEMORY;
-    }
-
     if(!doc->event_vector[eid]) {
         doc->event_vector[eid] = TRUE;
         add_nsevent_listener(doc, NULL, event_info[eid].name);
@@ -1336,6 +1370,28 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMElement *nselem)
     nsAString_Finish(&attr_name_str);
 }
 
+HRESULT doc_init_events(HTMLDocumentNode *doc)
+{
+    unsigned i;
+    HRESULT hres;
+
+    doc->event_vector = heap_alloc_zero(EVENTID_LAST*sizeof(BOOL));
+    if(!doc->event_vector)
+        return E_OUTOFMEMORY;
+
+    init_nsevents(doc);
+
+    for(i=0; i < EVENTID_LAST; i++) {
+        if(event_info[i].flags & EVENT_HASDEFAULTHANDLERS) {
+            hres = ensure_nsevent_handler(doc, NULL, NULL, i);
+            if(FAILED(hres))
+                return hres;
+        }
+    }
+
+    return S_OK;
+}
+
 void release_event_target(event_target_t *event_target)
 {
     int i, j;
