@@ -35,195 +35,198 @@ WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 typedef struct {
     HTMLElement element;
 
-    const IHTMLImgElementVtbl *lpHTMLImgElementVtbl;
+    IHTMLImgElement IHTMLImgElement_iface;
 
     nsIDOMHTMLImageElement *nsimg;
 } HTMLImgElement;
 
-#define HTMLIMG(x)  ((IHTMLImgElement*)  &(x)->lpHTMLImgElementVtbl)
-
-#define HTMLIMG_THIS(iface) DEFINE_THIS(HTMLImgElement, HTMLImgElement, iface)
+static inline HTMLImgElement *impl_from_IHTMLImgElement(IHTMLImgElement *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLImgElement, IHTMLImgElement_iface);
+}
 
 static HRESULT WINAPI HTMLImgElement_QueryInterface(IHTMLImgElement *iface, REFIID riid, void **ppv)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
 
-    return IHTMLDOMNode_QueryInterface(HTMLDOMNODE(&This->element.node), riid, ppv);
+    return IHTMLDOMNode_QueryInterface(&This->element.node.IHTMLDOMNode_iface, riid, ppv);
 }
 
 static ULONG WINAPI HTMLImgElement_AddRef(IHTMLImgElement *iface)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
 
-    return IHTMLDOMNode_AddRef(HTMLDOMNODE(&This->element.node));
+    return IHTMLDOMNode_AddRef(&This->element.node.IHTMLDOMNode_iface);
 }
 
 static ULONG WINAPI HTMLImgElement_Release(IHTMLImgElement *iface)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
 
-    return IHTMLDOMNode_Release(HTMLDOMNODE(&This->element.node));
+    return IHTMLDOMNode_Release(&This->element.node.IHTMLDOMNode_iface);
 }
 
 static HRESULT WINAPI HTMLImgElement_GetTypeInfoCount(IHTMLImgElement *iface, UINT *pctinfo)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
-    return IDispatchEx_GetTypeInfoCount(DISPATCHEX(&This->element.node.dispex), pctinfo);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
+    return IDispatchEx_GetTypeInfoCount(&This->element.node.dispex.IDispatchEx_iface, pctinfo);
 }
 
 static HRESULT WINAPI HTMLImgElement_GetTypeInfo(IHTMLImgElement *iface, UINT iTInfo,
                                               LCID lcid, ITypeInfo **ppTInfo)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
-    return IDispatchEx_GetTypeInfo(DISPATCHEX(&This->element.node.dispex), iTInfo, lcid, ppTInfo);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
+    return IDispatchEx_GetTypeInfo(&This->element.node.dispex.IDispatchEx_iface, iTInfo, lcid,
+            ppTInfo);
 }
 
 static HRESULT WINAPI HTMLImgElement_GetIDsOfNames(IHTMLImgElement *iface, REFIID riid,
                                                 LPOLESTR *rgszNames, UINT cNames,
                                                 LCID lcid, DISPID *rgDispId)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
-    return IDispatchEx_GetIDsOfNames(DISPATCHEX(&This->element.node.dispex), riid, rgszNames, cNames, lcid, rgDispId);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
+    return IDispatchEx_GetIDsOfNames(&This->element.node.dispex.IDispatchEx_iface, riid, rgszNames,
+            cNames, lcid, rgDispId);
 }
 
 static HRESULT WINAPI HTMLImgElement_Invoke(IHTMLImgElement *iface, DISPID dispIdMember,
                             REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
                             VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
-    return IDispatchEx_Invoke(DISPATCHEX(&This->element.node.dispex), dispIdMember, riid, lcid, wFlags, pDispParams,
-            pVarResult, pExcepInfo, puArgErr);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
+    return IDispatchEx_Invoke(&This->element.node.dispex.IDispatchEx_iface, dispIdMember, riid,
+            lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 }
 
 static HRESULT WINAPI HTMLImgElement_put_isMap(IHTMLImgElement *iface, VARIANT_BOOL v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%x)\n", This, v);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_isMap(IHTMLImgElement *iface, VARIANT_BOOL *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_useMap(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_useMap(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_mimeType(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_fileSize(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_fileCreatedDate(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_fileModifiedDate(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_fileUpdatedDate(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_protocol(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_href(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_nameProp(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_border(IHTMLImgElement *iface, VARIANT v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->()\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_border(IHTMLImgElement *iface, VARIANT *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_vspace(IHTMLImgElement *iface, LONG v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%d)\n", This, v);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_vspace(IHTMLImgElement *iface, LONG *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_hspace(IHTMLImgElement *iface, LONG v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%d)\n", This, v);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_hspace(IHTMLImgElement *iface, LONG *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_alt(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     nsAString alt_str;
     nsresult nsres;
 
@@ -240,7 +243,7 @@ static HRESULT WINAPI HTMLImgElement_put_alt(IHTMLImgElement *iface, BSTR v)
 
 static HRESULT WINAPI HTMLImgElement_get_alt(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     nsAString alt_str;
     nsresult nsres;
 
@@ -263,7 +266,7 @@ static HRESULT WINAPI HTMLImgElement_get_alt(IHTMLImgElement *iface, BSTR *p)
 
 static HRESULT WINAPI HTMLImgElement_put_src(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     nsAString src_str;
     nsresult nsres;
 
@@ -280,115 +283,125 @@ static HRESULT WINAPI HTMLImgElement_put_src(IHTMLImgElement *iface, BSTR v)
 
 static HRESULT WINAPI HTMLImgElement_get_src(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     const PRUnichar *src;
     nsAString src_str;
     nsresult nsres;
-    HRESULT hres;
+    HRESULT hres = S_OK;
+
+    static const WCHAR blockedW[] = {'B','L','O','C','K','E','D',':',':',0};
 
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&src_str, NULL);
     nsres = nsIDOMHTMLImageElement_GetSrc(This->nsimg, &src_str);
-    if(NS_FAILED(nsres)) {
+    if(NS_SUCCEEDED(nsres)) {
+        nsAString_GetData(&src_str, &src);
+
+        if(!strncmpiW(src, blockedW, sizeof(blockedW)/sizeof(WCHAR)-1)) {
+            TRACE("returning BLOCKED::\n");
+            *p = SysAllocString(blockedW);
+            if(!*p)
+                hres = E_OUTOFMEMORY;
+        }else {
+            hres = nsuri_to_url(src, TRUE, p);
+        }
+    }else {
         ERR("GetSrc failed: %08x\n", nsres);
-        return E_FAIL;
+        hres = E_FAIL;
     }
 
-    nsAString_GetData(&src_str, &src);
-    hres = nsuri_to_url(src, TRUE, p);
     nsAString_Finish(&src_str);
-
     return hres;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_lowsrc(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_lowsrc(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_vrml(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_vrml(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_dynsrc(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_dynsrc(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_readyState(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_complete(IHTMLImgElement *iface, VARIANT_BOOL *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_loop(IHTMLImgElement *iface, VARIANT v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->()\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_loop(IHTMLImgElement *iface, VARIANT *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_align(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_align(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_onload(IHTMLImgElement *iface, VARIANT v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
 
     TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
@@ -397,7 +410,7 @@ static HRESULT WINAPI HTMLImgElement_put_onload(IHTMLImgElement *iface, VARIANT
 
 static HRESULT WINAPI HTMLImgElement_get_onload(IHTMLImgElement *iface, VARIANT *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -406,42 +419,42 @@ static HRESULT WINAPI HTMLImgElement_get_onload(IHTMLImgElement *iface, VARIANT
 
 static HRESULT WINAPI HTMLImgElement_put_onerror(IHTMLImgElement *iface, VARIANT v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->()\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_onerror(IHTMLImgElement *iface, VARIANT *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_onabort(IHTMLImgElement *iface, VARIANT v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->()\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_onabort(IHTMLImgElement *iface, VARIANT *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_put_name(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_name(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     nsAString strName;
     nsresult nsres;
 
@@ -464,7 +477,7 @@ static HRESULT WINAPI HTMLImgElement_get_name(IHTMLImgElement *iface, BSTR *p)
 
 static HRESULT WINAPI HTMLImgElement_put_width(IHTMLImgElement *iface, LONG v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     nsresult nsres;
 
     TRACE("(%p)->(%d)\n", This, v);
@@ -480,7 +493,7 @@ static HRESULT WINAPI HTMLImgElement_put_width(IHTMLImgElement *iface, LONG v)
 
 static HRESULT WINAPI HTMLImgElement_get_width(IHTMLImgElement *iface, LONG *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     PRInt32 width;
     nsresult nsres;
 
@@ -498,7 +511,7 @@ static HRESULT WINAPI HTMLImgElement_get_width(IHTMLImgElement *iface, LONG *p)
 
 static HRESULT WINAPI HTMLImgElement_put_height(IHTMLImgElement *iface, LONG v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     nsresult nsres;
 
     TRACE("(%p)->(%d)\n", This, v);
@@ -514,7 +527,7 @@ static HRESULT WINAPI HTMLImgElement_put_height(IHTMLImgElement *iface, LONG v)
 
 static HRESULT WINAPI HTMLImgElement_get_height(IHTMLImgElement *iface, LONG *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     PRInt32 height;
     nsresult nsres;
 
@@ -532,20 +545,18 @@ static HRESULT WINAPI HTMLImgElement_get_height(IHTMLImgElement *iface, LONG *p)
 
 static HRESULT WINAPI HTMLImgElement_put_start(IHTMLImgElement *iface, BSTR v)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->()\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLImgElement_get_start(IHTMLImgElement *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_THIS(iface);
+    HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
     FIXME("(%p)->(%p)\n", This, p);
     return E_NOTIMPL;
 }
 
-#undef HTMLIMG_THIS
-
 static const IHTMLImgElementVtbl HTMLImgElementVtbl = {
     HTMLImgElement_QueryInterface,
     HTMLImgElement_AddRef,
@@ -604,17 +615,20 @@ static const IHTMLImgElementVtbl HTMLImgElementVtbl = {
     HTMLImgElement_get_start
 };
 
-#define HTMLIMG_NODE_THIS(iface) DEFINE_THIS2(HTMLImgElement, element.node, iface)
+static inline HTMLImgElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLImgElement, element.node);
+}
 
 static HRESULT HTMLImgElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
 {
-    HTMLImgElement *This = HTMLIMG_NODE_THIS(iface);
+    HTMLImgElement *This = impl_from_HTMLDOMNode(iface);
 
     *ppv = NULL;
 
     if(IsEqualGUID(&IID_IHTMLImgElement, riid)) {
         TRACE("(%p)->(IID_IHTMLImgElement %p)\n", This, ppv);
-        *ppv = HTMLIMG(This);
+        *ppv = &This->IHTMLImgElement_iface;
     }else {
         return HTMLElement_QI(&This->element.node, riid, ppv);
     }
@@ -625,7 +639,7 @@ static HRESULT HTMLImgElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
 
 static void HTMLImgElement_destructor(HTMLDOMNode *iface)
 {
-    HTMLImgElement *This = HTMLIMG_NODE_THIS(iface);
+    HTMLImgElement *This = impl_from_HTMLDOMNode(iface);
 
     if(This->nsimg)
         nsIDOMHTMLImageElement_Release(This->nsimg);
@@ -635,13 +649,11 @@ static void HTMLImgElement_destructor(HTMLDOMNode *iface)
 
 static HRESULT HTMLImgElement_get_readystate(HTMLDOMNode *iface, BSTR *p)
 {
-    HTMLImgElement *This = HTMLIMG_NODE_THIS(iface);
+    HTMLImgElement *This = impl_from_HTMLDOMNode(iface);
 
-    return IHTMLImgElement_get_readyState(HTMLIMG(This), p);
+    return IHTMLImgElement_get_readyState(&This->IHTMLImgElement_iface, p);
 }
 
-#undef HTMLIMG_NODE_THIS
-
 static const NodeImplVtbl HTMLImgElementImplVtbl = {
     HTMLImgElement_QI,
     HTMLImgElement_destructor,
@@ -651,6 +663,7 @@ static const NodeImplVtbl HTMLImgElementImplVtbl = {
     NULL,
     NULL,
     NULL,
+    NULL,
     HTMLImgElement_get_readystate
 };
 
@@ -666,38 +679,49 @@ static dispex_static_data_t HTMLImgElement_dispex = {
     HTMLImgElement_iface_tids
 };
 
-HTMLElement *HTMLImgElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem)
+HRESULT HTMLImgElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
 {
-    HTMLImgElement *ret = heap_alloc_zero(sizeof(HTMLImgElement));
+    HTMLImgElement *ret;
     nsresult nsres;
 
-    ret->lpHTMLImgElementVtbl = &HTMLImgElementVtbl;
+    ret = heap_alloc_zero(sizeof(HTMLImgElement));
+    if(!ret)
+        return E_OUTOFMEMORY;
+
+    ret->IHTMLImgElement_iface.lpVtbl = &HTMLImgElementVtbl;
     ret->element.node.vtbl = &HTMLImgElementImplVtbl;
 
     nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLImageElement, (void**)&ret->nsimg);
-    if(NS_FAILED(nsres))
+    if(NS_FAILED(nsres)) {
         ERR("Could not get nsIDOMHTMLImageElement: %08x\n", nsres);
+        heap_free(ret);
+        return E_FAIL;
+    }
 
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLImgElement_dispex);
 
-    return &ret->element;
+    *elem = &ret->element;
+    return S_OK;
 }
 
-#define HTMLIMGFACTORY_THIS(iface) DEFINE_THIS(HTMLImageElementFactory, HTMLImageElementFactory, iface)
+static inline HTMLImageElementFactory *impl_from_IHTMLImageElementFactory(IHTMLImageElementFactory *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLImageElementFactory, IHTMLImageElementFactory_iface);
+}
 
 static HRESULT WINAPI HTMLImageElementFactory_QueryInterface(IHTMLImageElementFactory *iface,
         REFIID riid, void **ppv)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
 
     *ppv = NULL;
 
     if(IsEqualGUID(&IID_IUnknown, riid)) {
         TRACE("(%p)->(IID_Unknown %p)\n", This, ppv);
-        *ppv = HTMLIMGFACTORY(This);
+        *ppv = &This->IHTMLImageElementFactory_iface;
     }else if(IsEqualGUID(&IID_IHTMLImageElementFactory, riid)) {
         TRACE("(%p)->(IID_IHTMLImageElementFactory %p)\n", This, ppv);
-        *ppv = HTMLIMGFACTORY(This);
+        *ppv = &This->IHTMLImageElementFactory_iface;
     }else if(dispex_query_interface(&This->dispex, riid, ppv))
         return *ppv ? S_OK : E_NOINTERFACE;
 
@@ -712,7 +736,7 @@ static HRESULT WINAPI HTMLImageElementFactory_QueryInterface(IHTMLImageElementFa
 
 static ULONG WINAPI HTMLImageElementFactory_AddRef(IHTMLImageElementFactory *iface)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -722,7 +746,7 @@ static ULONG WINAPI HTMLImageElementFactory_AddRef(IHTMLImageElementFactory *ifa
 
 static ULONG WINAPI HTMLImageElementFactory_Release(IHTMLImageElementFactory *iface)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -736,7 +760,7 @@ static ULONG WINAPI HTMLImageElementFactory_Release(IHTMLImageElementFactory *if
 static HRESULT WINAPI HTMLImageElementFactory_GetTypeInfoCount(IHTMLImageElementFactory *iface,
         UINT *pctinfo)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     FIXME("(%p)->(%p)\n", This, pctinfo);
     return E_NOTIMPL;
 }
@@ -744,7 +768,7 @@ static HRESULT WINAPI HTMLImageElementFactory_GetTypeInfoCount(IHTMLImageElement
 static HRESULT WINAPI HTMLImageElementFactory_GetTypeInfo(IHTMLImageElementFactory *iface,
         UINT iTInfo, LCID lcid, ITypeInfo **ppTInfo)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     FIXME("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
     return E_NOTIMPL;
 }
@@ -753,7 +777,7 @@ static HRESULT WINAPI HTMLImageElementFactory_GetIDsOfNames(IHTMLImageElementFac
         REFIID riid, LPOLESTR *rgszNames, UINT cNames, LCID lcid,
         DISPID *rgDispId)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     FIXME("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames,
             cNames, lcid, rgDispId);
     return E_NOTIMPL;
@@ -764,7 +788,7 @@ static HRESULT WINAPI HTMLImageElementFactory_Invoke(IHTMLImageElementFactory *i
         DISPPARAMS *pDispParams, VARIANT *pVarResult, EXCEPINFO *pExcepInfo,
         UINT *puArgErr)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     FIXME("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
             lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
     return E_NOTIMPL;
@@ -797,7 +821,7 @@ static LONG var_to_size(const VARIANT *v)
 static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *iface,
         VARIANT width, VARIANT height, IHTMLImgElement **img_elem)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
     IHTMLImgElement *img;
     HTMLElement *elem;
     nsIDOMHTMLElement *nselem;
@@ -820,20 +844,20 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
     if(FAILED(hres))
         return hres;
 
-    elem = HTMLElement_Create(This->window->doc, (nsIDOMNode*)nselem, FALSE);
-    if(!elem) {
+    hres = HTMLElement_Create(This->window->doc, (nsIDOMNode*)nselem, FALSE, &elem);
+    nsIDOMHTMLElement_Release(nselem);
+    if(FAILED(hres)) {
         ERR("HTMLElement_Create failed\n");
-        return E_FAIL;
+        return hres;
     }
 
-    hres = IHTMLElement_QueryInterface(HTMLELEM(elem), &IID_IHTMLImgElement, (void**)&img);
+    hres = IHTMLElement_QueryInterface(&elem->IHTMLElement_iface, &IID_IHTMLImgElement,
+            (void**)&img);
     if(FAILED(hres)) {
         ERR("IHTMLElement_QueryInterface failed: 0x%08x\n", hres);
         return hres;
     }
 
-    nsIDOMHTMLElement_Release(nselem);
-
     l = var_to_size(&width);
     if(l)
         IHTMLImgElement_put_width(img, l);
@@ -845,11 +869,27 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
     return S_OK;
 }
 
-static HRESULT HTMLImageElementFactory_value(IUnknown *iface, LCID lcid,
+static const IHTMLImageElementFactoryVtbl HTMLImageElementFactoryVtbl = {
+    HTMLImageElementFactory_QueryInterface,
+    HTMLImageElementFactory_AddRef,
+    HTMLImageElementFactory_Release,
+    HTMLImageElementFactory_GetTypeInfoCount,
+    HTMLImageElementFactory_GetTypeInfo,
+    HTMLImageElementFactory_GetIDsOfNames,
+    HTMLImageElementFactory_Invoke,
+    HTMLImageElementFactory_create
+};
+
+static inline HTMLImageElementFactory *impl_from_DispatchEx(DispatchEx *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLImageElementFactory, dispex);
+}
+
+static HRESULT HTMLImageElementFactory_value(DispatchEx *dispex, LCID lcid,
         WORD flags, DISPPARAMS *params, VARIANT *res, EXCEPINFO *ei,
         IServiceProvider *caller)
 {
-    HTMLImageElementFactory *This = HTMLIMGFACTORY_THIS(iface);
+    HTMLImageElementFactory *This = impl_from_DispatchEx(dispex);
     IHTMLImgElement *img;
     VARIANT empty, *width, *height;
     HRESULT hres;
@@ -862,7 +902,8 @@ static HRESULT HTMLImageElementFactory_value(IUnknown *iface, LCID lcid,
     width = argc >= 1 ? params->rgvarg + (params->cArgs - 1) : &empty;
     height = argc >= 2 ? params->rgvarg + (params->cArgs - 2) : &empty;
 
-    hres = IHTMLImageElementFactory_create(HTMLIMGFACTORY(This), *width, *height, &img);
+    hres = IHTMLImageElementFactory_create(&This->IHTMLImageElementFactory_iface, *width, *height,
+            &img);
     if(FAILED(hres))
         return hres;
 
@@ -872,19 +913,6 @@ static HRESULT HTMLImageElementFactory_value(IUnknown *iface, LCID lcid,
     return S_OK;
 }
 
-#undef HTMLIMGFACTORY_THIS
-
-static const IHTMLImageElementFactoryVtbl HTMLImageElementFactoryVtbl = {
-    HTMLImageElementFactory_QueryInterface,
-    HTMLImageElementFactory_AddRef,
-    HTMLImageElementFactory_Release,
-    HTMLImageElementFactory_GetTypeInfoCount,
-    HTMLImageElementFactory_GetTypeInfo,
-    HTMLImageElementFactory_GetIDsOfNames,
-    HTMLImageElementFactory_Invoke,
-    HTMLImageElementFactory_create
-};
-
 static const tid_t HTMLImageElementFactory_iface_tids[] = {
     IHTMLImageElementFactory_tid,
     0
@@ -909,11 +937,12 @@ HTMLImageElementFactory *HTMLImageElementFactory_Create(HTMLWindow *window)
 
     ret = heap_alloc(sizeof(HTMLImageElementFactory));
 
-    ret->lpHTMLImageElementFactoryVtbl = &HTMLImageElementFactoryVtbl;
+    ret->IHTMLImageElementFactory_iface.lpVtbl = &HTMLImageElementFactoryVtbl;
     ret->ref = 1;
     ret->window = window;
 
-    init_dispex(&ret->dispex, (IUnknown*)HTMLIMGFACTORY(ret), &HTMLImageElementFactory_dispex);
+    init_dispex(&ret->dispex, (IUnknown*)&ret->IHTMLImageElementFactory_iface,
+            &HTMLImageElementFactory_dispex);
 
     return ret;
 }
