@@ -31,7 +31,6 @@
 #include "htmlstyle.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -41,6 +40,8 @@ static const WCHAR attrBackgroundColor[] =
     {'b','a','c','k','g','r','o','u','n','d','-','c','o','l','o','r',0};
 static const WCHAR attrBackgroundImage[] =
     {'b','a','c','k','g','r','o','u','n','d','-','i','m','a','g','e',0};
+static const WCHAR attrBackgroundPosition[] =
+    {'b','a','c','k','g','r','o','u','n','d','-','p','o','s','i','t','i','o','n',0};
 static const WCHAR attrBackgroundPositionX[] =
     {'b','a','c','k','g','r','o','u','n','d','-','p','o','s','i','t','i','o','n','-','x',0};
 static const WCHAR attrBackgroundPositionY[] =
@@ -89,10 +90,14 @@ static const WCHAR attrBorderWidth[] =
     {'b','o','r','d','e','r','-','w','i','d','t','h',0};
 static const WCHAR attrBottom[] =
     {'b','o','t','t','o','m',0};
+static const WCHAR attrClip[] =
+    {'c','l','i','p',0};
 static const WCHAR attrColor[] =
     {'c','o','l','o','r',0};
 static const WCHAR attrCursor[] =
     {'c','u','r','s','o','r',0};
+static const WCHAR attrDirection[] =
+    {'d','i','r','e','c','t','i','o','n',0};
 static const WCHAR attrDisplay[] =
     {'d','i','s','p','l','a','y',0};
 static const WCHAR attrFilter[] =
@@ -139,6 +144,10 @@ static const WCHAR attrPaddingRight[] =
     {'p','a','d','d','i','n','g','-','r','i','g','h','t',0};
 static const WCHAR attrPaddingTop[] =
     {'p','a','d','d','i','n','g','-','t','o','p',0};
+static const WCHAR attrPageBreakAfter[] =
+    {'p','a','g','e','-','b','r','e','a','k','-','a','f','t','e','r',0};
+static const WCHAR attrPageBreakBefore[] =
+    {'p','a','g','e','-','b','r','e','a','k','-','b','e','f','o','r','e',0};
 static const WCHAR attrPosition[] =
     {'p','o','s','i','t','i','o','n',0};
 static const WCHAR attrRight[] =
@@ -171,6 +180,7 @@ static const struct{
     {attrBackground,           DISPID_IHTMLSTYLE_BACKGROUND},
     {attrBackgroundColor,      DISPID_IHTMLSTYLE_BACKGROUNDCOLOR},
     {attrBackgroundImage,      DISPID_IHTMLSTYLE_BACKGROUNDIMAGE},
+    {attrBackgroundPosition,   DISPID_IHTMLSTYLE_BACKGROUNDPOSITION},
     {attrBackgroundPositionX,  DISPID_IHTMLSTYLE_BACKGROUNDPOSITIONX},
     {attrBackgroundPositionY,  DISPID_IHTMLSTYLE_BACKGROUNDPOSITIONY},
     {attrBackgroundRepeat,     DISPID_IHTMLSTYLE_BACKGROUNDREPEAT},
@@ -195,8 +205,10 @@ static const struct{
     {attrBorderTopWidth,       DISPID_IHTMLSTYLE_BORDERTOPWIDTH},
     {attrBorderWidth,          DISPID_IHTMLSTYLE_BORDERWIDTH},
     {attrBottom,               DISPID_IHTMLSTYLE2_BOTTOM},
+    {attrClip,                 DISPID_IHTMLSTYLE_CLIP},
     {attrColor,                DISPID_IHTMLSTYLE_COLOR},
     {attrCursor,               DISPID_IHTMLSTYLE_CURSOR},
+    {attrDirection,            DISPID_IHTMLSTYLE2_DIRECTION},
     {attrDisplay,              DISPID_IHTMLSTYLE_DISPLAY},
     {attrFilter,               DISPID_IHTMLSTYLE_FILTER},
     {attrFontFamily,           DISPID_IHTMLSTYLE_FONTFAMILY},
@@ -220,6 +232,8 @@ static const struct{
     {attrPaddingLeft,          DISPID_IHTMLSTYLE_PADDINGLEFT},
     {attrPaddingRight,         DISPID_IHTMLSTYLE_PADDINGRIGHT},
     {attrPaddingTop,           DISPID_IHTMLSTYLE_PADDINGTOP},
+    {attrPageBreakAfter,       DISPID_IHTMLSTYLE_PAGEBREAKAFTER},
+    {attrPageBreakBefore,      DISPID_IHTMLSTYLE_PAGEBREAKBEFORE},
     {attrPosition,             DISPID_IHTMLSTYLE2_POSITION},
     {attrRight,                DISPID_IHTMLSTYLE2_RIGHT},
     {attrTextAlign,            DISPID_IHTMLSTYLE_TEXTALIGN},
@@ -311,8 +325,6 @@ HRESULT set_nsstyle_attr(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, LPCW
     LPWSTR val = NULL;
     nsresult nsres;
 
-    static const PRUnichar wszEmpty[] = {0};
-
     if(flags & ATTR_FIX_PX)
         val = fix_px_value(value);
     if(flags & ATTR_FIX_URL)
@@ -320,8 +332,7 @@ HRESULT set_nsstyle_attr(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, LPCW
 
     nsAString_InitDepend(&str_name, style_tbl[sid].name);
     nsAString_InitDepend(&str_value, val ? val : value);
-    nsAString_InitDepend(&str_empty, wszEmpty);
-    heap_free(val);
+    nsAString_InitDepend(&str_empty, emptyW);
 
     nsres = nsIDOMCSSStyleDeclaration_SetProperty(nsstyle, &str_name, &str_value, &str_empty);
     if(NS_FAILED(nsres))
@@ -330,42 +341,58 @@ HRESULT set_nsstyle_attr(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, LPCW
     nsAString_Finish(&str_name);
     nsAString_Finish(&str_value);
     nsAString_Finish(&str_empty);
+    heap_free(val);
 
     return S_OK;
 }
 
-HRESULT set_nsstyle_attr_var(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, VARIANT *value, DWORD flags)
+static HRESULT var_to_styleval(const VARIANT *v, WCHAR *buf, DWORD flags, const WCHAR **ret)
 {
-    switch(V_VT(value)) {
+    switch(V_VT(v)) {
     case VT_NULL:
-        return set_nsstyle_attr(nsstyle, sid, emptyW, flags);
+        *ret = emptyW;
+        return S_OK;
 
     case VT_BSTR:
-        return set_nsstyle_attr(nsstyle, sid, V_BSTR(value), flags);
+        *ret = V_BSTR(v);
+        return S_OK;
 
     case VT_BSTR|VT_BYREF:
-        return set_nsstyle_attr(nsstyle, sid, *V_BSTRREF(value), flags);
+        *ret = *V_BSTRREF(v);
+        return S_OK;
 
     case VT_I4: {
-        WCHAR str[14];
-
-        static const WCHAR format[] = {'%','d',0};
-        static const WCHAR px_format[] = {'%','d','p','x',0};
-        static const WCHAR hex_format[] = {'#','%','0','6','x',0};
+        static const WCHAR formatW[] = {'%','d',0};
+        static const WCHAR hex_formatW[] = {'#','%','0','6','x',0};
 
         if(flags & ATTR_HEX_INT)
-            wsprintfW(str, hex_format, V_I4(value));
+            wsprintfW(buf, hex_formatW, V_I4(v));
+        else if(flags & ATTR_FIX_PX)
+            wsprintfW(buf, px_formatW, V_I4(v));
         else
-            wsprintfW(str, flags&ATTR_FIX_PX ? px_format : format, V_I4(value));
-        return set_nsstyle_attr(nsstyle, sid, str, flags & ~ATTR_FIX_PX);
+            wsprintfW(buf, formatW, V_I4(v));
+
+        *ret = buf;
+        return S_OK;
     }
     default:
-        FIXME("not implemented vt %d\n", V_VT(value));
+        FIXME("not implemented vt %d\n", V_VT(v));
         return E_NOTIMPL;
 
     }
+}
 
-    return S_OK;
+HRESULT set_nsstyle_attr_var(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, VARIANT *value, DWORD flags)
+{
+    const WCHAR *val;
+    WCHAR buf[14];
+    HRESULT hres;
+
+    hres = var_to_styleval(value, buf, flags, &val);
+    if(FAILED(hres))
+        return hres;
+
+    return set_nsstyle_attr(nsstyle, sid, val, flags);
 }
 
 static inline HRESULT set_style_attr(HTMLStyle *This, styleid_t sid, LPCWSTR value, DWORD flags)
@@ -391,22 +418,68 @@ static HRESULT get_nsstyle_attr_nsval(nsIDOMCSSStyleDeclaration *nsstyle, stylei
     return NS_OK;
 }
 
-HRESULT get_nsstyle_attr(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, BSTR *p)
+static HRESULT nsstyle_to_bstr(const WCHAR *val, DWORD flags, BSTR *p)
+{
+    BSTR ret;
+    DWORD len;
+
+    if(!*val) {
+        *p = NULL;
+        return S_OK;
+    }
+
+    ret = SysAllocString(val);
+    if(!ret)
+        return E_OUTOFMEMORY;
+
+    len = SysStringLen(ret);
+
+    if(flags & ATTR_REMOVE_COMMA) {
+        DWORD new_len = len;
+        WCHAR *ptr, *ptr2;
+
+        for(ptr = ret; (ptr = strchrW(ptr, ',')); ptr++)
+            new_len--;
+
+        if(new_len != len) {
+            BSTR new_ret;
+
+            new_ret = SysAllocStringLen(NULL, new_len);
+            if(!new_ret) {
+                SysFreeString(ret);
+                return E_OUTOFMEMORY;
+            }
+
+            for(ptr2 = new_ret, ptr = ret; *ptr; ptr++) {
+                if(*ptr != ',')
+                    *ptr2++ = *ptr;
+            }
+
+            SysFreeString(ret);
+            ret = new_ret;
+        }
+    }
+
+    *p = ret;
+    return S_OK;
+}
+
+HRESULT get_nsstyle_attr(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, BSTR *p, DWORD flags)
 {
     nsAString str_value;
     const PRUnichar *value;
+    HRESULT hres;
 
     nsAString_Init(&str_value, NULL);
 
     get_nsstyle_attr_nsval(nsstyle, sid, &str_value);
 
     nsAString_GetData(&str_value, &value);
-    *p = *value ? SysAllocString(value) : NULL;
-
+    hres = nsstyle_to_bstr(value, flags, p);
     nsAString_Finish(&str_value);
 
     TRACE("%s -> %s\n", debugstr_w(style_tbl[sid].name), debugstr_w(*p));
-    return S_OK;
+    return hres;
 }
 
 HRESULT get_nsstyle_attr_var(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid, VARIANT *p, DWORD flags)
@@ -414,6 +487,7 @@ HRESULT get_nsstyle_attr_var(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid,
     nsAString str_value;
     const PRUnichar *value;
     BOOL set = FALSE;
+    HRESULT hres = S_OK;
 
     nsAString_Init(&str_value, NULL);
 
@@ -442,16 +516,13 @@ HRESULT get_nsstyle_attr_var(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid,
     }
 
     if(!set) {
-        BSTR str = NULL;
+        BSTR str;
 
-        if(*value) {
-            str = SysAllocString(value);
-            if(!str)
-                return E_OUTOFMEMORY;
+        hres = nsstyle_to_bstr(value, flags, &str);
+        if(SUCCEEDED(hres)) {
+            V_VT(p) = VT_BSTR;
+            V_BSTR(p) = str;
         }
-
-        V_VT(p) = VT_BSTR;
-        V_BSTR(p) = str;
     }
 
     nsAString_Finish(&str_value);
@@ -462,7 +533,7 @@ HRESULT get_nsstyle_attr_var(nsIDOMCSSStyleDeclaration *nsstyle, styleid_t sid,
 
 static inline HRESULT get_style_attr(HTMLStyle *This, styleid_t sid, BSTR *p)
 {
-    return get_nsstyle_attr(This->nsstyle, sid, p);
+    return get_nsstyle_attr(This->nsstyle, sid, p, 0);
 }
 
 static HRESULT check_style_attr_value(HTMLStyle *This, styleid_t sid, LPCWSTR exval, VARIANT_BOOL *p)
@@ -584,6 +655,12 @@ static HRESULT WINAPI HTMLStyle_QueryInterface(IHTMLStyle *iface, REFIID riid, v
     }else if(IsEqualGUID(&IID_IHTMLStyle4, riid)) {
         TRACE("(%p)->(IID_IHTMLStyle4 %p)\n", This, ppv);
         *ppv = &This->IHTMLStyle4_iface;
+    }else if(IsEqualGUID(&IID_IHTMLStyle5, riid)) {
+        TRACE("(%p)->(IID_IHTMLStyle5 %p)\n", This, ppv);
+        *ppv = &This->IHTMLStyle5_iface;
+    }else if(IsEqualGUID(&IID_IHTMLStyle6, riid)) {
+        TRACE("(%p)->(IID_IHTMLStyle6 %p)\n", This, ppv);
+        *ppv = &This->IHTMLStyle6_iface;
     }else if(dispex_query_interface(&This->dispex, riid, ppv)) {
         return *ppv ? S_OK : E_NOINTERFACE;
     }
@@ -617,6 +694,7 @@ static ULONG WINAPI HTMLStyle_Release(IHTMLStyle *iface)
     if(!ref) {
         if(This->nsstyle)
             nsIDOMCSSStyleDeclaration_Release(This->nsstyle);
+        heap_free(This->filter);
         release_dispex(&This->dispex);
         heap_free(This);
     }
@@ -952,43 +1030,203 @@ static HRESULT WINAPI HTMLStyle_get_backgroundAttachment(IHTMLStyle *iface, BSTR
 static HRESULT WINAPI HTMLStyle_put_backgroundPosition(IHTMLStyle *iface, BSTR v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    return set_style_attr(This, STYLEID_BACKGROUND_POSITION, v, 0);
 }
 
 static HRESULT WINAPI HTMLStyle_get_backgroundPosition(IHTMLStyle *iface, BSTR *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_style_attr(This, STYLEID_BACKGROUND_POSITION, p);
 }
 
 static HRESULT WINAPI HTMLStyle_put_backgroundPositionX(IHTMLStyle *iface, VARIANT v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
+    WCHAR buf[14], *pos_val;
+    nsAString pos_str;
+    const WCHAR *val;
+    DWORD val_len;
+    HRESULT hres;
+
     TRACE("(%p)->(v%d)\n", This, V_VT(&v));
-    return set_nsstyle_attr_var(This->nsstyle, STYLEID_BACKGROUND_POSITION_X, &v, 0);
+
+    hres = var_to_styleval(&v, buf, ATTR_FIX_PX, &val);
+    if(FAILED(hres))
+        return hres;
+
+    val_len = val ? strlenW(val) : 0;
+
+    nsAString_Init(&pos_str, NULL);
+    hres = get_nsstyle_attr_nsval(This->nsstyle, STYLEID_BACKGROUND_POSITION, &pos_str);
+    if(SUCCEEDED(hres)) {
+        const PRUnichar *pos, *posy;
+        DWORD posy_len;
+
+        nsAString_GetData(&pos_str, &pos);
+        posy = strchrW(pos, ' ');
+        if(!posy) {
+            static const WCHAR zero_pxW[] = {' ','0','p','x',0};
+
+            TRACE("no space in %s\n", debugstr_w(pos));
+            posy = zero_pxW;
+        }
+
+        posy_len = strlenW(posy);
+        pos_val = heap_alloc((val_len+posy_len+1)*sizeof(WCHAR));
+        if(pos_val) {
+            if(val_len)
+                memcpy(pos_val, val, val_len*sizeof(WCHAR));
+            if(posy_len)
+                memcpy(pos_val+val_len, posy, posy_len*sizeof(WCHAR));
+            pos_val[val_len+posy_len] = 0;
+        }else {
+            hres = E_OUTOFMEMORY;
+        }
+    }
+    nsAString_Finish(&pos_str);
+    if(FAILED(hres))
+        return hres;
+
+    TRACE("setting position to %s\n", debugstr_w(pos_val));
+    hres = set_nsstyle_attr(This->nsstyle, STYLEID_BACKGROUND_POSITION, pos_val, ATTR_FIX_PX);
+    heap_free(pos_val);
+    return hres;
 }
 
 static HRESULT WINAPI HTMLStyle_get_backgroundPositionX(IHTMLStyle *iface, VARIANT *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
+    nsAString pos_str;
+    BSTR ret;
+    HRESULT hres;
+
     TRACE("(%p)->(%p)\n", This, p);
-    return get_nsstyle_attr_var(This->nsstyle, STYLEID_BACKGROUND_POSITION_X, p, 0);
+
+    nsAString_Init(&pos_str, NULL);
+    hres = get_nsstyle_attr_nsval(This->nsstyle, STYLEID_BACKGROUND_POSITION, &pos_str);
+    if(SUCCEEDED(hres)) {
+        const PRUnichar *pos, *space;
+
+        nsAString_GetData(&pos_str, &pos);
+        space = strchrW(pos, ' ');
+        if(!space) {
+            WARN("no space in %s\n", debugstr_w(pos));
+            space = pos + strlenW(pos);
+        }
+
+        if(space != pos) {
+            ret = SysAllocStringLen(pos, space-pos);
+            if(!ret)
+                hres = E_OUTOFMEMORY;
+        }else {
+            ret = NULL;
+        }
+    }
+    nsAString_Finish(&pos_str);
+    if(FAILED(hres))
+        return hres;
+
+    TRACE("returning %s\n", debugstr_w(ret));
+    V_VT(p) = VT_BSTR;
+    V_BSTR(p) = ret;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLStyle_put_backgroundPositionY(IHTMLStyle *iface, VARIANT v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
+    WCHAR buf[14], *pos_val;
+    nsAString pos_str;
+    const WCHAR *val;
+    DWORD val_len;
+    HRESULT hres;
+
     TRACE("(%p)->(v%d)\n", This, V_VT(&v));
-    return set_nsstyle_attr_var(This->nsstyle, STYLEID_BACKGROUND_POSITION_Y, &v, 0);
+
+    hres = var_to_styleval(&v, buf, ATTR_FIX_PX, &val);
+    if(FAILED(hres))
+        return hres;
+
+    val_len = val ? strlenW(val) : 0;
+
+    nsAString_Init(&pos_str, NULL);
+    hres = get_nsstyle_attr_nsval(This->nsstyle, STYLEID_BACKGROUND_POSITION, &pos_str);
+    if(SUCCEEDED(hres)) {
+        const PRUnichar *pos, *space;
+        DWORD posx_len;
+
+        nsAString_GetData(&pos_str, &pos);
+        space = strchrW(pos, ' ');
+        if(space) {
+            space++;
+        }else {
+            static const WCHAR zero_pxW[] = {'0','p','x',' ',0};
+
+            TRACE("no space in %s\n", debugstr_w(pos));
+            pos = zero_pxW;
+            space = pos + sizeof(zero_pxW)/sizeof(WCHAR)-1;
+        }
+
+        posx_len = space-pos;
+
+        pos_val = heap_alloc((posx_len+val_len+1)*sizeof(WCHAR));
+        if(pos_val) {
+            memcpy(pos_val, pos, posx_len*sizeof(WCHAR));
+            if(val_len)
+                memcpy(pos_val+posx_len, val, val_len*sizeof(WCHAR));
+            pos_val[posx_len+val_len] = 0;
+        }else {
+            hres = E_OUTOFMEMORY;
+        }
+    }
+    nsAString_Finish(&pos_str);
+    if(FAILED(hres))
+        return hres;
+
+    TRACE("setting position to %s\n", debugstr_w(pos_val));
+    hres = set_nsstyle_attr(This->nsstyle, STYLEID_BACKGROUND_POSITION, pos_val, ATTR_FIX_PX);
+    heap_free(pos_val);
+    return hres;
 }
 
 static HRESULT WINAPI HTMLStyle_get_backgroundPositionY(IHTMLStyle *iface, VARIANT *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
+    nsAString pos_str;
+    BSTR ret;
+    HRESULT hres;
+
     TRACE("(%p)->(%p)\n", This, p);
-    return get_nsstyle_attr_var(This->nsstyle, STYLEID_BACKGROUND_POSITION_Y, p, 0);
+
+    nsAString_Init(&pos_str, NULL);
+    hres = get_nsstyle_attr_nsval(This->nsstyle, STYLEID_BACKGROUND_POSITION, &pos_str);
+    if(SUCCEEDED(hres)) {
+        const PRUnichar *pos, *posy;
+
+        nsAString_GetData(&pos_str, &pos);
+        posy = strchrW(pos, ' ');
+        if(posy) {
+            ret = SysAllocString(posy+1);
+            if(!ret)
+                hres = E_OUTOFMEMORY;
+        }else {
+            ret = NULL;
+        }
+    }
+    nsAString_Finish(&pos_str);
+    if(FAILED(hres))
+        return hres;
+
+    TRACE("returning %s\n", debugstr_w(ret));
+    V_VT(p) = VT_BSTR;
+    V_BSTR(p) = ret;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLStyle_put_wordSpacing(IHTMLStyle *iface, VARIANT v)
@@ -2121,29 +2359,37 @@ static HRESULT WINAPI HTMLStyle_get_overflow(IHTMLStyle *iface, BSTR *p)
 static HRESULT WINAPI HTMLStyle_put_pageBreakBefore(IHTMLStyle *iface, BSTR v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    return set_nsstyle_attr(This->nsstyle, STYLEID_PAGE_BREAK_BEFORE, v, 0);
 }
 
 static HRESULT WINAPI HTMLStyle_get_pageBreakBefore(IHTMLStyle *iface, BSTR *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_nsstyle_attr(This->nsstyle, STYLEID_PAGE_BREAK_BEFORE, p, 0);
 }
 
 static HRESULT WINAPI HTMLStyle_put_pageBreakAfter(IHTMLStyle *iface, BSTR v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    return set_nsstyle_attr(This->nsstyle, STYLEID_PAGE_BREAK_AFTER, v, 0);
 }
 
 static HRESULT WINAPI HTMLStyle_get_pageBreakAfter(IHTMLStyle *iface, BSTR *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_nsstyle_attr(This->nsstyle, STYLEID_PAGE_BREAK_AFTER, p, 0);
 }
 
 static HRESULT WINAPI HTMLStyle_put_cssText(IHTMLStyle *iface, BSTR v)
@@ -2357,35 +2603,159 @@ static HRESULT WINAPI HTMLStyle_get_cursor(IHTMLStyle *iface, BSTR *p)
 static HRESULT WINAPI HTMLStyle_put_clip(IHTMLStyle *iface, BSTR v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    return set_style_attr(This, STYLEID_CLIP, v, 0);
 }
 
 static HRESULT WINAPI HTMLStyle_get_clip(IHTMLStyle *iface, BSTR *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_nsstyle_attr(This->nsstyle, STYLEID_CLIP, p, ATTR_REMOVE_COMMA);
+}
+
+static void set_opacity(HTMLStyle *This, const WCHAR *val)
+{
+    nsAString name_str, val_str, empty_str;
+    nsresult nsres;
+
+    static const WCHAR opacityW[] = {'o','p','a','c','i','t','y',0};
+
+    TRACE("%s\n", debugstr_w(val));
+
+    nsAString_InitDepend(&name_str, opacityW);
+    nsAString_InitDepend(&val_str, val);
+    nsAString_InitDepend(&empty_str, emptyW);
+
+    nsres = nsIDOMCSSStyleDeclaration_SetProperty(This->nsstyle, &name_str, &val_str, &empty_str);
+    if(NS_FAILED(nsres))
+        ERR("SetProperty failed: %08x\n", nsres);
+
+    nsAString_Finish(&name_str);
+    nsAString_Finish(&val_str);
+    nsAString_Finish(&empty_str);
+}
+
+static void update_filter(HTMLStyle *This)
+{
+    const WCHAR *ptr = This->filter, *ptr2;
+
+    static const WCHAR alphaW[] = {'a','l','p','h','a'};
+
+    if(!ptr) {
+        set_opacity(This, emptyW);
+        return;
+    }
+
+    while(1) {
+        while(isspaceW(*ptr))
+            ptr++;
+        if(!*ptr)
+            break;
+
+        ptr2 = ptr;
+        while(isalnumW(*ptr))
+            ptr++;
+        if(ptr == ptr2) {
+            WARN("unexpected char '%c'\n", *ptr);
+            break;
+        }
+        if(*ptr != '(') {
+            WARN("expected '('\n");
+            continue;
+        }
+
+        if(ptr2 + sizeof(alphaW)/sizeof(WCHAR) == ptr && !memcmp(ptr2, alphaW, sizeof(alphaW))) {
+            static const WCHAR formatW[] = {'%','f',0};
+            static const WCHAR opacityW[] = {'o','p','a','c','i','t','y','='};
+
+            ptr++;
+            do {
+                while(isspaceW(*ptr))
+                    ptr++;
+
+                ptr2 = ptr;
+                while(*ptr && *ptr != ',' && *ptr != ')')
+                    ptr++;
+                if(!*ptr) {
+                    WARN("unexpected end of string\n");
+                    break;
+                }
+
+                if(ptr-ptr2 > sizeof(opacityW)/sizeof(WCHAR) && !memcmp(ptr2, opacityW, sizeof(opacityW))) {
+                    float fval = 0.0f, e = 0.1f;
+                    WCHAR buf[32];
+
+                    ptr2 += sizeof(opacityW)/sizeof(WCHAR);
+
+                    while(isdigitW(*ptr2))
+                        fval = fval*10.0f + (float)(*ptr2++ - '0');
+
+                    if(*ptr2 == '.') {
+                        while(isdigitW(*++ptr2)) {
+                            fval += e * (float)(*ptr2++ - '0');
+                            e *= 0.1f;
+                        }
+                    }
+
+                    sprintfW(buf, formatW, fval * 0.01f);
+                    set_opacity(This, buf);
+                }else {
+                    FIXME("unknown param %s\n", debugstr_wn(ptr2, ptr-ptr2));
+                }
+
+                if(*ptr == ',')
+                    ptr++;
+            }while(*ptr != ')');
+        }else {
+            FIXME("unknown filter %s\n", debugstr_wn(ptr2, ptr-ptr2));
+            ptr = strchrW(ptr, ')');
+            if(!ptr)
+                break;
+            ptr++;
+        }
+    }
 }
 
 static HRESULT WINAPI HTMLStyle_put_filter(IHTMLStyle *iface, BSTR v)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
+    WCHAR *new_filter = NULL;
 
-    WARN("(%p)->(%s)\n", This, debugstr_w(v));
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
 
-    /* FIXME: Handle MS-style filters */
-    return set_style_attr(This, STYLEID_FILTER, v, 0);
+    if(v) {
+        new_filter = heap_strdupW(v);
+        if(!new_filter)
+            return E_OUTOFMEMORY;
+    }
+
+    heap_free(This->filter);
+    This->filter = new_filter;
+
+    update_filter(This);
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLStyle_get_filter(IHTMLStyle *iface, BSTR *p)
 {
     HTMLStyle *This = impl_from_IHTMLStyle(iface);
 
-    WARN("(%p)->(%p)\n", This, p);
+    TRACE("(%p)->(%p)\n", This, p);
 
-    /* FIXME: Handle MS-style filters */
-    return get_style_attr(This, STYLEID_FILTER, p);
+    if(This->filter) {
+        *p = SysAllocString(This->filter);
+        if(!*p)
+            return E_OUTOFMEMORY;
+    }else {
+        *p = NULL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLStyle_setAttribute(IHTMLStyle *iface, BSTR strAttributeName,
@@ -2701,6 +3071,8 @@ static const tid_t HTMLStyle_iface_tids[] = {
     IHTMLStyle2_tid,
     IHTMLStyle3_tid,
     IHTMLStyle4_tid,
+    IHTMLStyle5_tid,
+    IHTMLStyle6_tid,
     0
 };
 static dispex_static_data_t HTMLStyle_dispex = {
@@ -2710,19 +3082,24 @@ static dispex_static_data_t HTMLStyle_dispex = {
     HTMLStyle_iface_tids
 };
 
-IHTMLStyle *HTMLStyle_Create(nsIDOMCSSStyleDeclaration *nsstyle)
+HRESULT HTMLStyle_Create(nsIDOMCSSStyleDeclaration *nsstyle, HTMLStyle **ret)
 {
-    HTMLStyle *ret = heap_alloc_zero(sizeof(HTMLStyle));
+    HTMLStyle *style;
 
-    ret->IHTMLStyle_iface.lpVtbl = &HTMLStyleVtbl;
-    ret->ref = 1;
-    ret->nsstyle = nsstyle;
-    HTMLStyle2_Init(ret);
-    HTMLStyle3_Init(ret);
+    style = heap_alloc_zero(sizeof(HTMLStyle));
+    if(!style)
+        return E_OUTOFMEMORY;
+
+    style->IHTMLStyle_iface.lpVtbl = &HTMLStyleVtbl;
+    style->ref = 1;
+    style->nsstyle = nsstyle;
+    HTMLStyle2_Init(style);
+    HTMLStyle3_Init(style);
 
     nsIDOMCSSStyleDeclaration_AddRef(nsstyle);
 
-    init_dispex(&ret->dispex, (IUnknown*)&ret->IHTMLStyle_iface, &HTMLStyle_dispex);
+    init_dispex(&style->dispex, (IUnknown*)&style->IHTMLStyle_iface, &HTMLStyle_dispex);
 
-    return &ret->IHTMLStyle_iface;
+    *ret = style;
+    return S_OK;
 }
