@@ -27,10 +27,14 @@
 #include "mshtmdid.h"
 #include "shlguid.h"
 
+#define NO_SHLWAPI_REG
+#include "shlwapi.h"
+
 #include "wine/debug.h"
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
+#include "binding.h"
 #include "resource.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
@@ -103,7 +107,7 @@ nsIDOMWindow *get_nsdoc_window(nsIDOMDocument *nsdoc)
     nsres = nsIDOMAbstractView_QueryInterface(nsview, &IID_nsIDOMWindow, (void**)&nswindow);
     nsIDOMAbstractView_Release(nsview);
     if(NS_FAILED(nsres)) {
-        ERR("Coult not get nsIDOMWindow iface: %08x\n", nsres);
+        ERR("Could not get nsIDOMWindow iface: %08x\n", nsres);
         return NULL;
     }
 
@@ -487,15 +491,25 @@ static HRESULT WINAPI HTMLWindow2_get_defaultStatus(IHTMLWindow2 *iface, BSTR *p
 static HRESULT WINAPI HTMLWindow2_put_status(IHTMLWindow2 *iface, BSTR v)
 {
     HTMLWindow *This = impl_from_IHTMLWindow2(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+
+    WARN("(%p)->(%s)\n", This, debugstr_w(v));
+
+    /*
+     * FIXME: Since IE7, setting status is blocked, but still possible in certain circumstances.
+     * Ignoring the call should be enough for us.
+     */
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow2_get_status(IHTMLWindow2 *iface, BSTR *p)
 {
     HTMLWindow *This = impl_from_IHTMLWindow2(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    /* See put_status */
+    *p = NULL;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLWindow2_setTimeout(IHTMLWindow2 *iface, BSTR expression,
@@ -1665,6 +1679,79 @@ static ULONG WINAPI HTMLPrivateWindow_Release(IHTMLPrivateWindow *iface)
     return IHTMLWindow2_Release(&This->IHTMLWindow2_iface);
 }
 
+typedef struct {
+    task_t header;
+    HTMLWindow *window;
+    IUri *uri;
+} navigate_javascript_task_t;
+
+static void navigate_javascript_proc(task_t *_task)
+{
+    navigate_javascript_task_t *task = (navigate_javascript_task_t*)_task;
+    HTMLWindow *window = task->window;
+    VARIANT v;
+    BSTR code;
+    HRESULT hres;
+
+    static const WCHAR jscriptW[] = {'j','s','c','r','i','p','t',0};
+
+    task->window->readystate = READYSTATE_COMPLETE;
+
+    hres = IUri_GetPath(task->uri, &code);
+    if(FAILED(hres))
+        return;
+
+    set_download_state(window->doc_obj, 1);
+
+    V_VT(&v) = VT_EMPTY;
+    hres = exec_script(window, code, jscriptW, &v);
+    SysFreeString(code);
+    if(SUCCEEDED(hres) && V_VT(&v) != VT_EMPTY) {
+        FIXME("javascirpt URL returned %s\n", debugstr_variant(&v));
+        VariantClear(&v);
+    }
+
+    if(window->doc_obj->view_sink)
+        IAdviseSink_OnViewChange(window->doc_obj->view_sink, DVASPECT_CONTENT, -1);
+
+    set_download_state(window->doc_obj, 0);
+}
+
+static void navigate_javascript_task_destr(task_t *_task)
+{
+    navigate_javascript_task_t *task = (navigate_javascript_task_t*)_task;
+
+    IUri_Release(task->uri);
+    heap_free(task);
+}
+
+typedef struct {
+    task_t header;
+    HTMLWindow *window;
+    nsChannelBSC *bscallback;
+    IMoniker *mon;
+} navigate_task_t;
+
+static void navigate_proc(task_t *_task)
+{
+    navigate_task_t *task = (navigate_task_t*)_task;
+    HRESULT hres;
+
+    hres = set_moniker(&task->window->doc_obj->basedoc, task->mon, NULL, task->bscallback, TRUE);
+    if(SUCCEEDED(hres))
+        hres = start_binding(task->window, NULL, (BSCallback*)task->bscallback, NULL);
+
+}
+
+static void navigate_task_destr(task_t *_task)
+{
+    navigate_task_t *task = (navigate_task_t*)_task;
+
+    IUnknown_Release((IUnknown*)task->bscallback);
+    IMoniker_Release(task->mon);
+    heap_free(task);
+}
+
 static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface, BSTR url, BSTR arg2, BSTR arg3,
         BSTR arg4, VARIANT *post_data_var, VARIANT *headers_var, ULONG flags)
 {
@@ -1674,7 +1761,9 @@ static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface,
     WCHAR *headers = NULL;
     nsChannelBSC *bsc;
     IMoniker *mon;
+    DWORD scheme;
     BSTR new_url;
+    IUri *uri;
     HRESULT hres;
 
     TRACE("(%p)->(%s %s %s %s %s %s %x)\n", This, debugstr_w(url), debugstr_w(arg2), debugstr_w(arg3), debugstr_w(arg4),
@@ -1709,15 +1798,19 @@ static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface,
         }
     }
 
-    /* FIXME: Why not set_ready_state? */
-    This->readystate = READYSTATE_UNINITIALIZED;
-
-    hres = CreateURLMoniker(NULL, new_url, &mon);
+    hres = CreateUri(new_url, 0, 0, &uri);
     if(new_url != url)
         SysFreeString(new_url);
     if(FAILED(hres))
         return hres;
 
+    hres = CreateURLMonikerEx2(NULL, uri, &mon, URL_MK_UNIFORM);
+    if(FAILED(hres))
+        return hres;
+
+    /* FIXME: Why not set_ready_state? */
+    This->readystate = READYSTATE_UNINITIALIZED;
+
     if(post_data_var) {
         if(V_VT(post_data_var) == (VT_ARRAY|VT_UI1)) {
             SafeArrayAccessData(V_ARRAY(post_data_var), (void**)&post_data);
@@ -1740,13 +1833,50 @@ static HRESULT WINAPI HTMLPrivateWindow_SuperNavigate(IHTMLPrivateWindow *iface,
         return hres;
     }
 
-    hres = set_moniker(&This->doc_obj->basedoc, mon, NULL, bsc, TRUE);
-    if(SUCCEEDED(hres))
-        hres = async_start_doc_binding(This, bsc);
+    prepare_for_binding(&This->doc_obj->basedoc, mon, NULL, TRUE);
 
-    IUnknown_Release((IUnknown*)bsc);
-    IMoniker_Release(mon);
-    return hres;
+    hres = IUri_GetScheme(uri, &scheme);
+
+    if(scheme != URL_SCHEME_JAVASCRIPT) {
+        navigate_task_t *task;
+
+        IUri_Release(uri);
+
+        task = heap_alloc(sizeof(*task));
+        if(!task) {
+            IUnknown_Release((IUnknown*)bsc);
+            IMoniker_Release(mon);
+            return E_OUTOFMEMORY;
+        }
+
+        task->window = This;
+        task->bscallback = bsc;
+        task->mon = mon;
+        push_task(&task->header, navigate_proc, navigate_task_destr, This->task_magic);
+
+        /* Silently and repeated when real loading starts? */
+        This->readystate = READYSTATE_LOADING;
+    }else {
+        navigate_javascript_task_t *task;
+
+        IUnknown_Release((IUnknown*)bsc);
+        IMoniker_Release(mon);
+
+        task = heap_alloc(sizeof(*task));
+        if(!task) {
+            IUri_Release(uri);
+            return E_OUTOFMEMORY;
+        }
+
+        task->window = This;
+        task->uri = uri;
+        push_task(&task->header, navigate_javascript_proc, navigate_javascript_task_destr, This->task_magic);
+
+        /* Why silently? */
+        This->readystate = READYSTATE_COMPLETE;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLPrivateWindow_GetPendingUrl(IHTMLPrivateWindow *iface, BSTR *url)
