@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -32,9 +33,9 @@
 #include "shlwapi.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 #include "mshtml_private.h"
+#include "binding.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -63,7 +64,6 @@ struct  nsWineURI {
     NSContainer *container;
     windowref_t *window_ref;
     nsChannelBSC *channel_bsc;
-    BSTR wine_url;
     IUri *uri;
     IUriBuilder *uri_builder;
     BOOL is_doc_uri;
@@ -73,29 +73,20 @@ static BOOL ensure_uri(nsWineURI *This)
 {
     HRESULT hres;
 
-    if(This->uri)
-        return TRUE;
+    assert(This->uri || This->uri_builder);
 
-    if(This->uri_builder) {
+    if(!This->uri) {
         hres = IUriBuilder_CreateUriSimple(This->uri_builder, 0, 0, &This->uri);
         if(FAILED(hres)) {
             WARN("CreateUriSimple failed: %08x\n", hres);
             return FALSE;
         }
-
-        return TRUE;
-    }
-
-    hres = CreateUri(This->wine_url, 0, 0, &This->uri);
-    if(FAILED(hres)) {
-        WARN("CreateUri failed: %08x\n", hres);
-        return FALSE;
     }
 
     return TRUE;
 }
 
-static nsresult create_uri(nsIURI*,HTMLWindow*,NSContainer*,nsWineURI**);
+static nsresult create_nsuri(IUri*,nsIURI*,HTMLWindow*,NSContainer*,nsWineURI**);
 
 static const char *debugstr_nsacstr(const nsACString *nsstr)
 {
@@ -155,6 +146,7 @@ static BOOL exec_shldocvw_67(HTMLDocumentObj *doc, BSTR url)
 static nsresult before_async_open(nsChannel *channel, NSContainer *container, BOOL *cancel)
 {
     HTMLDocumentObj *doc = container->doc;
+    BSTR display_uri;
     DWORD hlnf = 0;
     HRESULT hres;
 
@@ -172,23 +164,23 @@ static nsresult before_async_open(nsChannel *channel, NSContainer *container, BO
         return NS_OK;
     }
 
+    hres = IUri_GetDisplayUri(channel->uri->uri, &display_uri);
+    if(FAILED(hres))
+        return NS_ERROR_FAILURE;
+
     if(!hlnf) {
-        BSTR display_uri;
         BOOL b;
 
-        hres = IUri_GetDisplayUri(channel->uri->uri, &display_uri);
-        if(FAILED(hres))
-            return NS_ERROR_FAILURE;
-
         b = !exec_shldocvw_67(doc, display_uri);
-        SysFreeString(display_uri);
         if(b) {
+            SysFreeString(display_uri);
             *cancel = FALSE;
             return NS_OK;
         }
     }
 
-    hres = hlink_frame_navigate(&doc->basedoc, channel->uri->wine_url, channel, hlnf, cancel);
+    hres = hlink_frame_navigate(&doc->basedoc, display_uri, channel, hlnf, cancel);
+    SysFreeString(display_uri);
     if(FAILED(hres))
         *cancel = TRUE;
     return NS_OK;
@@ -230,26 +222,6 @@ HRESULT load_nsuri(HTMLWindow *window, nsWineURI *uri, nsChannelBSC *channelbsc,
     return S_OK;
 }
 
-static HRESULT set_wine_url(nsWineURI *This, LPCWSTR url)
-{
-    TRACE("(%p)->(%s)\n", This, debugstr_w(url));
-
-    if(url) {
-        BSTR new_url;
-
-        new_url = SysAllocString(url);
-        if(!new_url)
-            return E_OUTOFMEMORY;
-        SysFreeString(This->wine_url);
-        This->wine_url = new_url;
-    }else {
-        heap_free(This->wine_url);
-        This->wine_url = NULL;
-    }
-
-    return S_OK;
-}
-
 static void set_uri_nscontainer(nsWineURI *This, NSContainer *nscontainer)
 {
     if(This->container) {
@@ -916,8 +888,14 @@ static void start_binding_proc(task_t *_task)
     start_binding_task_t *task = (start_binding_task_t*)_task;
 
     start_binding(NULL, task->doc, (BSCallback*)task->bscallback, NULL);
+}
 
-    IUnknown_Release((IUnknown*)task->bscallback);
+static void start_binding_task_destr(task_t *_task)
+{
+    start_binding_task_t *task = (start_binding_task_t*)_task;
+
+    IBindStatusCallback_Release(&task->bscallback->bsc.IBindStatusCallback_iface);
+    heap_free(task);
 }
 
 static nsresult async_open(nsChannel *This, HTMLWindow *window, BOOL is_doc_channel, nsIStreamListener *listener,
@@ -952,7 +930,7 @@ static nsresult async_open(nsChannel *This, HTMLWindow *window, BOOL is_doc_chan
 
         task->doc = window->doc;
         task->bscallback = bscallback;
-        push_task(&task->header, start_binding_proc, window->doc->basedoc.task_magic);
+        push_task(&task->header, start_binding_proc, start_binding_task_destr, window->doc->basedoc.task_magic);
     }
 
     return NS_OK;
@@ -966,7 +944,7 @@ static nsresult NSAPI nsChannel_AsyncOpen(nsIHttpChannel *iface, nsIStreamListen
     BOOL cancel = FALSE;
     nsresult nsres = NS_OK;
 
-    TRACE("(%p)->(%p %p) opening %s\n", This, aListener, aContext, debugstr_w(This->uri->wine_url));
+    TRACE("(%p)->(%p %p)\n", This, aListener, aContext);
 
     if(!ensure_uri(This->uri))
         return NS_ERROR_FAILURE;
@@ -1025,7 +1003,7 @@ static nsresult NSAPI nsChannel_AsyncOpen(nsIHttpChannel *iface, nsIStreamListen
             nsres = before_async_open(This, window->doc_obj->nscontainer, &cancel);
             if(NS_SUCCEEDED(nsres)  && cancel) {
                 TRACE("canceled\n");
-                nsres = NS_ERROR_UNEXPECTED;
+                nsres = NS_BINDING_ABORTED;
             }
         }
     }
@@ -1565,7 +1543,7 @@ static BOOL ensure_uri_builder(nsWineURI *This)
 
         hres = CreateIUriBuilder(This->uri, 0, 0, &This->uri_builder);
         if(FAILED(hres)) {
-            WARN("CreateIUriBulder failed: %08x\n", hres);
+            WARN("CreateIUriBuilder failed: %08x\n", hres);
             return FALSE;
         }
     }
@@ -1574,23 +1552,6 @@ static BOOL ensure_uri_builder(nsWineURI *This)
     return TRUE;
 }
 
-/* Temporary helper */
-static void sync_wine_url(nsWineURI *This)
-{
-    BSTR new_url;
-    HRESULT hres;
-
-    if(!ensure_uri(This))
-        return;
-
-    hres = IUri_GetDisplayUri(This->uri, &new_url);
-    if(FAILED(hres))
-        return;
-
-    SysFreeString(This->wine_url);
-    This->wine_url = new_url;
-}
-
 static nsresult get_uri_string(nsWineURI *This, Uri_PROPERTY prop, nsACString *ret)
 {
     char *vala;
@@ -1679,7 +1640,6 @@ static nsrefcnt NSAPI nsURI_Release(nsIURL *iface)
             nsIURI_Release(This->nsuri);
         if(This->uri)
             IUri_Release(This->uri);
-        SysFreeString(This->wine_url);
         heap_free(This);
     }
 
@@ -1724,7 +1684,6 @@ static nsresult NSAPI nsURI_SetSpec(nsIURL *iface, const nsACString *aSpec)
     }
 
     This->uri = uri;
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1788,35 +1747,80 @@ static nsresult NSAPI nsURI_SetScheme(nsIURL *iface, const nsACString *aScheme)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
 static nsresult NSAPI nsURI_GetUserPass(nsIURL *iface, nsACString *aUserPass)
 {
     nsWineURI *This = impl_from_nsIURL(iface);
+    BSTR user, pass;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, aUserPass);
 
-    if(This->nsuri)
-        return nsIURI_GetUserPass(This->nsuri, aUserPass);
+    if(!ensure_uri(This))
+        return NS_ERROR_UNEXPECTED;
 
-    return get_uri_string(This, Uri_PROPERTY_USER_INFO, aUserPass);
+    hres = IUri_GetUserName(This->uri, &user);
+    if(FAILED(hres))
+        return NS_ERROR_FAILURE;
+
+    hres = IUri_GetPassword(This->uri, &pass);
+    if(FAILED(hres)) {
+        SysFreeString(user);
+        return NS_ERROR_FAILURE;
+    }
+
+    if(*user || *pass) {
+        FIXME("Construct user:pass string\n");
+    }else {
+        nsACString_SetData(aUserPass, "");
+    }
+
+    SysFreeString(user);
+    SysFreeString(pass);
+    return NS_OK;
 }
 
 static nsresult NSAPI nsURI_SetUserPass(nsIURL *iface, const nsACString *aUserPass)
 {
     nsWineURI *This = impl_from_nsIURL(iface);
+    WCHAR *user = NULL, *pass = NULL, *buf = NULL;
+    const char *user_pass;
+    HRESULT hres;
 
     TRACE("(%p)->(%s)\n", This, debugstr_nsacstr(aUserPass));
 
-    if(This->nsuri) {
-        invalidate_uri(This);
-        return nsIURI_SetUserPass(This->nsuri, aUserPass);
+    if(!ensure_uri_builder(This))
+        return NS_ERROR_UNEXPECTED;
+
+    nsACString_GetData(aUserPass, &user_pass);
+    if(*user_pass) {
+        WCHAR *ptr;
+
+        buf = heap_strdupAtoW(user_pass);
+        if(!buf)
+            return NS_ERROR_OUT_OF_MEMORY;
+
+        ptr = strchrW(buf, ':');
+        if(!ptr) {
+            user = buf;
+        }else if(ptr != buf) {
+            *ptr++ = 0;
+            user = buf;
+            if(*ptr)
+                pass = ptr;
+        }else {
+            pass = buf+1;
+        }
     }
 
-    FIXME("default action not implemented\n");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    hres = IUriBuilder_SetUserName(This->uri_builder, user);
+    if(SUCCEEDED(hres))
+        hres = IUriBuilder_SetPassword(This->uri_builder, pass);
+
+    heap_free(buf);
+    return SUCCEEDED(hres) ? NS_OK : NS_ERROR_FAILURE;
 }
 
 static nsresult NSAPI nsURI_GetUsername(nsIURL *iface, nsACString *aUsername)
@@ -1850,7 +1854,6 @@ static nsresult NSAPI nsURI_SetUsername(nsIURL *iface, const nsACString *aUserna
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1885,7 +1888,6 @@ static nsresult NSAPI nsURI_SetPassword(nsIURL *iface, const nsACString *aPasswo
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1964,7 +1966,6 @@ static nsresult NSAPI nsURI_SetHost(nsIURL *iface, const nsACString *aHost)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -1976,9 +1977,6 @@ static nsresult NSAPI nsURI_GetPort(nsIURL *iface, PRInt32 *aPort)
 
     TRACE("(%p)->(%p)\n", This, aPort);
 
-    if(This->nsuri)
-        return nsIURI_GetPort(This->nsuri, aPort);
-
     if(!ensure_uri(This))
         return NS_ERROR_UNEXPECTED;
 
@@ -1995,16 +1993,15 @@ static nsresult NSAPI nsURI_GetPort(nsIURL *iface, PRInt32 *aPort)
 static nsresult NSAPI nsURI_SetPort(nsIURL *iface, PRInt32 aPort)
 {
     nsWineURI *This = impl_from_nsIURL(iface);
+    HRESULT hres;
 
     TRACE("(%p)->(%d)\n", This, aPort);
 
-    if(This->nsuri) {
-        invalidate_uri(This);
-        return nsIURI_SetPort(This->nsuri, aPort);
-    }
+    if(!ensure_uri_builder(This))
+        return NS_ERROR_UNEXPECTED;
 
-    FIXME("default action not implemented\n");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    hres = IUriBuilder_SetPort(This->uri_builder, aPort != -1, aPort);
+    return SUCCEEDED(hres) ? NS_OK : NS_ERROR_FAILURE;
 }
 
 static nsresult NSAPI nsURI_GetPath(nsIURL *iface, nsACString *aPath)
@@ -2038,7 +2035,6 @@ static nsresult NSAPI nsURI_SetPath(nsIURL *iface, const nsACString *aPath)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -2111,16 +2107,12 @@ static nsresult NSAPI nsURI_Clone(nsIURL *iface, nsIURI **_retval)
         }
     }
 
-    nsres = create_uri(nsuri, This->window_ref ? This->window_ref->window : NULL, This->container, &wine_uri);
+    nsres = create_nsuri(This->uri, nsuri, This->window_ref ? This->window_ref->window : NULL, This->container, &wine_uri);
     if(NS_FAILED(nsres)) {
-        WARN("create_uri failed: %08x\n", nsres);
+        WARN("create_nsuri failed: %08x\n", nsres);
         return nsres;
     }
 
-    wine_uri->uri = This->uri;
-    IUri_AddRef(wine_uri->uri);
-    sync_wine_url(wine_uri);
-
     *_retval = (nsIURI*)&wine_uri->nsIURL_iface;
     return NS_OK;
 }
@@ -2187,7 +2179,7 @@ static nsresult NSAPI nsURI_GetAsciiHost(nsIURL *iface, nsACString *aAsciiHost)
     if(This->nsuri)
         return nsIURI_GetAsciiHost(This->nsuri, aAsciiHost);
 
-    FIXME("Use Uri_PUNYCODE_IDN_HOST flag\n");
+    WARN("Use Uri_PUNYCODE_IDN_HOST flag\n");
     return get_uri_string(This, Uri_PROPERTY_HOST, aAsciiHost);
 }
 
@@ -2276,7 +2268,6 @@ static nsresult NSAPI nsURL_SetQuery(nsIURL *iface, const nsACString *aQuery)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
@@ -2328,21 +2319,41 @@ static nsresult NSAPI nsURL_SetRef(nsIURL *iface, const nsACString *aRef)
     if(FAILED(hres))
         return NS_ERROR_UNEXPECTED;
 
-    sync_wine_url(This);
     return NS_OK;
 }
 
 static nsresult NSAPI nsURL_GetDirectory(nsIURL *iface, nsACString *aDirectory)
 {
     nsWineURI *This = impl_from_nsIURL(iface);
+    char *dir = NULL;
+    WCHAR *ptr;
+    BSTR path;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, aDirectory);
 
-    if(This->nsurl)
-        return nsIURL_GetDirectory(This->nsurl, aDirectory);
+    if(!ensure_uri(This))
+        return NS_ERROR_UNEXPECTED;
 
-    FIXME("default action not implemented\n");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    hres = IUri_GetPath(This->uri, &path);
+    if(FAILED(hres))
+        return NS_ERROR_FAILURE;
+
+    ptr = strrchrW(path, '/');
+    if(ptr) {
+        if(ptr[1])
+            *ptr = 0;
+        dir = heap_strdupWtoA(path);
+        if(!dir) {
+            SysFreeString(path);
+            return NS_ERROR_OUT_OF_MEMORY;
+        }
+    }
+
+    SysFreeString(path);
+    TRACE("ret %s\n", debugstr_a(dir));
+    nsACString_SetData(aDirectory, dir ? dir : "");
+    return NS_OK;
 }
 
 static nsresult NSAPI nsURL_SetDirectory(nsIURL *iface, const nsACString *aDirectory)
@@ -2515,7 +2526,7 @@ static const nsIURLVtbl nsURLVtbl = {
     nsURL_GetRelativeSpec
 };
 
-static nsresult create_uri(nsIURI *nsuri, HTMLWindow *window, NSContainer *container, nsWineURI **_retval)
+static nsresult create_nsuri(IUri *iuri, nsIURI *nsuri, HTMLWindow *window, NSContainer *container, nsWineURI **_retval)
 {
     nsWineURI *ret = heap_alloc_zero(sizeof(nsWineURI));
 
@@ -2526,6 +2537,9 @@ static nsresult create_uri(nsIURI *nsuri, HTMLWindow *window, NSContainer *conta
     set_uri_nscontainer(ret, container);
     set_uri_window(ret, window);
 
+    IUri_AddRef(iuri);
+    ret->uri = iuri;
+
     if(nsuri)
         nsIURI_QueryInterface(nsuri, &IID_nsIURL, (void**)&ret->nsurl);
 
@@ -2537,13 +2551,19 @@ static nsresult create_uri(nsIURI *nsuri, HTMLWindow *window, NSContainer *conta
 HRESULT create_doc_uri(HTMLWindow *window, WCHAR *url, nsWineURI **ret)
 {
     nsWineURI *uri;
+    IUri *iuri;
     nsresult nsres;
+    HRESULT hres;
 
-    nsres = create_uri(NULL, window, window->doc_obj->nscontainer, &uri);
+    hres = CreateUri(url, 0, 0, &iuri);
+    if(FAILED(hres))
+        return hres;
+
+    nsres = create_nsuri(iuri, NULL, window, window->doc_obj->nscontainer, &uri);
+    IUri_Release(iuri);
     if(NS_FAILED(nsres))
         return E_FAIL;
 
-    set_wine_url(uri, url);
     uri->is_doc_uri = TRUE;
 
     *ret = uri;
@@ -2586,22 +2606,23 @@ HRESULT create_redirect_nschannel(const WCHAR *url, nsChannel *orig_channel, nsC
     HTMLWindow *window = NULL;
     nsChannel *channel;
     nsWineURI *uri;
+    IUri *iuri;
     nsresult nsres;
     HRESULT hres;
 
+    hres = CreateUri(url, 0, 0, &iuri);
+    if(FAILED(hres))
+        return hres;
+
     if(orig_channel->uri->window_ref)
         window = orig_channel->uri->window_ref->window;
-    nsres = create_uri(NULL, window, NULL, &uri);
+    nsres = create_nsuri(iuri, NULL, window, NULL, &uri);
+    IUri_Release(iuri);
     if(NS_FAILED(nsres))
         return E_FAIL;
 
-    hres = CreateUri(url, 0, 0, &uri->uri);
-    if(SUCCEEDED(hres))
-        nsres = create_nschannel(uri, &channel);
-    sync_wine_url(uri);
+    nsres = create_nschannel(uri, &channel);
     nsIURL_Release(&uri->nsIURL_iface);
-    if(FAILED(hres))
-        return hres;
     if(NS_FAILED(nsres))
         return E_FAIL;
 
@@ -2918,16 +2939,14 @@ static nsresult NSAPI nsIOService_NewURI(nsIIOService *iface, const nsACString *
         return nsres;
     }
 
-    nsres = create_uri(uri, window, NULL, &wine_uri);
+    nsres = create_nsuri(urlmon_uri, uri, window, NULL, &wine_uri);
+    IUri_Release(urlmon_uri);
     if(base_wine_uri)
         nsIURI_Release(&base_wine_uri->nsIURL_iface);
     if(NS_FAILED(nsres))
         return nsres;
 
-    wine_uri->uri = urlmon_uri;
-
-    sync_wine_url(wine_uri);
-    *_retval = (nsIURI*)wine_uri;
+    *_retval = (nsIURI*)&wine_uri->nsIURL_iface;
     return nsres;
 }
 
