@@ -28,7 +28,6 @@
 #include "ole2.h"
 #include "activscp.h"
 #include "activdbg.h"
-#include "objsafe.h"
 
 #include "wine/debug.h"
 
@@ -51,11 +50,11 @@ static const CLSID CLSID_JScript =
     {0xf414c260,0x6ac0,0x11cf,{0xb6,0xd1,0x00,0xaa,0x00,0xbb,0xbb,0x58}};
 
 struct ScriptHost {
-    const IActiveScriptSiteVtbl               *lpIActiveScriptSiteVtbl;
-    const IActiveScriptSiteInterruptPollVtbl  *lpIActiveScriptSiteInterruptPollVtbl;
-    const IActiveScriptSiteWindowVtbl         *lpIActiveScriptSiteWindowVtbl;
-    const IActiveScriptSiteDebugVtbl          *lpIActiveScriptSiteDebugVtbl;
-    const IServiceProviderVtbl                *lpServiceProviderVtbl;
+    IActiveScriptSite              IActiveScriptSite_iface;
+    IActiveScriptSiteInterruptPoll IActiveScriptSiteInterruptPoll_iface;
+    IActiveScriptSiteWindow        IActiveScriptSiteWindow_iface;
+    IActiveScriptSiteDebug         IActiveScriptSiteDebug_iface;
+    IServiceProvider               IServiceProvider_iface;
 
     LONG ref;
 
@@ -71,11 +70,6 @@ struct ScriptHost {
     struct list entry;
 };
 
-#define ACTSCPSITE(x)  ((IActiveScriptSite*)               &(x)->lpIActiveScriptSiteVtbl)
-#define ACTSCPPOLL(x)  (&(x)->lpIActiveScriptSiteInterruptPollVtbl)
-#define ACTSCPWIN(x)   (&(x)->lpIActiveScriptSiteWindowVtbl)
-#define ACTSCPDBG(x)   (&(x)->lpIActiveScriptSiteDebugVtbl)
-
 static void set_script_prop(ScriptHost *script_host, DWORD property, VARIANT *val)
 {
     IActiveScriptProperty *script_prop;
@@ -145,7 +139,7 @@ static BOOL init_script_engine(ScriptHost *script_host)
         return FALSE;
     }
 
-    hres = IActiveScript_SetScriptSite(script_host->script, ACTSCPSITE(script_host));
+    hres = IActiveScript_SetScriptSite(script_host->script, &script_host->IActiveScriptSite_iface);
     if(FAILED(hres)) {
         WARN("SetScriptSite failed: %08x\n", hres);
         IActiveScript_Close(script_host->script);
@@ -225,32 +219,35 @@ void connect_scripts(HTMLWindow *window)
     }
 }
 
-#define ACTSCPSITE_THIS(iface) DEFINE_THIS(ScriptHost, IActiveScriptSite, iface)
+static inline ScriptHost *impl_from_IActiveScriptSite(IActiveScriptSite *iface)
+{
+    return CONTAINING_RECORD(iface, ScriptHost, IActiveScriptSite_iface);
+}
 
 static HRESULT WINAPI ActiveScriptSite_QueryInterface(IActiveScriptSite *iface, REFIID riid, void **ppv)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
     *ppv = NULL;
 
     if(IsEqualGUID(&IID_IUnknown, riid)) {
         TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = ACTSCPSITE(This);
+        *ppv = &This->IActiveScriptSite_iface;
     }else if(IsEqualGUID(&IID_IActiveScriptSite, riid)) {
         TRACE("(%p)->(IID_IActiveScriptSite %p)\n", This, ppv);
-        *ppv = ACTSCPSITE(This);
+        *ppv = &This->IActiveScriptSite_iface;
     }else if(IsEqualGUID(&IID_IActiveScriptSiteInterruptPoll, riid)) {
         TRACE("(%p)->(IID_IActiveScriptSiteInterruprtPoll %p)\n", This, ppv);
-        *ppv = ACTSCPPOLL(This);
+        *ppv = &This->IActiveScriptSiteInterruptPoll_iface;
     }else if(IsEqualGUID(&IID_IActiveScriptSiteWindow, riid)) {
         TRACE("(%p)->(IID_IActiveScriptSiteWindow %p)\n", This, ppv);
-        *ppv = ACTSCPWIN(This);
+        *ppv = &This->IActiveScriptSiteWindow_iface;
     }else if(IsEqualGUID(&IID_IActiveScriptSiteDebug, riid)) {
         TRACE("(%p)->(IID_IActiveScriptSiteDebug %p)\n", This, ppv);
-        *ppv = ACTSCPDBG(This);
+        *ppv = &This->IActiveScriptSiteDebug_iface;
     }else if(IsEqualGUID(&IID_IServiceProvider, riid)) {
         TRACE("(%p)->(IID_IServiceProvider %p)\n", This, ppv);
-        *ppv = SERVPROV(This);
+        *ppv = &This->IServiceProvider_iface;
     }else if(IsEqualGUID(&IID_ICanHandleException, riid)) {
         TRACE("(%p)->(IID_ICanHandleException not supported %p)\n", This, ppv);
         return E_NOINTERFACE;
@@ -265,7 +262,7 @@ static HRESULT WINAPI ActiveScriptSite_QueryInterface(IActiveScriptSite *iface,
 
 static ULONG WINAPI ActiveScriptSite_AddRef(IActiveScriptSite *iface)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -275,7 +272,7 @@ static ULONG WINAPI ActiveScriptSite_AddRef(IActiveScriptSite *iface)
 
 static ULONG WINAPI ActiveScriptSite_Release(IActiveScriptSite *iface)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -292,7 +289,7 @@ static ULONG WINAPI ActiveScriptSite_Release(IActiveScriptSite *iface)
 
 static HRESULT WINAPI ActiveScriptSite_GetLCID(IActiveScriptSite *iface, LCID *plcid)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
     TRACE("(%p)->(%p)\n", This, plcid);
 
@@ -303,7 +300,7 @@ static HRESULT WINAPI ActiveScriptSite_GetLCID(IActiveScriptSite *iface, LCID *p
 static HRESULT WINAPI ActiveScriptSite_GetItemInfo(IActiveScriptSite *iface, LPCOLESTR pstrName,
         DWORD dwReturnMask, IUnknown **ppiunkItem, ITypeInfo **ppti)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
     TRACE("(%p)->(%s %x %p %p)\n", This, debugstr_w(pstrName), dwReturnMask, ppiunkItem, ppti);
 
@@ -321,7 +318,7 @@ static HRESULT WINAPI ActiveScriptSite_GetItemInfo(IActiveScriptSite *iface, LPC
         return E_FAIL;
 
     /* FIXME: Return proxy object */
-    *ppiunkItem = (IUnknown*)HTMLWINDOW2(This->window);
+    *ppiunkItem = (IUnknown*)&This->window->IHTMLWindow2_iface;
     IUnknown_AddRef(*ppiunkItem);
 
     return S_OK;
@@ -329,7 +326,7 @@ static HRESULT WINAPI ActiveScriptSite_GetItemInfo(IActiveScriptSite *iface, LPC
 
 static HRESULT WINAPI ActiveScriptSite_GetDocVersionString(IActiveScriptSite *iface, BSTR *pbstrVersion)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
     FIXME("(%p)->(%p)\n", This, pbstrVersion);
     return E_NOTIMPL;
 }
@@ -337,14 +334,14 @@ static HRESULT WINAPI ActiveScriptSite_GetDocVersionString(IActiveScriptSite *if
 static HRESULT WINAPI ActiveScriptSite_OnScriptTerminate(IActiveScriptSite *iface,
         const VARIANT *pvarResult, const EXCEPINFO *pexcepinfo)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
     FIXME("(%p)->(%p %p)\n", This, pvarResult, pexcepinfo);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ActiveScriptSite_OnStateChange(IActiveScriptSite *iface, SCRIPTSTATE ssScriptState)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
     TRACE("(%p)->(%x)\n", This, ssScriptState);
 
@@ -354,14 +351,14 @@ static HRESULT WINAPI ActiveScriptSite_OnStateChange(IActiveScriptSite *iface, S
 
 static HRESULT WINAPI ActiveScriptSite_OnScriptError(IActiveScriptSite *iface, IActiveScriptError *pscripterror)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
     FIXME("(%p)->(%p)\n", This, pscripterror);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ActiveScriptSite_OnEnterScript(IActiveScriptSite *iface)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
     TRACE("(%p)->()\n", This);
 
@@ -370,15 +367,13 @@ static HRESULT WINAPI ActiveScriptSite_OnEnterScript(IActiveScriptSite *iface)
 
 static HRESULT WINAPI ActiveScriptSite_OnLeaveScript(IActiveScriptSite *iface)
 {
-    ScriptHost *This = ACTSCPSITE_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSite(iface);
 
     TRACE("(%p)->()\n", This);
 
     return S_OK;
 }
 
-#undef ACTSCPSITE_THIS
-
 static const IActiveScriptSiteVtbl ActiveScriptSiteVtbl = {
     ActiveScriptSite_QueryInterface,
     ActiveScriptSite_AddRef,
@@ -393,38 +388,39 @@ static const IActiveScriptSiteVtbl ActiveScriptSiteVtbl = {
     ActiveScriptSite_OnLeaveScript
 };
 
-#define ACTSCPPOLL_THIS(iface) DEFINE_THIS(ScriptHost, IActiveScriptSiteInterruptPoll, iface)
+static inline ScriptHost *impl_from_IActiveScriptSiteInterruptPoll(IActiveScriptSiteInterruptPoll *iface)
+{
+    return CONTAINING_RECORD(iface, ScriptHost, IActiveScriptSiteInterruptPoll_iface);
+}
 
 static HRESULT WINAPI ActiveScriptSiteInterruptPoll_QueryInterface(IActiveScriptSiteInterruptPoll *iface,
         REFIID riid, void **ppv)
 {
-    ScriptHost *This = ACTSCPPOLL_THIS(iface);
-    return IActiveScriptSite_QueryInterface(ACTSCPSITE(This), riid, ppv);
+    ScriptHost *This = impl_from_IActiveScriptSiteInterruptPoll(iface);
+    return IActiveScriptSite_QueryInterface(&This->IActiveScriptSite_iface, riid, ppv);
 }
 
 static ULONG WINAPI ActiveScriptSiteInterruptPoll_AddRef(IActiveScriptSiteInterruptPoll *iface)
 {
-    ScriptHost *This = ACTSCPPOLL_THIS(iface);
-    return IActiveScriptSite_AddRef(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IActiveScriptSiteInterruptPoll(iface);
+    return IActiveScriptSite_AddRef(&This->IActiveScriptSite_iface);
 }
 
 static ULONG WINAPI ActiveScriptSiteInterruptPoll_Release(IActiveScriptSiteInterruptPoll *iface)
 {
-    ScriptHost *This = ACTSCPPOLL_THIS(iface);
-    return IActiveScriptSite_Release(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IActiveScriptSiteInterruptPoll(iface);
+    return IActiveScriptSite_Release(&This->IActiveScriptSite_iface);
 }
 
 static HRESULT WINAPI ActiveScriptSiteInterruptPoll_QueryContinue(IActiveScriptSiteInterruptPoll *iface)
 {
-    ScriptHost *This = ACTSCPPOLL_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteInterruptPoll(iface);
 
     TRACE("(%p)\n", This);
 
     return S_OK;
 }
 
-#undef ACTSCPPOLL_THIS
-
 static const IActiveScriptSiteInterruptPollVtbl ActiveScriptSiteInterruptPollVtbl = {
     ActiveScriptSiteInterruptPoll_QueryInterface,
     ActiveScriptSiteInterruptPoll_AddRef,
@@ -432,43 +428,44 @@ static const IActiveScriptSiteInterruptPollVtbl ActiveScriptSiteInterruptPollVtb
     ActiveScriptSiteInterruptPoll_QueryContinue
 };
 
-#define ACTSCPWIN_THIS(iface) DEFINE_THIS(ScriptHost, IActiveScriptSiteWindow, iface)
+static inline ScriptHost *impl_from_IActiveScriptSiteWindow(IActiveScriptSiteWindow *iface)
+{
+    return CONTAINING_RECORD(iface, ScriptHost, IActiveScriptSiteWindow_iface);
+}
 
 static HRESULT WINAPI ActiveScriptSiteWindow_QueryInterface(IActiveScriptSiteWindow *iface,
         REFIID riid, void **ppv)
 {
-    ScriptHost *This = ACTSCPWIN_THIS(iface);
-    return IActiveScriptSite_QueryInterface(ACTSCPSITE(This), riid, ppv);
+    ScriptHost *This = impl_from_IActiveScriptSiteWindow(iface);
+    return IActiveScriptSite_QueryInterface(&This->IActiveScriptSite_iface, riid, ppv);
 }
 
 static ULONG WINAPI ActiveScriptSiteWindow_AddRef(IActiveScriptSiteWindow *iface)
 {
-    ScriptHost *This = ACTSCPWIN_THIS(iface);
-    return IActiveScriptSite_AddRef(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IActiveScriptSiteWindow(iface);
+    return IActiveScriptSite_AddRef(&This->IActiveScriptSite_iface);
 }
 
 static ULONG WINAPI ActiveScriptSiteWindow_Release(IActiveScriptSiteWindow *iface)
 {
-    ScriptHost *This = ACTSCPWIN_THIS(iface);
-    return IActiveScriptSite_Release(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IActiveScriptSiteWindow(iface);
+    return IActiveScriptSite_Release(&This->IActiveScriptSite_iface);
 }
 
 static HRESULT WINAPI ActiveScriptSiteWindow_GetWindow(IActiveScriptSiteWindow *iface, HWND *phwnd)
 {
-    ScriptHost *This = ACTSCPWIN_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteWindow(iface);
     FIXME("(%p)->(%p)\n", This, phwnd);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ActiveScriptSiteWindow_EnableModeless(IActiveScriptSiteWindow *iface, BOOL fEnable)
 {
-    ScriptHost *This = ACTSCPWIN_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteWindow(iface);
     FIXME("(%p)->(%x)\n", This, fEnable);
     return E_NOTIMPL;
 }
 
-#undef ACTSCPWIN_THIS
-
 static const IActiveScriptSiteWindowVtbl ActiveScriptSiteWindowVtbl = {
     ActiveScriptSiteWindow_QueryInterface,
     ActiveScriptSiteWindow_AddRef,
@@ -477,31 +474,34 @@ static const IActiveScriptSiteWindowVtbl ActiveScriptSiteWindowVtbl = {
     ActiveScriptSiteWindow_EnableModeless
 };
 
-#define ACTSCPDBG_THIS(iface) DEFINE_THIS(ScriptHost, IActiveScriptSiteDebug, iface)
+static inline ScriptHost *impl_from_IActiveScriptSiteDebug(IActiveScriptSiteDebug *iface)
+{
+    return CONTAINING_RECORD(iface, ScriptHost, IActiveScriptSiteDebug_iface);
+}
 
 static HRESULT WINAPI ActiveScriptSiteDebug_QueryInterface(IActiveScriptSiteDebug *iface,
         REFIID riid, void **ppv)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
-    return IActiveScriptSite_QueryInterface(ACTSCPSITE(This), riid, ppv);
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
+    return IActiveScriptSite_QueryInterface(&This->IActiveScriptSite_iface, riid, ppv);
 }
 
 static ULONG WINAPI ActiveScriptSiteDebug_AddRef(IActiveScriptSiteDebug *iface)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
-    return IActiveScriptSite_AddRef(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
+    return IActiveScriptSite_AddRef(&This->IActiveScriptSite_iface);
 }
 
 static ULONG WINAPI ActiveScriptSiteDebug_Release(IActiveScriptSiteDebug *iface)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
-    return IActiveScriptSite_Release(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
+    return IActiveScriptSite_Release(&This->IActiveScriptSite_iface);
 }
 
 static HRESULT WINAPI ActiveScriptSiteDebug_GetDocumentContextFromPosition(IActiveScriptSiteDebug *iface,
             CTXARG_T dwSourceContext, ULONG uCharacterOffset, ULONG uNumChars, IDebugDocumentContext **ppsc)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
     FIXME("(%p)->(%s %u %u %p)\n", This, wine_dbgstr_longlong(dwSourceContext), uCharacterOffset,
           uNumChars, ppsc);
     return E_NOTIMPL;
@@ -509,7 +509,7 @@ static HRESULT WINAPI ActiveScriptSiteDebug_GetDocumentContextFromPosition(IActi
 
 static HRESULT WINAPI ActiveScriptSiteDebug_GetApplication(IActiveScriptSiteDebug *iface, IDebugApplication **ppda)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
     FIXME("(%p)->(%p)\n", This, ppda);
     return E_NOTIMPL;
 }
@@ -517,7 +517,7 @@ static HRESULT WINAPI ActiveScriptSiteDebug_GetApplication(IActiveScriptSiteDebu
 static HRESULT WINAPI ActiveScriptSiteDebug_GetRootApplicationNode(IActiveScriptSiteDebug *iface,
             IDebugApplicationNode **ppdanRoot)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
     FIXME("(%p)->(%p)\n", This, ppdanRoot);
     return E_NOTIMPL;
 }
@@ -525,13 +525,11 @@ static HRESULT WINAPI ActiveScriptSiteDebug_GetRootApplicationNode(IActiveScript
 static HRESULT WINAPI ActiveScriptSiteDebug_OnScriptErrorDebug(IActiveScriptSiteDebug *iface,
             IActiveScriptErrorDebug *pErrorDebug, BOOL *pfEnterDebugger, BOOL *pfCallOnScriptErrorWhenContinuing)
 {
-    ScriptHost *This = ACTSCPDBG_THIS(iface);
+    ScriptHost *This = impl_from_IActiveScriptSiteDebug(iface);
     FIXME("(%p)->(%p %p %p)\n", This, pErrorDebug, pfEnterDebugger, pfCallOnScriptErrorWhenContinuing);
     return E_NOTIMPL;
 }
 
-#undef ACTSCPDBG_THIS
-
 static const IActiveScriptSiteDebugVtbl ActiveScriptSiteDebugVtbl = {
     ActiveScriptSiteDebug_QueryInterface,
     ActiveScriptSiteDebug_AddRef,
@@ -542,30 +540,33 @@ static const IActiveScriptSiteDebugVtbl ActiveScriptSiteDebugVtbl = {
     ActiveScriptSiteDebug_OnScriptErrorDebug
 };
 
-#define SERVPROV_THIS(iface) DEFINE_THIS(ScriptHost, ServiceProvider, iface)
+static inline ScriptHost *impl_from_IServiceProvider(IServiceProvider *iface)
+{
+    return CONTAINING_RECORD(iface, ScriptHost, IServiceProvider_iface);
+}
 
 static HRESULT WINAPI ASServiceProvider_QueryInterface(IServiceProvider *iface, REFIID riid, void **ppv)
 {
-    ScriptHost *This = SERVPROV_THIS(iface);
-    return IActiveScriptSite_QueryInterface(ACTSCPSITE(This), riid, ppv);
+    ScriptHost *This = impl_from_IServiceProvider(iface);
+    return IActiveScriptSite_QueryInterface(&This->IActiveScriptSite_iface, riid, ppv);
 }
 
 static ULONG WINAPI ASServiceProvider_AddRef(IServiceProvider *iface)
 {
-    ScriptHost *This = SERVPROV_THIS(iface);
-    return IActiveScriptSite_AddRef(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IServiceProvider(iface);
+    return IActiveScriptSite_AddRef(&This->IActiveScriptSite_iface);
 }
 
 static ULONG WINAPI ASServiceProvider_Release(IServiceProvider *iface)
 {
-    ScriptHost *This = SERVPROV_THIS(iface);
-    return IActiveScriptSite_Release(ACTSCPSITE(This));
+    ScriptHost *This = impl_from_IServiceProvider(iface);
+    return IActiveScriptSite_Release(&This->IActiveScriptSite_iface);
 }
 
 static HRESULT WINAPI ASServiceProvider_QueryService(IServiceProvider *iface, REFGUID guidService,
         REFIID riid, void **ppv)
 {
-    ScriptHost *This = SERVPROV_THIS(iface);
+    ScriptHost *This = impl_from_IServiceProvider(iface);
 
     if(IsEqualGUID(&SID_SInternetHostSecurityManager, guidService)) {
         TRACE("(%p)->(SID_SInternetHostSecurityManager)\n", This);
@@ -573,15 +574,14 @@ static HRESULT WINAPI ASServiceProvider_QueryService(IServiceProvider *iface, RE
         if(!This->window || !This->window->doc)
             return E_NOINTERFACE;
 
-        return IInternetHostSecurityManager_QueryInterface(HOSTSECMGR(This->window->doc), riid, ppv);
+        return IInternetHostSecurityManager_QueryInterface(&This->window->doc->IInternetHostSecurityManager_iface,
+                riid, ppv);
     }
 
     FIXME("(%p)->(%s %s %p)\n", This, debugstr_guid(guidService), debugstr_guid(riid), ppv);
     return E_NOINTERFACE;
 }
 
-#undef SERVPROV_THIS
-
 static const IServiceProviderVtbl ASServiceProviderVtbl = {
     ASServiceProvider_QueryInterface,
     ASServiceProvider_AddRef,
@@ -595,11 +595,11 @@ static ScriptHost *create_script_host(HTMLWindow *window, const GUID *guid)
     HRESULT hres;
 
     ret = heap_alloc_zero(sizeof(*ret));
-    ret->lpIActiveScriptSiteVtbl               = &ActiveScriptSiteVtbl;
-    ret->lpIActiveScriptSiteInterruptPollVtbl  = &ActiveScriptSiteInterruptPollVtbl;
-    ret->lpIActiveScriptSiteWindowVtbl         = &ActiveScriptSiteWindowVtbl;
-    ret->lpIActiveScriptSiteDebugVtbl          = &ActiveScriptSiteDebugVtbl;
-    ret->lpServiceProviderVtbl                 = &ASServiceProviderVtbl;
+    ret->IActiveScriptSite_iface.lpVtbl = &ActiveScriptSiteVtbl;
+    ret->IActiveScriptSiteInterruptPoll_iface.lpVtbl = &ActiveScriptSiteInterruptPollVtbl;
+    ret->IActiveScriptSiteWindow_iface.lpVtbl = &ActiveScriptSiteWindowVtbl;
+    ret->IActiveScriptSiteDebug_iface.lpVtbl = &ActiveScriptSiteDebugVtbl;
+    ret->IServiceProvider_iface.lpVtbl = &ASServiceProviderVtbl;
     ret->ref = 1;
     ret->window = window;
     ret->script_state = SCRIPTSTATE_UNINITIALIZED;
@@ -794,11 +794,6 @@ static ScriptHost *get_script_host(HTMLWindow *window, const GUID *guid)
 {
     ScriptHost *iter;
 
-    if(IsEqualGUID(&CLSID_JScript, guid) && window->scriptmode != SCRIPTMODE_ACTIVESCRIPT) {
-        TRACE("Ignoring JScript\n");
-        return NULL;
-    }
-
     LIST_FOR_EACH_ENTRY(iter, &window->script_hosts, ScriptHost, entry) {
         if(IsEqualGUID(guid, &iter->guid))
             return iter;
@@ -817,6 +812,11 @@ void doc_insert_script(HTMLWindow *window, nsIDOMHTMLScriptElement *nsscript)
         return;
     }
 
+    if(IsEqualGUID(&CLSID_JScript, &guid) && window->scriptmode != SCRIPTMODE_ACTIVESCRIPT) {
+        TRACE("Ignoring JScript\n");
+        return;
+    }
+
     script_host = get_script_host(window, &guid);
     if(!script_host)
         return;
@@ -858,6 +858,11 @@ IDispatch *script_parse_event(HTMLWindow *window, LPCWSTR text)
         ptr = text;
     }
 
+    if(IsEqualGUID(&CLSID_JScript, &guid) && window->scriptmode != SCRIPTMODE_ACTIVESCRIPT) {
+        TRACE("Ignoring JScript\n");
+        return NULL;
+    }
+
     script_host = get_script_host(window, &guid);
     if(!script_host || !script_host->parse_proc)
         return NULL;
@@ -874,6 +879,42 @@ IDispatch *script_parse_event(HTMLWindow *window, LPCWSTR text)
     return disp;
 }
 
+HRESULT exec_script(HTMLWindow *window, const WCHAR *code, const WCHAR *lang, VARIANT *ret)
+{
+    ScriptHost *script_host;
+    EXCEPINFO ei;
+    GUID guid;
+    HRESULT hres;
+
+    static const WCHAR delimW[] = {'"',0};
+
+    if(!get_guid_from_language(lang, &guid)) {
+        WARN("Could not find script GUID\n");
+        return CO_E_CLASSSTRING;
+    }
+
+    script_host = get_script_host(window, &guid);
+    if(!script_host) {
+        FIXME("No script host\n");
+        return E_FAIL;
+    }
+
+    if(!script_host->parse) {
+        FIXME("script_host->parse == NULL\n");
+        return E_FAIL;
+    }
+
+    memset(&ei, 0, sizeof(ei));
+    TRACE(">>>\n");
+    hres = IActiveScriptParse64_ParseScriptText(script_host->parse, code, NULL, NULL, delimW, 0, 0, SCRIPTTEXT_ISVISIBLE, ret, &ei);
+    if(SUCCEEDED(hres))
+        TRACE("<<<\n");
+    else
+        WARN("<<< %08x\n", hres);
+
+    return hres;
+}
+
 IDispatch *get_script_disp(ScriptHost *script_host)
 {
     IDispatch *disp;
@@ -978,6 +1019,6 @@ void release_script_hosts(HTMLWindow *window)
         release_script_engine(iter);
         list_remove(&iter->entry);
         iter->window = NULL;
-        IActiveScript_Release(ACTSCPSITE(iter));
+        IActiveScript_Release(&iter->IActiveScriptSite_iface);
     }
 }
