@@ -154,19 +154,19 @@ static UINT UPDATE_get_dimensions( struct tagMSIVIEW *view, UINT *rows, UINT *co
     return wv->ops->get_dimensions( wv, rows, cols );
 }
 
-static UINT UPDATE_get_column_info( struct tagMSIVIEW *view,
-                UINT n, LPWSTR *name, UINT *type, BOOL *temporary )
+static UINT UPDATE_get_column_info( struct tagMSIVIEW *view, UINT n, LPCWSTR *name,
+                                    UINT *type, BOOL *temporary, LPCWSTR *table_name )
 {
     MSIUPDATEVIEW *uv = (MSIUPDATEVIEW*)view;
     MSIVIEW *wv;
 
-    TRACE("%p %d %p %p %p\n", uv, n, name, type, temporary );
+    TRACE("%p %d %p %p %p %p\n", uv, n, name, type, temporary, table_name );
 
     wv = uv->wv;
     if( !wv )
         return ERROR_FUNCTION_FAILED;
 
-    return wv->ops->get_column_info( wv, n, name, type, temporary );
+    return wv->ops->get_column_info( wv, n, name, type, temporary, table_name );
 }
 
 static UINT UPDATE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
@@ -261,7 +261,10 @@ UINT UPDATE_CreateView( MSIDATABASE *db, MSIVIEW **view, LPCWSTR table,
 
     uv = msi_alloc_zero( sizeof *uv );
     if( !uv )
+    {
+        wv->ops->delete( wv );
         return ERROR_FUNCTION_FAILED;
+    }
 
     /* fill the structure */
     uv->view.ops = &update_ops;
