@@ -328,11 +328,15 @@ static UINT WHERE_evaluate( MSIWHEREVIEW *wv, UINT row,
     {
     case EXPR_COL_NUMBER:
         r = wv->table->ops->fetch_int( wv->table, row, cond->u.col_number, &tval );
+        if( r != ERROR_SUCCESS )
+            return r;
         *val = tval - 0x8000;
         return ERROR_SUCCESS;
 
     case EXPR_COL_NUMBER32:
         r = wv->table->ops->fetch_int( wv->table, row, cond->u.col_number, &tval );
+        if( r != ERROR_SUCCESS )
+            return r;
         *val = tval - 0x80000000;
         return r;
 
@@ -492,9 +496,8 @@ static UINT WHERE_get_dimensions( struct tagMSIVIEW *view, UINT *rows, UINT *col
     return wv->table->ops->get_dimensions( wv->table, NULL, cols );
 }
 
-static UINT WHERE_get_column_info( struct tagMSIVIEW *view,
-                UINT n, LPWSTR *name, UINT *type, BOOL *temporary,
-                LPWSTR *table_name)
+static UINT WHERE_get_column_info( struct tagMSIVIEW *view, UINT n, LPCWSTR *name,
+                                   UINT *type, BOOL *temporary, LPCWSTR *table_name )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
 
