@@ -500,24 +500,30 @@ void CDECL MSVCP__String_base_Xinvarg(void)
 /* ?npos@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@2_KB */
 const MSVCP_size_t MSVCP_basic_string_char_npos = -1;
 
-/* Internal: basic_string_char_ptr - return pointer to stored string */
-static char* basic_string_char_ptr(basic_string_char *this)
+/* ?_Myptr@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IAEPADXZ */
+/* ?_Myptr@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IEAAPEADXZ */
+DEFINE_THISCALL_WRAPPER(basic_string_char_ptr, 4)
+char* __thiscall basic_string_char_ptr(basic_string_char *this)
 {
     if(this->res == BUF_SIZE_CHAR-1)
         return this->data.buf;
     return this->data.ptr;
 }
 
-/* Internal: basic_string_char_const_ptr - returns const pointer to stored string */
-static const char* basic_string_char_const_ptr(const basic_string_char *this)
+/* ?_Myptr@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IBEPBDXZ */
+/* ?_Myptr@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IEBAPEBDXZ */
+DEFINE_THISCALL_WRAPPER(basic_string_char_const_ptr, 4)
+const char* __thiscall basic_string_char_const_ptr(const basic_string_char *this)
 {
     if(this->res == BUF_SIZE_CHAR-1)
         return this->data.buf;
     return this->data.ptr;
 }
 
-/* Internal: basic_string_char_eos - sets string length, puts '\0' on the end */
-static void basic_string_char_eos(basic_string_char *this, MSVCP_size_t len)
+/* ?_Eos@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IAEXI@Z */
+/* ?_Eos@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IEAAX_K@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_char_eos, 8)
+void __thiscall basic_string_char_eos(basic_string_char *this, MSVCP_size_t len)
 {
     static const char nullbyte = '\0';
 
@@ -525,8 +531,10 @@ static void basic_string_char_eos(basic_string_char *this, MSVCP_size_t len)
     MSVCP_char_traits_char_assign(basic_string_char_ptr(this)+len, &nullbyte);
 }
 
-/* Internal: basic_string_char_inside - checks if given pointer points inside stored string */
-static MSVCP_bool basic_string_char_inside(
+/* ?_Inside@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IAE_NPBD@Z */
+/* ?_Inside@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IEAA_NPEBD@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_char_inside, 8)
+MSVCP_bool __thiscall basic_string_char_inside(
         basic_string_char *this, const char *ptr)
 {
     char *cstr = basic_string_char_ptr(this);
@@ -534,9 +542,10 @@ static MSVCP_bool basic_string_char_inside(
     return (ptr<cstr || ptr>=cstr+this->size) ? FALSE : TRUE;
 }
 
-/* Internal: basic_string_char_tidy - initialize basic_string buffer, deallocates data */
-/* Caution: new_size have to be smaller than BUF_SIZE_CHAR */
-static void basic_string_char_tidy(basic_string_char *this,
+/* ?_Tidy@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IAEX_NI@Z */
+/* ?_Tidy@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IEAAX_N_K@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_char_tidy, 12)
+void __thiscall basic_string_char_tidy(basic_string_char *this,
         MSVCP_bool built, MSVCP_size_t new_size)
 {
     if(built && BUF_SIZE_CHAR<=this->res) {
@@ -551,8 +560,19 @@ static void basic_string_char_tidy(basic_string_char *this,
     basic_string_char_eos(this, new_size);
 }
 
-/* Internal: basic_string_char_grow - changes size of internal buffer */
-static MSVCP_bool basic_string_char_grow(
+/* Exported only from msvcp60/70 */
+/* ?_Tidy@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@AAEX_N@Z */
+/* ?_Tidy@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@AEAAX_N@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_char_tidy_built, 8)
+void __thiscall basic_string_char_tidy_built(basic_string_char *this, MSVCP_bool built)
+{
+    return basic_string_char_tidy(this, built, 0);
+}
+
+/* ?_Grow@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IAE_NI_N@Z */
+/* ?_Grow@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@IEAA_N_K_N@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_char_grow, 12)
+MSVCP_bool __thiscall basic_string_char_grow(
         basic_string_char *this, MSVCP_size_t new_size, MSVCP_bool trim)
 {
     if(this->res < new_size) {
@@ -1042,6 +1062,54 @@ int __thiscall MSVCP_basic_string_char_compare(
             basic_string_char_const_ptr(compare), compare->size);
 }
 
+/* ??$?8DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@0@Z */
+/* ??$?8DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@0@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_char_equal(
+        const basic_string_char *left, const basic_string_char *right)
+{
+    return MSVCP_basic_string_char_compare(left, right) == 0;
+}
+
+/* ??$?8DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@PBD@Z */
+/* ??$?8DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@PEBD@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_char_equal_str_cstr(
+        const basic_string_char *left, const char *right)
+{
+    return MSVCP_basic_string_char_compare_cstr(left, right) == 0;
+}
+
+/* ??$?8DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NPBDABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@@Z */
+/* ??$?8DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NPEBDAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_char_equal_cstr_str(
+        const char *left, const basic_string_char *right)
+{
+    return MSVCP_basic_string_char_compare_cstr(right, left) == 0;
+}
+
+/* ??$?9DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@0@Z */
+/* ??$?9DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@0@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_char_not_equal(
+        const basic_string_char *left, const basic_string_char *right)
+{
+    return MSVCP_basic_string_char_compare(left, right) != 0;
+}
+
+/* ??$?9DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@PBD@Z */
+/* ??$?9DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@PEBD@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_char_not_equal_str_cstr(
+        const basic_string_char *left, const char *right)
+{
+    return MSVCP_basic_string_char_compare_cstr(left, right) != 0;
+}
+
+/* ??$?9DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NPBDABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@@Z */
+/* ??$?9DU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NPEBDAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_char_not_equal_cstr_str(
+        const char *left, const basic_string_char *right)
+{
+    return MSVCP_basic_string_char_compare_cstr(right, left) != 0;
+}
+
 /* ??$?MDU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NABV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@0@Z */
 /* ??$?MDU?$char_traits@D@std@@V?$allocator@D@1@@std@@YA_NAEBV?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@0@0@Z */
 MSVCP_bool __cdecl MSVCP_basic_string_char_lower(
@@ -1101,6 +1169,108 @@ MSVCP_size_t __thiscall MSVCP_basic_string_char_find_ch(
     return MSVCP_basic_string_char_find_cstr_substr(this, &ch, pos, 1);
 }
 
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIPBDII@Z */
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KPEBD_K1@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_first_of_cstr_substr, 16)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_first_of_cstr_substr(
+        const basic_string_char *this, const char *find, MSVCP_size_t off, MSVCP_size_t len)
+{
+    const char *p, *end;
+
+    TRACE("%p %p %lu %lu\n", this, find, off, len);
+
+    if(len>0 && off<this->size) {
+        end = basic_string_char_const_ptr(this)+this->size;
+        for(p=basic_string_char_const_ptr(this)+off; p<end; p++)
+            if(MSVCP_char_traits_char_find(find, len, p))
+                return p-basic_string_char_const_ptr(this);
+    }
+
+    return MSVCP_basic_string_char_npos;
+}
+
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIABV12@I@Z */
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KAEBV12@_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_first_of, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_first_of(
+        const basic_string_char *this, const basic_string_char *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_char_find_first_of_cstr_substr(this,
+            basic_string_char_const_ptr(find), off, find->size);
+}
+
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIPBDI@Z */
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KPEBD_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_first_of_cstr, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_first_of_cstr(
+        const basic_string_char *this, const char *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_char_find_first_of_cstr_substr(
+            this, find, off, MSVCP_char_traits_char_length(find));
+}
+
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIDI@Z */
+/* ?find_first_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KD_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_first_of_ch, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_first_of_ch(
+        const basic_string_char *this, char ch, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_char_find_first_of_cstr_substr(this, &ch, off, 1);
+}
+
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIPBDII@Z */
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KPEBD_K1@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_last_of_cstr_substr, 16)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_last_of_cstr_substr(
+        const basic_string_char *this, const char *find, MSVCP_size_t off, MSVCP_size_t len)
+{
+    const char *p, *beg;
+
+    TRACE("%p %p %lu %lu\n", this, find, off, len);
+
+
+    if(len>0 && this->size>0) {
+        if(off >= this->size)
+            off = this->size-1;
+
+        beg = basic_string_char_const_ptr(this);
+        for(p=beg+off; p>=beg; p--)
+            if(MSVCP_char_traits_char_find(find, len, p))
+                return p-beg;
+    }
+
+    return MSVCP_basic_string_char_npos;
+}
+
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIABV12@I@Z */
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KAEBV12@_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_last_of, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_last_of(
+        const basic_string_char *this, const basic_string_char *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_char_find_last_of_cstr_substr(this,
+            basic_string_char_const_ptr(find), off, find->size);
+}
+
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIPBDI@Z */
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KPEBD_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_last_of_cstr, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_last_of_cstr(
+        const basic_string_char *this, const char *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_char_find_last_of_cstr_substr(
+            this, find, off, MSVCP_char_traits_char_length(find));
+}
+
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QBEIDI@Z */
+/* ?find_last_of@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEBA_KD_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_char_find_last_of_ch, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_char_find_last_of_ch(
+        const basic_string_char *this, char ch, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_char_find_last_of_cstr_substr(this, &ch, off, 1);
+}
+
 /* ?at@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QAEAADI@Z */
 /* ?at@?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QEAAAEAD_K@Z */
 /* ??A?$basic_string@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@QAEAADI@Z */
@@ -1170,24 +1340,30 @@ void __thiscall MSVCP_basic_string_char_clear(basic_string_char *this)
 /* ?npos@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@2_KB */
 const MSVCP_size_t MSVCP_basic_string_wchar_npos = -1;
 
-/* Internal: basic_string_wchar_ptr - return pointer to stored string */
-static wchar_t* basic_string_wchar_ptr(basic_string_wchar *this)
+/* ?_Myptr@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IAEPA_WXZ */
+/* ?_Myptr@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IEAAPEA_WXZ */
+DEFINE_THISCALL_WRAPPER(basic_string_wchar_ptr, 4)
+wchar_t* __thiscall basic_string_wchar_ptr(basic_string_wchar *this)
 {
     if(this->res == BUF_SIZE_WCHAR-1)
         return this->data.buf;
     return this->data.ptr;
 }
 
-/* Internal: basic_string_wchar_const_ptr - returns const pointer to stored string */
-static const wchar_t* basic_string_wchar_const_ptr(const basic_string_wchar *this)
+/* ?_Myptr@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IBEPB_WXZ */
+/* ?_Myptr@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IEBAPEB_WXZ */
+DEFINE_THISCALL_WRAPPER(basic_string_wchar_const_ptr, 4)
+const wchar_t* __thiscall basic_string_wchar_const_ptr(const basic_string_wchar *this)
 {
     if(this->res == BUF_SIZE_WCHAR-1)
         return this->data.buf;
     return this->data.ptr;
 }
 
-/* Internal: basic_string_wchar_eos - sets string length, puts '\0' on the end */
-static void basic_string_wchar_eos(basic_string_wchar *this, MSVCP_size_t len)
+/* ?_Eos@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IAEXI@Z */
+/* ?_Eos@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IEAAX_K@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_wchar_eos, 8)
+void __thiscall basic_string_wchar_eos(basic_string_wchar *this, MSVCP_size_t len)
 {
     static const wchar_t nullbyte_w = '\0';
 
@@ -1195,8 +1371,10 @@ static void basic_string_wchar_eos(basic_string_wchar *this, MSVCP_size_t len)
     MSVCP_char_traits_wchar_assign(basic_string_wchar_ptr(this)+len, &nullbyte_w);
 }
 
-/* Internal: basic_string_char_inside - checks if given pointer points inside stored string */
-static MSVCP_bool basic_string_wchar_inside(
+/* ?_Inside@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IAE_NPB_W@Z */
+/* ?_Inside@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IEAA_NPEB_W@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_wchar_inside, 8)
+MSVCP_bool __thiscall basic_string_wchar_inside(
         basic_string_wchar *this, const wchar_t *ptr)
 {
     wchar_t *cstr = basic_string_wchar_ptr(this);
@@ -1204,9 +1382,10 @@ static MSVCP_bool basic_string_wchar_inside(
     return (ptr<cstr || ptr>=cstr+this->size) ? FALSE : TRUE;
 }
 
-/* Internal: basic_string_char_tidy - initialize basic_string buffer, deallocates data */
-/* Caution: new_size have to be smaller than BUF_SIZE_WCHAR */
-static void basic_string_wchar_tidy(basic_string_wchar *this,
+/* ?_Tidy@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IAEX_NI@Z */
+/* ?_Tidy@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IEAAX_N_K@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_wchar_tidy, 12)
+void __thiscall basic_string_wchar_tidy(basic_string_wchar *this,
         MSVCP_bool built, MSVCP_size_t new_size)
 {
     if(built && BUF_SIZE_WCHAR<=this->res) {
@@ -1221,8 +1400,10 @@ static void basic_string_wchar_tidy(basic_string_wchar *this,
     basic_string_wchar_eos(this, new_size);
 }
 
-/* Internal: basic_string_wchar_grow - changes size of internal buffer */
-static MSVCP_bool basic_string_wchar_grow(
+/* ?_Grow@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IAE_NI_N@Z */
+/* ?_Grow@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@IEAA_N_K_N@Z */
+DEFINE_THISCALL_WRAPPER(basic_string_wchar_grow, 12)
+MSVCP_bool __thiscall basic_string_wchar_grow(
         basic_string_wchar *this, MSVCP_size_t new_size, MSVCP_bool trim)
 {
     if(this->res < new_size) {
@@ -1710,6 +1891,54 @@ int __thiscall MSVCP_basic_string_wchar_compare(
             basic_string_wchar_const_ptr(compare), compare->size);
 }
 
+/* ??$?8_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@0@Z */
+/* ??$?8_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@0@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_wchar_equal(
+        const basic_string_wchar *left, const basic_string_wchar *right)
+{
+    return MSVCP_basic_string_wchar_compare(left, right) == 0;
+}
+
+/* ??$?8_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@PB_W@Z */
+/* ??$?8_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@PEB_W@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_wchar_equal_str_cstr(
+        const basic_string_wchar *left, const wchar_t *right)
+{
+    return MSVCP_basic_string_wchar_compare_cstr(left, right) == 0;
+}
+
+/* ??$?8_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NPB_WABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@@Z */
+/* ??$?8_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NPEB_WAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_wchar_equal_cstr_str(
+        const wchar_t *left, const basic_string_wchar *right)
+{
+    return MSVCP_basic_string_wchar_compare_cstr(right, left) == 0;
+}
+
+/* ??$?9_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@0@Z */
+/* ??$?9_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@0@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_wchar_not_equal(
+        const basic_string_wchar *left, const basic_string_wchar *right)
+{
+    return MSVCP_basic_string_wchar_compare(left, right) != 0;
+}
+
+/* ??$?9_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@PB_W@Z */
+/* ??$?9_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@PEB_W@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_wchar_not_equal_str_cstr(
+        const basic_string_wchar *left, const wchar_t *right)
+{
+    return MSVCP_basic_string_wchar_compare_cstr(left, right) != 0;
+}
+
+/* ??$?9_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NPB_WABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@@Z */
+/* ??$?9_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NPEB_WAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@@Z */
+MSVCP_bool __cdecl MSVCP_basic_string_wchar_not_equal_cstr_str(
+        const wchar_t *left, const basic_string_wchar *right)
+{
+    return MSVCP_basic_string_wchar_compare_cstr(right, left) != 0;
+}
+
 /* ??$?M_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NABV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@0@Z */
 /* ??$?M_WU?$char_traits@_W@std@@V?$allocator@_W@1@@std@@YA_NAEBV?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@0@0@Z */
 MSVCP_bool __cdecl MSVCP_basic_string_wchar_lower(
@@ -1769,6 +1998,108 @@ MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_ch(
     return MSVCP_basic_string_wchar_find_cstr_substr(this, &ch, pos, 1);
 }
 
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEIPB_WII@Z */
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_KPEB_W_K1@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_first_of_cstr_substr, 16)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_first_of_cstr_substr(
+        const basic_string_wchar *this, const wchar_t *find, MSVCP_size_t off, MSVCP_size_t len)
+{
+    const wchar_t *p, *end;
+
+    TRACE("%p %p %lu %lu\n", this, find, off, len);
+
+    if(len>0 && off<this->size) {
+        end = basic_string_wchar_const_ptr(this)+this->size;
+        for(p=basic_string_wchar_const_ptr(this)+off; p<end; p++)
+            if(MSVCP_char_traits_wchar_find(find, len, p))
+                return p-basic_string_wchar_const_ptr(this);
+    }
+
+    return MSVCP_basic_string_wchar_npos;
+}
+
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEIABV12@I@Z */
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_KAEBV12@_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_first_of, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_first_of(
+        const basic_string_wchar *this, const basic_string_wchar *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_wchar_find_first_of_cstr_substr(this,
+            basic_string_wchar_const_ptr(find), off, find->size);
+}
+
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEIPB_WI@Z */
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_KPEB_W_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_first_of_cstr, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_first_of_cstr(
+        const basic_string_wchar *this, const wchar_t *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_wchar_find_first_of_cstr_substr(
+            this, find, off, MSVCP_char_traits_wchar_length(find));
+}
+
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEI_WI@Z */
+/* ?find_first_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_K_W_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_first_of_ch, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_first_of_ch(
+        const basic_string_wchar *this, wchar_t ch, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_wchar_find_first_of_cstr_substr(this, &ch, off, 1);
+}
+
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEIPB_WII@Z */
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_KPEB_W_K1@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_last_of_cstr_substr, 16)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_last_of_cstr_substr(
+        const basic_string_wchar *this, const wchar_t *find, MSVCP_size_t off, MSVCP_size_t len)
+{
+    const wchar_t *p, *beg;
+
+    TRACE("%p %p %lu %lu\n", this, find, off, len);
+
+
+    if(len>0 && this->size>0) {
+        if(off >= this->size)
+            off = this->size-1;
+
+        beg = basic_string_wchar_const_ptr(this);
+        for(p=beg+off; p>=beg; p--)
+            if(MSVCP_char_traits_wchar_find(find, len, p))
+                return p-beg;
+    }
+
+    return MSVCP_basic_string_wchar_npos;
+}
+
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEIABV12@I@Z */
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_KAEBV12@_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_last_of, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_last_of(
+        const basic_string_wchar *this, const basic_string_wchar *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_wchar_find_last_of_cstr_substr(this,
+            basic_string_wchar_const_ptr(find), off, find->size);
+}
+
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEIPB_WI@Z */
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_KPEB_W_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_last_of_cstr, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_last_of_cstr(
+        const basic_string_wchar *this, const wchar_t *find, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_wchar_find_last_of_cstr_substr(
+            this, find, off, MSVCP_char_traits_wchar_length(find));
+}
+
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QBEI_WI@Z */
+/* ?find_last_of@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEBA_K_W_K@Z */
+DEFINE_THISCALL_WRAPPER(MSVCP_basic_string_wchar_find_last_of_ch, 12)
+MSVCP_size_t __thiscall MSVCP_basic_string_wchar_find_last_of_ch(
+        const basic_string_wchar *this, wchar_t ch, MSVCP_size_t off)
+{
+    return MSVCP_basic_string_wchar_find_last_of_cstr_substr(this, &ch, off, 1);
+}
+
 /* ?at@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QAEAA_WI@Z */
 /* ?at@?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QEAAAEA_W_K@Z */
 /* ??A?$basic_string@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@QAEAA_WI@Z */
