@@ -1012,6 +1012,7 @@ static void test_mxwriter_properties(void)
     hr = IMXWriter_get_encoding(writer, &str);
     ok(hr == S_OK, "got %08x\n", hr);
     ok(lstrcmpW(str, utf16W) == 0, "expected empty string, got %s\n", wine_dbgstr_w(str));
+    SysFreeString(str);
 
     /* invalid encoding name */
     str = SysAllocString(testW);
@@ -1025,6 +1026,7 @@ static void test_mxwriter_properties(void)
     hr = IMXWriter_get_version(writer, &str);
     ok(hr == S_OK, "got %08x\n", hr);
     ok(!lstrcmpW(str, _bstr_("1.0")), "got %s\n", wine_dbgstr_w(str));
+    SysFreeString(str);
 
     IMXWriter_Release(writer);
     free_bstrs();
@@ -1048,7 +1050,7 @@ static void test_mxwriter_flush(void)
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
     EXPECT_REF(stream, 1);
 
-    /* detach whe nothing was attached */
+    /* detach when nothing was attached */
     V_VT(&dest) = VT_EMPTY;
     hr = IMXWriter_put_output(writer, dest);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
@@ -1143,7 +1145,7 @@ static void test_mxwriter_startenddocument(void)
     /* now try another startDocument */
     hr = ISAXContentHandler_startDocument(content);
     ok(hr == S_OK, "got %08x\n", hr);
-    /* and get duplcated prolog */
+    /* and get duplicated prolog */
     V_VT(&dest) = VT_EMPTY;
     hr = IMXWriter_get_output(writer, &dest);
     ok(hr == S_OK, "got %08x\n", hr);
