@@ -90,6 +90,28 @@
 #define IDS_COMMON_PICTURES         69
 #define IDS_COMMON_VIDEO            70
 #define IDS_CDBURN_AREA             71
+#define IDS_PROGRAM_FILESX86        72
+#define IDS_PROGRAM_FILES_COMMONX86 73
+#define IDS_CONTACTS                74
+#define IDS_LINKS                   75
+#define IDS_PHOTO_ALBUMS            76
+#define IDS_PICTURES_LIBRARY        77
+#define IDS_PLAYLISTS               78
+#define IDS_PUBLIC_DOWNLOADS        79
+#define IDS_PUBLIC_GAME_TASKS       84
+#define IDS_PUBLIC_LIBRARIES        85
+#define IDS_PUBLIC_RINGTONES        86
+#define IDS_SAMPLE_MUSIC            87
+#define IDS_SAMPLE_PICTURES         88
+#define IDS_SAMPLE_PLAYLISTS        89
+#define IDS_SAMPLE_VIDEOS           90
+#define IDS_SAVED_GAMES             91
+#define IDS_SAVED_SEARCHES          92
+#define IDS_USER_PROFILES           93
+#define IDS_COMMON_OEM_LINKS        94
+#define IDS_DOCUMENTS               95
+#define IDS_DOWNLOADS               96
+#define IDS_LOCAL_APPDATA_LOW       97
 
 #define IDS_CREATEFOLDER_DENIED     128
 #define IDS_CREATEFOLDER_CAPTION    129
@@ -115,9 +137,18 @@
 #define IDS_RUNDLG_ERROR            160
 #define IDS_RUNDLG_BROWSE_ERROR     161
 #define IDS_RUNDLG_BROWSE_CAPTION   162
-#define IDS_RUNDLG_BROWSE_FILTER    163
+#define IDS_RUNDLG_BROWSE_FILTER_EXE 163
+#define IDS_RUNDLG_BROWSE_FILTER_ALL 164
+
+#define IDS_SHLEXEC_NOASSOC         165
+
+#define IDS_RECYCLEBIN_ERASEITEM         166
+#define IDS_RECYCLEBIN_ERASEMULTIPLE     167
+#define IDS_RECYCLEBIN_ERASE_CAPTION     168
+#define IDS_RECYCLEBIN_OVERWRITEFILE     169
+#define IDS_RECYCLEBIN_OVERWRITEFOLDER   170
+#define IDS_RECYCLEBIN_OVERWRITE_CAPTION 171
 
-#define IDS_SHLEXEC_NOASSOC         164
 
 #define IDS_LICENSE                 256
 #define IDS_LICENSE_CAPTION         257
@@ -129,6 +160,10 @@
 #define IDM_CPANEL_ABOUT            202
 #define IDM_CPANEL_APPLET_BASE      210
 
+#define MENU_RECYCLEBIN             300
+#define IDM_RECYCLEBIN_RESTORE      301
+#define IDM_RECYCLEBIN_ERASE        302
+
 /* Note: this string is referenced from the registry*/
 #define IDS_RECYCLEBIN_FOLDER_NAME   8964
 
