@@ -469,7 +469,7 @@ static HRESULT get_zone_from_domains(LPCWSTR url, LPCWSTR schema, DWORD *zone)
      * The reason is with how CoInternetParseUrl handles unknown scheme types
      * when it's parsing the domain of a URL (IE it always returns E_FAIL).
      *
-     * Windows doesn't compenstate for this and simply doesn't check if
+     * Windows doesn't compensate for this and simply doesn't check if
      * the URL maps into any domains.
      */
     if(hres != S_OK) {
@@ -1823,12 +1823,6 @@ HRESULT WINAPI CoInternetGetSecurityUrlEx(IUri *pUri, IUri **ppSecUri, PSUACTION
     if(FAILED(hres))
         return hres;
 
-    hres = CreateUri(ret_url, Uri_CREATE_ALLOW_IMPLICIT_WILDCARD_SCHEME, 0, ppSecUri);
-    if(FAILED(hres)) {
-        CoTaskMemFree(ret_url);
-        return hres;
-    }
-
     /* File URIs have to hierarchical. */
     hres = IUri_GetScheme(pUri, (DWORD*)&scheme_type);
     if(SUCCEEDED(hres) && scheme_type == URL_SCHEME_FILE) {
