@@ -57,8 +57,8 @@ static const WCHAR szDllName[] = {'D','l','l','N','a','m','e','\0'};
 
 static const WCHAR szIniDocumentation[] = {'d','o','c','u','m','e','n','t','a','t','i','o','n','\0'};
 
-HINSTANCE hDllInst;
-ATOM atDialogThemeEnabled;
+DECLSPEC_HIDDEN HINSTANCE hDllInst;
+DECLSPEC_HIDDEN ATOM atDialogThemeEnabled;
 
 static DWORD dwThemeAppProperties = STAP_ALLOW_NONCLIENT | STAP_ALLOW_CONTROLS;
 static ATOM atWindowTheme;
