@@ -229,19 +229,22 @@ static HRESULT alsa_get_card_devices(EDataFlow flow, WCHAR **ids, char **keys,
     static const WCHAR dashW[] = {' ','-',' ',0};
     int err, device;
     snd_pcm_info_t *info;
+    snd_pcm_stream_t stream = (flow == eRender ? SND_PCM_STREAM_PLAYBACK :
+        SND_PCM_STREAM_CAPTURE);
 
     info = HeapAlloc(GetProcessHeap(), 0, snd_pcm_info_sizeof());
     if(!info)
         return E_OUTOFMEMORY;
 
     snd_pcm_info_set_subdevice(info, 0);
-    snd_pcm_info_set_stream(info,
-            flow == eRender ? SND_PCM_STREAM_PLAYBACK : SND_PCM_STREAM_CAPTURE);
+    snd_pcm_info_set_stream(info, stream);
 
     device = -1;
     for(err = snd_ctl_pcm_next_device(ctl, &device); device != -1 && err >= 0;
             err = snd_ctl_pcm_next_device(ctl, &device)){
         const char *devname;
+        char devnode[32];
+        snd_pcm_t *handle;
 
         snd_pcm_info_set_device(info, device);
 
@@ -255,6 +258,15 @@ static HRESULT alsa_get_card_devices(EDataFlow flow, WCHAR **ids, char **keys,
             continue;
         }
 
+        sprintf(devnode, "hw:%d,%d", card, device);
+        if((err = snd_pcm_open(&handle, devnode, stream, SND_PCM_NONBLOCK)) < 0){
+            WARN("The device \"%s\" failed to open, pretending it doesn't exist: %d (%s)\n",
+                    devnode, err, snd_strerror(err));
+            continue;
+        }
+
+        snd_pcm_close(handle);
+
         if(ids && keys){
             DWORD len, cardlen;
 
@@ -286,7 +298,7 @@ static HRESULT alsa_get_card_devices(EDataFlow flow, WCHAR **ids, char **keys,
                 HeapFree(GetProcessHeap(), 0, ids[*num]);
                 return E_OUTOFMEMORY;
             }
-            sprintf(keys[*num], "hw:%d,%d", card, device);
+            memcpy(keys[*num], devnode, sizeof(devnode));
         }
 
         ++(*num);
@@ -395,12 +407,96 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, char ***keys,
     return S_OK;
 }
 
+/* Using the pulse PCM device from alsa-plugins 1.0.24 triggers a bug
+ * which causes audio to cease playing after a few minutes of playback.
+ * Setting handle_underrun=1 on pulse-backed ALSA devices seems to work
+ * around this issue. */
+static snd_config_t *make_handle_underrun_config(const char *name)
+{
+    snd_config_t *lconf, *dev_node, *hu_node, *type_node;
+    char dev_node_name[64];
+    const char *type_str;
+    int err;
+
+    snd_config_update();
+
+    if((err = snd_config_copy(&lconf, snd_config)) < 0){
+        WARN("snd_config_copy failed: %d (%s)\n", err, snd_strerror(err));
+        return NULL;
+    }
+
+    sprintf(dev_node_name, "pcm.%s", name);
+    err = snd_config_search(lconf, dev_node_name, &dev_node);
+    if(err == -ENOENT){
+        snd_config_delete(lconf);
+        return NULL;
+    }
+    if(err < 0){
+        snd_config_delete(lconf);
+        WARN("snd_config_search failed: %d (%s)\n", err, snd_strerror(err));
+        return NULL;
+    }
+
+    /* ALSA is extremely fragile. If it runs into a config setting it doesn't
+     * recognize, it tends to fail or assert. So we only want to inject
+     * handle_underrun=1 on devices that we know will recognize it. */
+    err = snd_config_search(dev_node, "type", &type_node);
+    if(err == -ENOENT){
+        snd_config_delete(lconf);
+        return NULL;
+    }
+    if(err < 0){
+        snd_config_delete(lconf);
+        WARN("snd_config_search failed: %d (%s)\n", err, snd_strerror(err));
+        return NULL;
+    }
+
+    if((err = snd_config_get_string(type_node, &type_str)) < 0){
+        snd_config_delete(lconf);
+        return NULL;
+    }
+
+    if(strcmp(type_str, "pulse") != 0){
+        snd_config_delete(lconf);
+        return NULL;
+    }
+
+    err = snd_config_search(dev_node, "handle_underrun", &hu_node);
+    if(err >= 0){
+        /* user already has an explicit handle_underrun setting, so don't
+         * use a local config */
+        snd_config_delete(lconf);
+        return NULL;
+    }
+    if(err != -ENOENT){
+        snd_config_delete(lconf);
+        WARN("snd_config_search failed: %d (%s)\n", err, snd_strerror(err));
+        return NULL;
+    }
+
+    if((err = snd_config_imake_integer(&hu_node, "handle_underrun", 1)) < 0){
+        snd_config_delete(lconf);
+        WARN("snd_config_imake_integer failed: %d (%s)\n", err,
+                snd_strerror(err));
+        return NULL;
+    }
+
+    if((err = snd_config_add(dev_node, hu_node)) < 0){
+        snd_config_delete(lconf);
+        WARN("snd_config_add failed: %d (%s)\n", err, snd_strerror(err));
+        return NULL;
+    }
+
+    return lconf;
+}
+
 HRESULT WINAPI AUDDRV_GetAudioEndpoint(const char *key, IMMDevice *dev,
         EDataFlow dataflow, IAudioClient **out)
 {
     ACImpl *This;
     int err;
     snd_pcm_stream_t stream;
+    snd_config_t *lconf;
 
     TRACE("\"%s\" %p %d %p\n", key, dev, dataflow, out);
 
@@ -424,13 +520,27 @@ HRESULT WINAPI AUDDRV_GetAudioEndpoint(const char *key, IMMDevice *dev,
         return E_UNEXPECTED;
     }
 
+    lconf = make_handle_underrun_config(key);
+
     This->dataflow = dataflow;
-    if((err = snd_pcm_open(&This->pcm_handle, key, stream,
-                    SND_PCM_NONBLOCK)) < 0){
-        HeapFree(GetProcessHeap(), 0, This);
-        WARN("Unable to open PCM \"%s\": %d (%s)\n", key, err,
-                snd_strerror(err));
-        return E_FAIL;
+    if(lconf){
+        if((err = snd_pcm_open_lconf(&This->pcm_handle, key, stream,
+                        SND_PCM_NONBLOCK, lconf)) < 0){
+            snd_config_delete(lconf);
+            HeapFree(GetProcessHeap(), 0, This);
+            WARN("Unable to open PCM \"%s\": %d (%s)\n", key, err,
+                    snd_strerror(err));
+            return E_FAIL;
+        }
+        snd_config_delete(lconf);
+    }else{
+        if((err = snd_pcm_open(&This->pcm_handle, key, stream,
+                        SND_PCM_NONBLOCK)) < 0){
+            HeapFree(GetProcessHeap(), 0, This);
+            WARN("Unable to open PCM \"%s\": %d (%s)\n", key, err,
+                    snd_strerror(err));
+            return E_FAIL;
+        }
     }
 
     This->hw_params = HeapAlloc(GetProcessHeap(), 0,
@@ -787,6 +897,8 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
         goto exit;
     }
 
+    if(!duration)
+        duration = 300000; /* 0.03s */
     This->bufsize_frames = ceil((duration / 10000000.) * fmt->nSamplesPerSec);
     This->local_buffer = HeapAlloc(GetProcessHeap(), 0,
             This->bufsize_frames * fmt->nBlockAlign);
@@ -828,6 +940,14 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
         goto exit;
     }
 
+    if((err = snd_pcm_sw_params_set_silence_size(This->pcm_handle,
+                    sw_params, boundary)) < 0){
+        WARN("Unable to set silence size to %lx: %d (%s)\n", boundary, err,
+                snd_strerror(err));
+        hr = E_FAIL;
+        goto exit;
+    }
+
     if((err = snd_pcm_sw_params(This->pcm_handle, sw_params)) < 0){
         WARN("Unable to set sw params: %d (%s)\n", err, snd_strerror(err));
         hr = E_FAIL;
@@ -1863,8 +1983,7 @@ static HRESULT WINAPI AudioRenderClient_GetBuffer(IAudioRenderClient *iface,
         *data = This->tmp_buffer;
         This->buf_state = LOCKED_WRAPPED;
     }else{
-        *data = This->local_buffer +
-            This->lcl_offs_frames * This->fmt->nBlockAlign;
+        *data = This->local_buffer + write_pos * This->fmt->nBlockAlign;
         This->buf_state = LOCKED_NORMAL;
     }
 
@@ -1910,7 +2029,7 @@ static HRESULT WINAPI AudioRenderClient_ReleaseBuffer(
 
     if(This->buf_state == LOCKED_NORMAL)
         buffer = This->local_buffer +
-            This->lcl_offs_frames * This->fmt->nBlockAlign;
+            (This->lcl_offs_frames + This->held_frames) * This->fmt->nBlockAlign;
     else
         buffer = This->tmp_buffer;
 
@@ -2965,7 +3084,7 @@ static const IChannelAudioVolumeVtbl ChannelAudioVolume_Vtbl =
     ChannelAudioVolume_GetAllVolumes
 };
 
-HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
+static HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
         REFIID riid, void **ppv)
 {
     TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppv);
@@ -2987,7 +3106,7 @@ HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
     return E_NOINTERFACE;
 }
 
-ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
+static ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
     ULONG ref;
@@ -2996,7 +3115,7 @@ ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
     return ref;
 }
 
-ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
+static ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
     ULONG ref;
@@ -3007,7 +3126,7 @@ ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
     return ref;
 }
 
-HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
+static HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
         IAudioSessionManager2 *iface, const GUID *session_guid, DWORD flags,
         IAudioSessionControl **out)
 {
@@ -3034,7 +3153,7 @@ HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
     return S_OK;
 }
 
-HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
+static HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
         IAudioSessionManager2 *iface, const GUID *session_guid, DWORD flags,
         ISimpleAudioVolume **out)
 {
@@ -3061,7 +3180,7 @@ HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
     return S_OK;
 }
 
-HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
+static HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
         IAudioSessionManager2 *iface, IAudioSessionEnumerator **out)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -3069,7 +3188,7 @@ HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
+static HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
         IAudioSessionManager2 *iface, IAudioSessionNotification *notification)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -3077,7 +3196,7 @@ HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
+static HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
         IAudioSessionManager2 *iface, IAudioSessionNotification *notification)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -3085,7 +3204,7 @@ HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
+static HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
         IAudioSessionManager2 *iface, const WCHAR *session_id,
         IAudioVolumeDuckNotification *notification)
 {
@@ -3094,7 +3213,7 @@ HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_UnregisterDuckNotification(
+static HRESULT WINAPI AudioSessionManager_UnregisterDuckNotification(
         IAudioSessionManager2 *iface,
         IAudioVolumeDuckNotification *notification)
 {
