@@ -862,6 +862,8 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
     }else
         This->period_ms = MinimumPeriod / 10000;
 
+    if(!duration)
+        duration = 300000; /* 0.03s */
     This->bufsize_frames = ceil(fmt->nSamplesPerSec * (duration / 10000000.));
 
     if(This->dataflow == eCapture){
@@ -2191,7 +2193,7 @@ static HRESULT WINAPI AudioSessionControl_GetGroupingParam(
 }
 
 static HRESULT WINAPI AudioSessionControl_SetGroupingParam(
-        IAudioSessionControl2 *iface, GUID *group, const GUID *session)
+        IAudioSessionControl2 *iface, const GUID *group, const GUID *session)
 {
     AudioSessionWrapper *This = impl_from_IAudioSessionControl2(iface);
 
@@ -2790,7 +2792,7 @@ static const IChannelAudioVolumeVtbl ChannelAudioVolume_Vtbl =
     ChannelAudioVolume_GetAllVolumes
 };
 
-HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
+static HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
         REFIID riid, void **ppv)
 {
     TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppv);
@@ -2812,7 +2814,7 @@ HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
     return E_NOINTERFACE;
 }
 
-ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
+static ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
     ULONG ref;
@@ -2821,7 +2823,7 @@ ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
     return ref;
 }
 
-ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
+static ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
     ULONG ref;
@@ -2832,7 +2834,7 @@ ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
     return ref;
 }
 
-HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
+static HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
         IAudioSessionManager2 *iface, const GUID *session_guid, DWORD flags,
         IAudioSessionControl **out)
 {
@@ -2859,7 +2861,7 @@ HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
     return S_OK;
 }
 
-HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
+static HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
         IAudioSessionManager2 *iface, const GUID *session_guid, DWORD flags,
         ISimpleAudioVolume **out)
 {
@@ -2886,7 +2888,7 @@ HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
     return S_OK;
 }
 
-HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
+static HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
         IAudioSessionManager2 *iface, IAudioSessionEnumerator **out)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -2894,7 +2896,7 @@ HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
+static HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
         IAudioSessionManager2 *iface, IAudioSessionNotification *notification)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -2902,7 +2904,7 @@ HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
+static HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
         IAudioSessionManager2 *iface, IAudioSessionNotification *notification)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -2910,7 +2912,7 @@ HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
+static HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
         IAudioSessionManager2 *iface, const WCHAR *session_id,
         IAudioVolumeDuckNotification *notification)
 {
@@ -2919,7 +2921,7 @@ HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_UnregisterDuckNotification(
+static HRESULT WINAPI AudioSessionManager_UnregisterDuckNotification(
         IAudioSessionManager2 *iface,
         IAudioVolumeDuckNotification *notification)
 {
