@@ -74,7 +74,6 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
     const DWORD               *pIdxBufL     = NULL;
     UINT vx_index;
     const struct wined3d_state *state = &device->stateBlock->state;
-    const struct wined3d_stream_state *streams = state->streams;
     LONG SkipnStrides = startIdx;
     BOOL pixelShader = use_ps(state);
     BOOL specular_fog = FALSE;
@@ -111,13 +110,13 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
     if (si->use_map & (1 << WINED3D_FFP_POSITION))
     {
         element = &si->elements[WINED3D_FFP_POSITION];
-        position = element->data + streams[element->stream_idx].offset;
+        position = element->data.addr;
     }
 
     if (si->use_map & (1 << WINED3D_FFP_NORMAL))
     {
         element = &si->elements[WINED3D_FFP_NORMAL];
-        normal = element->data + streams[element->stream_idx].offset;
+        normal = element->data.addr;
     }
     else
     {
@@ -128,7 +127,7 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
     if (si->use_map & (1 << WINED3D_FFP_DIFFUSE))
     {
         element = &si->elements[WINED3D_FFP_DIFFUSE];
-        diffuse = element->data + streams[element->stream_idx].offset;
+        diffuse = element->data.addr;
 
         if (num_untracked_materials && element->format->id != WINED3DFMT_B8G8R8A8_UNORM)
             FIXME("Implement diffuse color tracking from %s\n", debug_d3dformat(element->format->id));
@@ -141,7 +140,7 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
     if (si->use_map & (1 << WINED3D_FFP_SPECULAR))
     {
         element = &si->elements[WINED3D_FFP_SPECULAR];
-        specular = element->data + streams[element->stream_idx].offset;
+        specular = element->data.addr;
 
         /* special case where the fog density is stored in the specular alpha channel */
         if (state->render_states[WINED3DRS_FOGENABLE]
@@ -201,7 +200,7 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
         if (si->use_map & (1 << (WINED3D_FFP_TEXCOORD0 + coordIdx)))
         {
             element = &si->elements[WINED3D_FFP_TEXCOORD0 + coordIdx];
-            texCoords[coordIdx] = element->data + streams[element->stream_idx].offset;
+            texCoords[coordIdx] = element->data.addr;
             tex_mask |= (1 << textureNo);
         }
         else
@@ -465,8 +464,7 @@ static void drawStridedSlowVs(const struct wined3d_gl_info *gl_info, const struc
         {
             if (!(si->use_map & (1 << i))) continue;
 
-            ptr = si->elements[i].data + si->elements[i].stride * SkipnStrides
-                    + state->streams[si->elements[i].stream_idx].offset;
+            ptr = si->elements[i].data.addr + si->elements[i].stride * SkipnStrides;
 
             send_attribute(gl_info, si->elements[i].format->id, i, ptr);
         }
@@ -529,10 +527,9 @@ static void drawStridedInstanced(const struct wined3d_gl_info *gl_info, const st
     for(i = 0; i < numInstances; i++) {
         /* Specify the instanced attributes using immediate mode calls */
         for(j = 0; j < numInstancedAttribs; j++) {
-            const BYTE *ptr = si->elements[instancedData[j]].data
-                    + si->elements[instancedData[j]].stride * i
-                    + state->streams[si->elements[instancedData[j]].stream_idx].offset;
-            if (si->elements[instancedData[j]].buffer_object)
+            const BYTE *ptr = si->elements[instancedData[j]].data.addr
+                    + si->elements[instancedData[j]].stride * i;
+            if (si->elements[instancedData[j]].data.buffer_object)
             {
                 struct wined3d_buffer *vb = state->streams[si->elements[instancedData[j]].stream_idx].buffer;
                 ptr += (ULONG_PTR)buffer_get_sysmem(vb, gl_info);
@@ -568,11 +565,11 @@ static void remove_vbos(const struct wined3d_gl_info *gl_info,
         if (!(s->use_map & (1 << i))) continue;
 
         e = &s->elements[i];
-        if (e->buffer_object)
+        if (e->data.buffer_object)
         {
             struct wined3d_buffer *vb = state->streams[e->stream_idx].buffer;
-            e->buffer_object = 0;
-            e->data = (BYTE *)((ULONG_PTR)e->data + (ULONG_PTR)buffer_get_sysmem(vb, gl_info));
+            e->data.buffer_object = 0;
+            e->data.addr = (BYTE *)((ULONG_PTR)e->data.addr + (ULONG_PTR)buffer_get_sysmem(vb, gl_info));
         }
     }
 }
@@ -611,13 +608,6 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
         return;
     }
 
-    if (!context_apply_draw_state(context, device))
-    {
-        context_release(context);
-        WARN("Unable to apply draw state, skipping draw.\n");
-        return;
-    }
-
     if (device->fb.depth_stencil)
     {
         /* Note that this depends on the context_acquire() call above to set
@@ -639,7 +629,7 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
             else
                 SetRectEmpty(&current_rect);
 
-            device_get_draw_rect(device, &draw_rect);
+            wined3d_get_draw_rect(state, &draw_rect);
 
             IntersectRect(&r, &draw_rect, &current_rect);
             if (!EqualRect(&r, &draw_rect))
@@ -653,6 +643,13 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
         }
     }
 
+    if (!context_apply_draw_state(context, device))
+    {
+        context_release(context);
+        WARN("Unable to apply draw state, skipping draw.\n");
+        return;
+    }
+
     if ((!context->gl_info->supported[WINED3D_GL_VERSION_2_0]
             || (!glPointParameteri && !context->gl_info->supported[NV_POINT_SPRITE]))
             && context->render_offscreen
@@ -793,9 +790,11 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
 {
     unsigned int i, j, num_quads, out_vertex_size, buffer_size, d3d_out_vertex_size;
     float max_x = 0.0f, max_y = 0.0f, max_z = 0.0f, neg_z = 0.0f;
+    struct wined3d_state *state = &This->stateBlock->state;
     struct wined3d_stream_info stream_info;
     struct wined3d_stream_info_element *e;
     struct wined3d_context *context;
+    struct wined3d_shader *vs;
     const BYTE *data;
     const WINED3DRECTPATCH_INFO *info = &patch->RectPatchInfo;
     DWORD vtxStride;
@@ -808,19 +807,21 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
     context = context_acquire(This, NULL);
     context_apply_blit_state(context, This);
 
-    /* First, locate the position data. This is provided in a vertex buffer in the stateblock.
-     * Beware of vbos
-     */
-    device_stream_info_from_declaration(This, FALSE, &stream_info, NULL);
+    /* First, locate the position data. This is provided in a vertex buffer in
+     * the stateblock. Beware of VBOs. */
+    vs = state->vertex_shader;
+    state->vertex_shader = NULL;
+    device_stream_info_from_declaration(This, &stream_info, NULL);
+    state->vertex_shader = vs;
 
     e = &stream_info.elements[WINED3D_FFP_POSITION];
-    if (e->buffer_object)
+    if (e->data.buffer_object)
     {
-        struct wined3d_buffer *vb = This->stateBlock->state.streams[e->stream_idx].buffer;
-        e->data = (BYTE *)((ULONG_PTR)e->data + (ULONG_PTR)buffer_get_sysmem(vb, context->gl_info));
+        struct wined3d_buffer *vb = state->streams[e->stream_idx].buffer;
+        e->data.addr = (BYTE *)((ULONG_PTR)e->data.addr + (ULONG_PTR)buffer_get_sysmem(vb, context->gl_info));
     }
     vtxStride = e->stride;
-    data = e->data +
+    data = e->data.addr +
            vtxStride * info->Stride * info->StartVertexOffsetHeight +
            vtxStride * info->StartVertexOffsetWidth;
 
@@ -874,7 +875,7 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
      */
     glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     checkGLcall("glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)");
-    device_invalidate_state(This, STATE_RENDER(WINED3DRS_FILLMODE));
+    context_invalidate_state(context, STATE_RENDER(WINED3DRS_FILLMODE));
     if (patch->has_normals)
     {
         static const GLfloat black[] = {0.0f, 0.0f, 0.0f, 0.0f};
@@ -886,30 +887,30 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
         checkGLcall("glEnable(GL_LIGHTING)");
         glLightModelfv(GL_LIGHT_MODEL_AMBIENT, black);
         checkGLcall("glLightModel for MODEL_AMBIENT");
-        device_invalidate_state(This, STATE_RENDER(WINED3DRS_AMBIENT));
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_AMBIENT));
 
         for (i = 3; i < context->gl_info->limits.lights; ++i)
         {
             glDisable(GL_LIGHT0 + i);
             checkGLcall("glDisable(GL_LIGHT0 + i)");
-            device_invalidate_state(This, STATE_ACTIVELIGHT(i));
+            context_invalidate_state(context, STATE_ACTIVELIGHT(i));
         }
 
-        device_invalidate_state(This, STATE_ACTIVELIGHT(0));
+        context_invalidate_state(context, STATE_ACTIVELIGHT(0));
         glLightfv(GL_LIGHT0, GL_DIFFUSE, red);
         glLightfv(GL_LIGHT0, GL_SPECULAR, black);
         glLightfv(GL_LIGHT0, GL_AMBIENT, black);
         glLightfv(GL_LIGHT0, GL_POSITION, red);
         glEnable(GL_LIGHT0);
         checkGLcall("Setting up light 1");
-        device_invalidate_state(This, STATE_ACTIVELIGHT(1));
+        context_invalidate_state(context, STATE_ACTIVELIGHT(1));
         glLightfv(GL_LIGHT1, GL_DIFFUSE, green);
         glLightfv(GL_LIGHT1, GL_SPECULAR, black);
         glLightfv(GL_LIGHT1, GL_AMBIENT, black);
         glLightfv(GL_LIGHT1, GL_POSITION, green);
         glEnable(GL_LIGHT1);
         checkGLcall("Setting up light 2");
-        device_invalidate_state(This, STATE_ACTIVELIGHT(2));
+        context_invalidate_state(context, STATE_ACTIVELIGHT(2));
         glLightfv(GL_LIGHT2, GL_DIFFUSE, blue);
         glLightfv(GL_LIGHT2, GL_SPECULAR, black);
         glLightfv(GL_LIGHT2, GL_AMBIENT, black);
@@ -917,8 +918,8 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
         glEnable(GL_LIGHT2);
         checkGLcall("Setting up light 3");
 
-        device_invalidate_state(This, STATE_MATERIAL);
-        device_invalidate_state(This, STATE_RENDER(WINED3DRS_COLORVERTEX));
+        context_invalidate_state(context, STATE_MATERIAL);
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_COLORVERTEX));
         glDisable(GL_COLOR_MATERIAL);
         glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, black);
         glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, black);
