@@ -277,11 +277,10 @@ HRESULT CDECL wined3d_swapchain_get_gamma_ramp(const struct wined3d_swapchain *s
 }
 
 /* A GL context is provided by the caller */
-static void swapchain_blit(struct wined3d_swapchain *swapchain,
+static void swapchain_blit(const struct wined3d_swapchain *swapchain,
         struct wined3d_context *context, const RECT *src_rect, const RECT *dst_rect)
 {
     struct wined3d_surface *backbuffer = swapchain->back_buffers[0];
-    struct wined3d_device *device = swapchain->device;
     UINT src_w = src_rect->right - src_rect->left;
     UINT src_h = src_rect->bottom - src_rect->top;
     GLenum gl_filter;
@@ -307,18 +306,18 @@ static void swapchain_blit(struct wined3d_swapchain *swapchain,
         glReadBuffer(GL_COLOR_ATTACHMENT0);
         context_check_fbo_status(context, GL_READ_FRAMEBUFFER);
 
-        context_bind_fbo(context, GL_DRAW_FRAMEBUFFER, NULL);
+        context_apply_fbo_state_blit(context, GL_DRAW_FRAMEBUFFER, swapchain->front_buffer, NULL, SFLAG_INDRAWABLE);
         context_set_draw_buffer(context, GL_BACK);
-        device_invalidate_state(device, STATE_FRAMEBUFFER);
+        context_invalidate_state(context, STATE_FRAMEBUFFER);
 
         glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-        device_invalidate_state(device, STATE_RENDER(WINED3DRS_COLORWRITEENABLE));
-        device_invalidate_state(device, STATE_RENDER(WINED3DRS_COLORWRITEENABLE1));
-        device_invalidate_state(device, STATE_RENDER(WINED3DRS_COLORWRITEENABLE2));
-        device_invalidate_state(device, STATE_RENDER(WINED3DRS_COLORWRITEENABLE3));
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_COLORWRITEENABLE));
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_COLORWRITEENABLE1));
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_COLORWRITEENABLE2));
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_COLORWRITEENABLE3));
 
         glDisable(GL_SCISSOR_TEST);
-        device_invalidate_state(device, STATE_RENDER(WINED3DRS_SCISSORTESTENABLE));
+        context_invalidate_state(context, STATE_RENDER(WINED3DRS_SCISSORTESTENABLE));
 
         /* Note that the texture is upside down */
         gl_info->fbo_ops.glBlitFramebuffer(src_rect->left, src_rect->top, src_rect->right, src_rect->bottom,
@@ -329,6 +328,7 @@ static void swapchain_blit(struct wined3d_swapchain *swapchain,
     }
     else
     {
+        struct wined3d_device *device = swapchain->device;
         struct wined3d_context *context2;
         float tex_left = src_rect->left;
         float tex_top = src_rect->top;
@@ -350,7 +350,7 @@ static void swapchain_blit(struct wined3d_swapchain *swapchain,
             gl_filter = GL_NEAREST;
 
         ENTER_GL();
-        context_bind_fbo(context2, GL_FRAMEBUFFER, NULL);
+        context_apply_fbo_state_blit(context2, GL_FRAMEBUFFER, swapchain->front_buffer, NULL, SFLAG_INDRAWABLE);
 
         /* Set up the texture. The surface is not in a wined3d_texture
          * container, so there are no D3D texture settings to dirtify. */
@@ -644,9 +644,9 @@ static const struct wined3d_swapchain_ops swapchain_gl_ops =
 };
 
 /* Helper function that blits the front buffer contents to the target window. */
-void x11_copy_to_screen(struct wined3d_swapchain *swapchain, const RECT *rect)
+void x11_copy_to_screen(const struct wined3d_swapchain *swapchain, const RECT *rect)
 {
-    struct wined3d_surface *front;
+    const struct wined3d_surface *front;
     POINT offset = {0, 0};
     HDC src_dc, dst_dc;
     RECT draw_rect;
