@@ -28,13 +28,14 @@
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
+WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
 struct wined3d_wndproc
 {
     HWND window;
     BOOL unicode;
     WNDPROC proc;
-    IWineD3DDeviceImpl *device;
+    struct wined3d_device *device;
 };
 
 struct wined3d_wndproc_table
@@ -60,9 +61,19 @@ static CRITICAL_SECTION_DEBUG wined3d_cs_debug =
 };
 static CRITICAL_SECTION wined3d_cs = {&wined3d_cs_debug, -1, 0, 0, 0, 0};
 
+static CRITICAL_SECTION wined3d_wndproc_cs;
+static CRITICAL_SECTION_DEBUG wined3d_wndproc_cs_debug =
+{
+    0, 0, &wined3d_wndproc_cs,
+    {&wined3d_wndproc_cs_debug.ProcessLocksList,
+    &wined3d_wndproc_cs_debug.ProcessLocksList},
+    0, 0, {(DWORD_PTR)(__FILE__ ": wined3d_wndproc_cs")}
+};
+static CRITICAL_SECTION wined3d_wndproc_cs = {&wined3d_wndproc_cs_debug, -1, 0, 0, 0, 0};
+
 /* When updating default value here, make sure to update winecfg as well,
  * where appropriate. */
-wined3d_settings_t wined3d_settings =
+struct wined3d_settings wined3d_settings =
 {
     VS_HW,          /* Hardware by default */
     PS_HW,          /* Hardware by default */
@@ -78,7 +89,7 @@ wined3d_settings_t wined3d_settings =
 };
 
 /* Do not call while under the GL lock. */
-struct wined3d * CDECL wined3d_create(UINT version, void *parent)
+struct wined3d * CDECL wined3d_create(UINT version, DWORD flags, void *parent)
 {
     struct wined3d *object;
     HRESULT hr;
@@ -90,7 +101,7 @@ struct wined3d * CDECL wined3d_create(UINT version, void *parent)
         return NULL;
     }
 
-    hr = wined3d_init(object, version, parent);
+    hr = wined3d_init(object, version, flags, parent);
     if (FAILED(hr))
     {
         WARN("Failed to initialize wined3d object, hr %#x.\n", hr);
@@ -228,6 +239,7 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
         {
             if (!strcmp(buffer,"disabled"))
             {
+                ERR_(winediag)("The GLSL shader backend has been disabled. You get to keep all the pieces if it breaks.\n");
                 TRACE("Use of GL Shading Language disabled\n");
                 wined3d_settings.glslRequested = FALSE;
             }
@@ -355,8 +367,13 @@ static BOOL wined3d_dll_destroy(HINSTANCE hInstDLL)
 
     for (i = 0; i < wndproc_table.count; ++i)
     {
-        struct wined3d_wndproc *entry = &wndproc_table.entries[i];
-        SetWindowLongPtrW(entry->window, GWLP_WNDPROC, (LONG_PTR)entry->proc);
+        /* Trying to unregister these would be futile. These entries can only
+         * exist if either we skipped them in wined3d_unregister_window() due
+         * to the application replacing the wndproc after the entry was
+         * registered, or if the application still has an active wined3d
+         * device. In the latter case the application has bigger problems than
+         * these entries. */
+        WARN("Leftover wndproc table entry %p.\n", &wndproc_table.entries[i]);
     }
     HeapFree(GetProcessHeap(), 0, wndproc_table.entries);
 
@@ -376,6 +393,16 @@ void WINAPI wined3d_mutex_unlock(void)
     LeaveCriticalSection(&wined3d_cs);
 }
 
+static void wined3d_wndproc_mutex_lock(void)
+{
+    EnterCriticalSection(&wined3d_wndproc_cs);
+}
+
+static void wined3d_wndproc_mutex_unlock(void)
+{
+    LeaveCriticalSection(&wined3d_wndproc_cs);
+}
+
 static struct wined3d_wndproc *wined3d_find_wndproc(HWND window)
 {
     unsigned int i;
@@ -394,16 +421,16 @@ static struct wined3d_wndproc *wined3d_find_wndproc(HWND window)
 static LRESULT CALLBACK wined3d_wndproc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
 {
     struct wined3d_wndproc *entry;
-    IWineD3DDeviceImpl *device;
+    struct wined3d_device *device;
     BOOL unicode;
     WNDPROC proc;
 
-    wined3d_mutex_lock();
+    wined3d_wndproc_mutex_lock();
     entry = wined3d_find_wndproc(window);
 
     if (!entry)
     {
-        wined3d_mutex_unlock();
+        wined3d_wndproc_mutex_unlock();
         ERR("Window %p is not registered with wined3d.\n", window);
         return DefWindowProcW(window, message, wparam, lparam);
     }
@@ -411,20 +438,20 @@ static LRESULT CALLBACK wined3d_wndproc(HWND window, UINT message, WPARAM wparam
     device = entry->device;
     unicode = entry->unicode;
     proc = entry->proc;
-    wined3d_mutex_unlock();
+    wined3d_wndproc_mutex_unlock();
 
     return device_process_message(device, window, unicode, message, wparam, lparam, proc);
 }
 
-BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
+BOOL wined3d_register_window(HWND window, struct wined3d_device *device)
 {
     struct wined3d_wndproc *entry;
 
-    wined3d_mutex_lock();
+    wined3d_wndproc_mutex_lock();
 
     if (wined3d_find_wndproc(window))
     {
-        wined3d_mutex_unlock();
+        wined3d_wndproc_mutex_unlock();
         WARN("Window %p is already registered with wined3d.\n", window);
         return TRUE;
     }
@@ -439,7 +466,7 @@ BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
 
         if (!new_entries)
         {
-            wined3d_mutex_unlock();
+            wined3d_wndproc_mutex_unlock();
             ERR("Failed to grow table.\n");
             return FALSE;
         }
@@ -460,7 +487,7 @@ BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
         entry->proc = (WNDPROC)SetWindowLongPtrA(window, GWLP_WNDPROC, (LONG_PTR)wined3d_wndproc);
     entry->device = device;
 
-    wined3d_mutex_unlock();
+    wined3d_wndproc_mutex_unlock();
 
     return TRUE;
 }
@@ -470,11 +497,11 @@ void wined3d_unregister_window(HWND window)
     struct wined3d_wndproc *entry, *last;
     LONG_PTR proc;
 
-    wined3d_mutex_lock();
+    wined3d_wndproc_mutex_lock();
 
     if (!(entry = wined3d_find_wndproc(window)))
     {
-        wined3d_mutex_unlock();
+        wined3d_wndproc_mutex_unlock();
         ERR("Window %p is not registered with wined3d.\n", window);
         return;
     }
@@ -484,7 +511,7 @@ void wined3d_unregister_window(HWND window)
         proc = GetWindowLongPtrW(window, GWLP_WNDPROC);
         if (proc != (LONG_PTR)wined3d_wndproc)
         {
-            wined3d_mutex_unlock();
+            wined3d_wndproc_mutex_unlock();
             WARN("Not unregistering window %p, window proc %#lx doesn't match wined3d window proc %p.\n",
                     window, proc, wined3d_wndproc);
             return;
@@ -497,7 +524,7 @@ void wined3d_unregister_window(HWND window)
         proc = GetWindowLongPtrA(window, GWLP_WNDPROC);
         if (proc != (LONG_PTR)wined3d_wndproc)
         {
-            wined3d_mutex_unlock();
+            wined3d_wndproc_mutex_unlock();
             WARN("Not unregistering window %p, window proc %#lx doesn't match wined3d window proc %p.\n",
                     window, proc, wined3d_wndproc);
             return;
@@ -509,7 +536,7 @@ void wined3d_unregister_window(HWND window)
     last = &wndproc_table.entries[--wndproc_table.count];
     if (entry != last) *entry = *last;
 
-    wined3d_mutex_unlock();
+    wined3d_wndproc_mutex_unlock();
 }
 
 /* At process attach */
