@@ -233,7 +233,40 @@ BOOL WINAPI DllMain(HINSTANCE dll, DWORD reason, void *reserved)
     return TRUE;
 }
 
-HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, void ***keys,
+static UINT get_default_index(EDataFlow flow, char **keys, UINT num)
+{
+    int fd = -1, err, i;
+    oss_audioinfo ai;
+
+    if(flow == eRender)
+        fd = open("/dev/dsp", O_WRONLY);
+    else
+        fd = open("/dev/dsp", O_RDONLY);
+
+    if(fd < 0){
+        WARN("Couldn't open default device!\n");
+        return 0;
+    }
+
+    ai.dev = -1;
+    if((err = ioctl(fd, SNDCTL_ENGINEINFO, &ai)) < 0){
+        WARN("SNDCTL_ENGINEINFO failed: %d (%s)\n", err, strerror(err));
+        close(fd);
+        return 0;
+    }
+
+    close(fd);
+
+    TRACE("Default devnode: %s\n", ai.devnode);
+    for(i = 0; i < num; ++i)
+        if(!strcmp(ai.devnode, keys[i]))
+            return i;
+
+    WARN("Couldn't find default device! Choosing first.\n");
+    return 0;
+}
+
+HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, char ***keys,
         UINT *num, UINT *def_index)
 {
     int i, mixer_fd;
@@ -282,9 +315,9 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, void ***keys,
     *keys = HeapAlloc(GetProcessHeap(), 0, sysinfo.numaudios * sizeof(char *));
 
     *num = 0;
-    *def_index = -1;
     for(i = 0; i < sysinfo.numaudios; ++i){
         oss_audioinfo ai = {0};
+        int fd;
 
         ai.dev = i;
         if(ioctl(mixer_fd, SNDCTL_AUDIOINFO, &ai) < 0){
@@ -293,6 +326,17 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, void ***keys,
             continue;
         }
 
+        if(flow == eRender)
+            fd = open(ai.devnode, O_WRONLY, 0);
+        else
+            fd = open(ai.devnode, O_RDONLY, 0);
+        if(fd < 0){
+            WARN("Opening device \"%s\" failed, pretending it doesn't exist: %d (%s)\n",
+                    ai.devnode, errno, strerror(errno));
+            continue;
+        }
+        close(fd);
+
         if((flow == eCapture && (ai.caps & PCM_CAP_INPUT)) ||
                 (flow == eRender && (ai.caps & PCM_CAP_OUTPUT))){
             size_t len;
@@ -328,18 +372,14 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, void ***keys,
             MultiByteToWideChar(CP_UNIXCP, 0, ai.name, -1,
                     (*ids)[*num], len);
 
-            if(ai.caps & PCM_CAP_DEFAULT)
-                *def_index = *num;
-
             (*num)++;
         }
     }
 
-    if(*def_index == -1)
-        *def_index = 0;
-
     close(mixer_fd);
 
+    *def_index = get_default_index(flow, *keys, *num);
+
     return S_OK;
 }
 
@@ -807,6 +847,8 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
     else
         This->period_us = DefaultPeriod / 10;
 
+    if(!duration)
+        duration = 300000; /* 0.03s */
     This->bufsize_frames = ceil(fmt->nSamplesPerSec * (duration / 10000000.));
     This->local_buffer = HeapAlloc(GetProcessHeap(), 0,
             This->bufsize_frames * fmt->nBlockAlign);
@@ -2120,7 +2162,7 @@ static HRESULT WINAPI AudioSessionControl_GetGroupingParam(
 }
 
 static HRESULT WINAPI AudioSessionControl_SetGroupingParam(
-        IAudioSessionControl2 *iface, GUID *group, const GUID *session)
+        IAudioSessionControl2 *iface, const GUID *group, const GUID *session)
 {
     AudioSessionWrapper *This = impl_from_IAudioSessionControl2(iface);
 
@@ -2755,7 +2797,7 @@ static const IChannelAudioVolumeVtbl ChannelAudioVolume_Vtbl =
     ChannelAudioVolume_GetAllVolumes
 };
 
-HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
+static HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
         REFIID riid, void **ppv)
 {
     TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppv);
@@ -2777,7 +2819,7 @@ HRESULT WINAPI AudioSessionManager_QueryInterface(IAudioSessionManager2 *iface,
     return E_NOINTERFACE;
 }
 
-ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
+static ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
     ULONG ref;
@@ -2786,7 +2828,7 @@ ULONG WINAPI AudioSessionManager_AddRef(IAudioSessionManager2 *iface)
     return ref;
 }
 
-ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
+static ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
     ULONG ref;
@@ -2797,7 +2839,7 @@ ULONG WINAPI AudioSessionManager_Release(IAudioSessionManager2 *iface)
     return ref;
 }
 
-HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
+static HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
         IAudioSessionManager2 *iface, const GUID *session_guid, DWORD flags,
         IAudioSessionControl **out)
 {
@@ -2824,7 +2866,7 @@ HRESULT WINAPI AudioSessionManager_GetAudioSessionControl(
     return S_OK;
 }
 
-HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
+static HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
         IAudioSessionManager2 *iface, const GUID *session_guid, DWORD flags,
         ISimpleAudioVolume **out)
 {
@@ -2851,7 +2893,7 @@ HRESULT WINAPI AudioSessionManager_GetSimpleAudioVolume(
     return S_OK;
 }
 
-HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
+static HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
         IAudioSessionManager2 *iface, IAudioSessionEnumerator **out)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -2859,7 +2901,7 @@ HRESULT WINAPI AudioSessionManager_GetSessionEnumerator(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
+static HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
         IAudioSessionManager2 *iface, IAudioSessionNotification *notification)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -2867,7 +2909,7 @@ HRESULT WINAPI AudioSessionManager_RegisterSessionNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
+static HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
         IAudioSessionManager2 *iface, IAudioSessionNotification *notification)
 {
     SessionMgr *This = impl_from_IAudioSessionManager2(iface);
@@ -2875,7 +2917,7 @@ HRESULT WINAPI AudioSessionManager_UnregisterSessionNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
+static HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
         IAudioSessionManager2 *iface, const WCHAR *session_id,
         IAudioVolumeDuckNotification *notification)
 {
@@ -2884,7 +2926,7 @@ HRESULT WINAPI AudioSessionManager_RegisterDuckNotification(
     return E_NOTIMPL;
 }
 
-HRESULT WINAPI AudioSessionManager_UnregisterDuckNotification(
+static HRESULT WINAPI AudioSessionManager_UnregisterDuckNotification(
         IAudioSessionManager2 *iface,
         IAudioVolumeDuckNotification *notification)
 {
