@@ -17,13 +17,26 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
+
+#include "config.h"
+#include "wine/port.h"
+
 #include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <signal.h>
+#include <errno.h>
+#include <fcntl.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
 #include "wine/wingdi16.h"
-#include "wownt32.h"
+#include "winreg.h"
 #include "psdrv.h"
 #include "wine/debug.h"
 #include "winspool.h"
@@ -33,12 +46,26 @@ WINE_DEFAULT_DEBUG_CHANNEL(psdrv);
 static const char psbegindocument[] =
 "%%BeginDocument: Wine passthrough\n";
 
+
+DWORD write_spool( PHYSDEV dev, const void *data, DWORD num )
+{
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+    DWORD written;
+
+    if (!WritePrinter(physDev->job.hprinter, (LPBYTE) data, num, &written) || (written != num))
+        return SP_OUTOFDISK;
+
+    return num;
+}
+
 /**********************************************************************
  *           ExtEscape  (WINEPS.@)
  */
-INT CDECL PSDRV_ExtEscape( PSDRV_PDEVICE *physDev, INT nEscape, INT cbInput, LPCVOID in_data,
-                           INT cbOutput, LPVOID out_data )
+INT PSDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_data,
+                     INT cbOutput, LPVOID out_data )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+
     switch(nEscape)
     {
     case QUERYESCSUPPORT:
@@ -128,7 +155,7 @@ INT CDECL PSDRV_ExtEscape( PSDRV_PDEVICE *physDev, INT nEscape, INT cbInput, LPC
         r->bottom = 0;
 	TRACE("NEXTBAND rect to 0,0 - 0,0\n" );
 	physDev->job.banding = FALSE;
-        return EndPage( physDev->hdc );
+        return EndPage( dev->hdc );
     }
 
     case SETCOPYCOUNT:
@@ -252,10 +279,10 @@ INT CDECL PSDRV_ExtEscape( PSDRV_PDEVICE *physDev, INT nEscape, INT cbInput, LPC
              * in_data[0] instead.
              */
             if(!physDev->job.in_passthrough) {
-                WriteSpool16(physDev->job.hJob, (LPSTR)psbegindocument, sizeof(psbegindocument)-1);
+                write_spool(dev, psbegindocument, sizeof(psbegindocument)-1);
                 physDev->job.in_passthrough = TRUE;
             }
-            return WriteSpool16(physDev->job.hJob,((char*)in_data)+2,*(const WORD*)in_data);
+            return write_spool(dev,((char*)in_data)+2,*(const WORD*)in_data);
         }
 
     case POSTSCRIPT_IGNORE:
@@ -287,14 +314,14 @@ INT CDECL PSDRV_ExtEscape( PSDRV_PDEVICE *physDev, INT nEscape, INT cbInput, LPC
 
 	TRACE("END_PATH\n");
         if(!physDev->pathdepth) {
-	    ERR("END_PATH called without a BEIGN_PATH\n");
+	    ERR("END_PATH called without a BEGIN_PATH\n");
 	    return -1;
 	}
 	TRACE("RenderMode = %d, FillMode = %d, BkMode = %d\n",
 	      info->RenderMode, info->FillMode, info->BkMode);
 	switch(info->RenderMode) {
 	case RENDERMODE_NO_DISPLAY:
-	    PSDRV_WriteClosePath(physDev); /* not sure if this is necessary, but it can't hurt */
+	    PSDRV_WriteClosePath(dev); /* not sure if this is necessary, but it can't hurt */
 	    break;
 	case RENDERMODE_OPEN:
 	case RENDERMODE_CLOSED:
@@ -312,17 +339,17 @@ INT CDECL PSDRV_ExtEscape( PSDRV_PDEVICE *physDev, INT nEscape, INT cbInput, LPC
 	switch(mode) {
 	case CLIP_SAVE:
 	    TRACE("CLIP_TO_PATH: CLIP_SAVE\n");
-	    PSDRV_WriteGSave(physDev);
+	    PSDRV_WriteGSave(dev);
 	    return 1;
 	case CLIP_RESTORE:
 	    TRACE("CLIP_TO_PATH: CLIP_RESTORE\n");
-	    PSDRV_WriteGRestore(physDev);
+	    PSDRV_WriteGRestore(dev);
 	    return 1;
 	case CLIP_INCLUSIVE:
 	    TRACE("CLIP_TO_PATH: CLIP_INCLUSIVE\n");
 	    /* FIXME to clip or eoclip ? (see PATH_INFO.FillMode) */
-	    PSDRV_WriteClip(physDev);
-            PSDRV_WriteNewPath(physDev);
+	    PSDRV_WriteClip(dev);
+            PSDRV_WriteNewPath(dev);
 	    return 1;
 	case CLIP_EXCLUSIVE:
 	    FIXME("CLIP_EXCLUSIVE: not implemented\n");
@@ -341,19 +368,21 @@ INT CDECL PSDRV_ExtEscape( PSDRV_PDEVICE *physDev, INT nEscape, INT cbInput, LPC
 /************************************************************************
  *           PSDRV_StartPage
  */
-INT CDECL PSDRV_StartPage( PSDRV_PDEVICE *physDev )
+INT PSDRV_StartPage( PHYSDEV dev )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+
     if(!physDev->job.OutOfPage) {
         FIXME("Already started a page?\n");
 	return 1;
     }
 
     if(physDev->job.PageNo++ == 0) {
-        if(!PSDRV_WriteHeader( physDev, physDev->job.DocName ))
+        if(!PSDRV_WriteHeader( dev, physDev->job.DocName ))
             return 0;
     }
 
-    if(!PSDRV_WriteNewPage( physDev ))
+    if(!PSDRV_WriteNewPage( dev ))
         return 0;
     physDev->job.OutOfPage = FALSE;
     return 1;
@@ -363,15 +392,17 @@ INT CDECL PSDRV_StartPage( PSDRV_PDEVICE *physDev )
 /************************************************************************
  *           PSDRV_EndPage
  */
-INT CDECL PSDRV_EndPage( PSDRV_PDEVICE *physDev )
+INT PSDRV_EndPage( PHYSDEV dev )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+
     if(physDev->job.OutOfPage) {
         FIXME("Already ended a page?\n");
 	return 1;
     }
-    if(!PSDRV_WriteEndPage( physDev ))
+    if(!PSDRV_WriteEndPage( dev ))
         return 0;
-    PSDRV_EmptyDownloadList(physDev, FALSE);
+    PSDRV_EmptyDownloadList(dev, FALSE);
     physDev->job.OutOfPage = TRUE;
     return 1;
 }
@@ -380,35 +411,44 @@ INT CDECL PSDRV_EndPage( PSDRV_PDEVICE *physDev )
 /************************************************************************
  *           PSDRV_StartDocA
  */
-static INT PSDRV_StartDocA( PSDRV_PDEVICE *physDev, const DOCINFOA *doc )
+static INT PSDRV_StartDocA( PHYSDEV dev, const DOCINFOA *doc )
 {
-    LPCSTR output = "LPT1:";
-    BYTE buf[300];
-    HANDLE hprn = INVALID_HANDLE_VALUE;
-    PRINTER_INFO_5A *pi5 = (PRINTER_INFO_5A*)buf;
-    DWORD needed;
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
+    DOC_INFO_1A di;
 
-    if(physDev->job.hJob) {
+    TRACE("(%p, %p) => %s, %s, %s\n", physDev, doc, debugstr_a(doc->lpszDocName),
+        debugstr_a(doc->lpszOutput), debugstr_a(doc->lpszDatatype));
+
+    if(physDev->job.id) {
         FIXME("hJob != 0. Now what?\n");
 	return 0;
     }
 
-    if(doc->lpszOutput)
-        output = doc->lpszOutput;
-    else if(physDev->job.output)
-        output = physDev->job.output;
-    else {
-        if(OpenPrinterA(physDev->pi->FriendlyName, &hprn, NULL) &&
-           GetPrinterA(hprn, 5, buf, sizeof(buf), &needed)) {
-            output = pi5->pPortName;
-        }
-        if(hprn != INVALID_HANDLE_VALUE)
-            ClosePrinter(hprn);
+    /* FIXME: use PRINTER_DEFAULTS here */
+    if(!OpenPrinterA(physDev->pi->FriendlyName, &physDev->job.hprinter, NULL)) {
+        WARN("OpenPrinter(%s, ...) failed: %d\n",
+            debugstr_a(physDev->pi->FriendlyName), GetLastError());
+        return 0;
     }
 
-    physDev->job.hJob = OpenJob16(output,  doc->lpszDocName, HDC_16(physDev->hdc) );
-    if(!physDev->job.hJob) {
-        WARN("OpenJob failed\n");
+    di.pDocName = (LPSTR) doc->lpszDocName;
+    di.pDatatype = NULL;
+
+    if(doc->lpszOutput)
+        di.pOutputFile = (LPSTR) doc->lpszOutput;
+    else if(physDev->job.output)
+        di.pOutputFile = physDev->job.output;
+    else
+        di.pOutputFile = NULL;
+
+    TRACE("using output: %s\n", debugstr_a(di.pOutputFile));
+
+    /* redirection located in HKCU\Software\Wine\Printing\Spooler
+       is done during winspool.drv,ScheduleJob */
+    physDev->job.id = StartDocPrinterA(physDev->job.hprinter, 1, (LPBYTE) &di);
+    if(!physDev->job.id) {
+        WARN("StartDocPrinter() failed: %d\n", GetLastError());
+        ClosePrinter(physDev->job.hprinter);
 	return 0;
     }
     physDev->job.banding = FALSE;
@@ -423,18 +463,21 @@ static INT PSDRV_StartDocA( PSDRV_PDEVICE *physDev, const DOCINFOA *doc )
     } else
         physDev->job.DocName = NULL;
 
-    return physDev->job.hJob;
+    return physDev->job.id;
 }
 
 /************************************************************************
  *           PSDRV_StartDoc
  */
-INT CDECL PSDRV_StartDoc( PSDRV_PDEVICE *physDev, const DOCINFOW *doc )
+INT PSDRV_StartDoc( PHYSDEV dev, const DOCINFOW *doc )
 {
     DOCINFOA docA;
     INT ret, len;
     LPSTR docname = NULL, output = NULL, datatype = NULL;
 
+    TRACE("(%p, %p) => %d,%s,%s,%s\n", dev, doc, doc->cbSize, debugstr_w(doc->lpszDocName),
+        debugstr_w(doc->lpszOutput), debugstr_w(doc->lpszDatatype));
+
     docA.cbSize = doc->cbSize;
     if (doc->lpszDocName)
     {
@@ -459,7 +502,7 @@ INT CDECL PSDRV_StartDoc( PSDRV_PDEVICE *physDev, const DOCINFOW *doc )
     docA.lpszDatatype = datatype;
     docA.fwType = doc->fwType;
 
-    ret = PSDRV_StartDocA(physDev, &docA);
+    ret = PSDRV_StartDocA(dev, &docA);
 
     HeapFree( GetProcessHeap(), 0, docname );
     HeapFree( GetProcessHeap(), 0, output );
@@ -471,25 +514,26 @@ INT CDECL PSDRV_StartDoc( PSDRV_PDEVICE *physDev, const DOCINFOW *doc )
 /************************************************************************
  *           PSDRV_EndDoc
  */
-INT CDECL PSDRV_EndDoc( PSDRV_PDEVICE *physDev )
+INT PSDRV_EndDoc( PHYSDEV dev )
 {
+    PSDRV_PDEVICE *physDev = get_psdrv_dev( dev );
     INT ret = 1;
-    if(!physDev->job.hJob) {
+
+    if(!physDev->job.id) {
         FIXME("hJob == 0. Now what?\n");
 	return 0;
     }
 
     if(!physDev->job.OutOfPage) {
         WARN("Somebody forgot an EndPage\n");
-	PSDRV_EndPage( physDev );
+	PSDRV_EndPage( dev );
     }
-    PSDRV_WriteFooter( physDev );
+    PSDRV_WriteFooter( dev );
 
-    if( CloseJob16( physDev->job.hJob ) == SP_ERROR ) {
-        WARN("CloseJob error\n");
-	ret = 0;
-    }
-    physDev->job.hJob = 0;
+    ret = EndDocPrinter(physDev->job.hprinter);
+    ClosePrinter(physDev->job.hprinter);
+    physDev->job.hprinter = NULL;
+    physDev->job.id = 0;
     HeapFree(GetProcessHeap(), 0, physDev->job.DocName);
     physDev->job.DocName = NULL;
 
