@@ -30,6 +30,9 @@
 #ifdef HAVE_X11_EXTENSIONS_XF86VMODE_H
 #include <X11/extensions/xf86vmode.h>
 #endif
+#ifdef HAVE_X11_EXTENSIONS_XF86VMPROTO_H
+#include <X11/extensions/xf86vmproto.h>
+#endif
 
 #include "xvidmode.h"
 
@@ -263,15 +266,6 @@ void X11DRV_XF86VM_Cleanup(void)
   wine_tsx11_unlock();
 }
 
-void X11DRV_XF86VM_SetExclusiveMode(int lock)
-{
-  if (!dd_modes) return; /* no XVidMode */
-
-  wine_tsx11_lock();
-  pXF86VidModeLockModeSwitch(gdi_display, DefaultScreen(gdi_display), lock);
-  wine_tsx11_unlock();
-}
-
 /***** GAMMA CONTROL *****/
 /* (only available in XF86VidMode 2.x) */
 
@@ -395,7 +389,7 @@ static BOOL X11DRV_XF86VM_SetGammaRamp(LPDDGAMMARAMP ramp)
 #ifdef X_XF86VidModeSetGamma
   XF86VidModeGamma gamma;
 
-  if (xf86vm_major < 2) return FALSE; /* no gamma control */
+  if (xf86vm_major < 2 || !usexvidmode) return FALSE; /* no gamma control */
 #ifdef X_XF86VidModeSetGammaRamp
   else if (xf86vm_use_gammaramp)
   {
@@ -432,7 +426,7 @@ static BOOL X11DRV_XF86VM_SetGammaRamp(LPDDGAMMARAMP ramp)
  * the stuff in graphics/x11drv/ has been moved to dlls/x11drv, so that
  * they can include xvidmode.h directly
  */
-BOOL CDECL X11DRV_GetDeviceGammaRamp(X11DRV_PDEVICE *physDev, LPVOID ramp)
+BOOL X11DRV_GetDeviceGammaRamp(PHYSDEV dev, LPVOID ramp)
 {
 #ifdef SONAME_LIBXXF86VM
   return X11DRV_XF86VM_GetGammaRamp(ramp);
@@ -448,7 +442,7 @@ BOOL CDECL X11DRV_GetDeviceGammaRamp(X11DRV_PDEVICE *physDev, LPVOID ramp)
  * the stuff in graphics/x11drv/ has been moved to dlls/x11drv, so that
  * they can include xvidmode.h directly
  */
-BOOL CDECL X11DRV_SetDeviceGammaRamp(X11DRV_PDEVICE *physDev, LPVOID ramp)
+BOOL X11DRV_SetDeviceGammaRamp(PHYSDEV dev, LPVOID ramp)
 {
 #ifdef SONAME_LIBXXF86VM
   return X11DRV_XF86VM_SetGammaRamp(ramp);
