@@ -271,6 +271,7 @@ typedef struct _CRYPTUI_WIZ_EXPORT_INFO
         PCCERT_CONTEXT pCertContext;
         PCCTL_CONTEXT  pCTLContext;
         PCCRL_CONTEXT  pCRLContext;
+        HCERTSTORE     hCertStore;
     } DUMMYUNIONNAME;
     DWORD       cStores;
     HCERTSTORE *rghStores;
@@ -347,12 +348,14 @@ typedef struct _CRYPTUI_SELECTCERTIFICATE_STRUCTA
     DWORD               dwSize;
     HWND                hwndParent;
     DWORD               dwFlags;
-    LPCWSTR             szTitle;
+    LPCSTR              szTitle;
     DWORD               dwDontUseColumn;
-    LPCWSTR             szDisplayString;
+    LPCSTR              szDisplayString;
     PFNCFILTERPROC      pFilterCallback;
     PFNCCERTDISPLAYPROC pDisplayCallback;
     void               *pvCallbackData;
+    DWORD               cDisplayStores;
+    HCERTSTORE         *rghDisplayStores;
     DWORD               cStores;
     HCERTSTORE         *rghStores;
     DWORD               cPropSheetPages;
@@ -373,6 +376,8 @@ typedef struct _CRYPTUI_SELECTCERTIFICATE_STRUCTW
     PFNCFILTERPROC      pFilterCallback;
     PFNCCERTDISPLAYPROC pDisplayCallback;
     void               *pvCallbackData;
+    DWORD               cDisplayStores;
+    HCERTSTORE         *rghDisplayStores;
     DWORD               cStores;
     HCERTSTORE         *rghStores;
     DWORD               cPropSheetPages;
