@@ -473,63 +473,64 @@ void WCMD_copy (void) {
 /****************************************************************************
  * WCMD_create_dir
  *
- * Create a directory.
+ * Create a directory (and, if needed, any intermediate directories).
  *
- * this works recursively. so mkdir dir1\dir2\dir3 will create dir1 and dir2 if
- * they do not already exist.
+ * Modifies its argument by replacing slashes temporarily with nulls.
  */
 
 static BOOL create_full_path(WCHAR* path)
 {
-    int len;
-    WCHAR *new_path;
-    BOOL ret = TRUE;
+    WCHAR *p, *start;
 
-    new_path = HeapAlloc(GetProcessHeap(),0,(strlenW(path)+1) * sizeof(WCHAR));
-    strcpyW(new_path,path);
+    /* don't mess with drive letter portion of path, if any */
+    start = path;
+    if (path[1] == ':')
+        start = path+2;
 
-    while ((len = strlenW(new_path)) && new_path[len - 1] == '\\')
-        new_path[len - 1] = 0;
+    /* Strip trailing slashes. */
+    for (p = path + strlenW(path) - 1; p != start && *p == '\\'; p--)
+        *p = 0;
 
-    while (!CreateDirectoryW(new_path,NULL))
-    {
-        WCHAR *slash;
-        DWORD last_error = GetLastError();
-        if (last_error == ERROR_ALREADY_EXISTS)
-            break;
-
-        if (last_error != ERROR_PATH_NOT_FOUND)
-        {
-            ret = FALSE;
-            break;
+    /* Step through path, creating intermediate directories as needed. */
+    /* First component includes drive letter, if any. */
+    p = start;
+    for (;;) {
+        DWORD rv;
+        /* Skip to end of component */
+        while (*p == '\\') p++;
+        while (*p && *p != '\\') p++;
+        if (!*p) {
+            /* path is now the original full path */
+            return CreateDirectoryW(path, NULL);
         }
-
-        if (!(slash = strrchrW(new_path,'\\')) && ! (slash = strrchrW(new_path,'/')))
-        {
-            ret = FALSE;
-            break;
-        }
-
-        len = slash - new_path;
-        new_path[len] = 0;
-        if (!create_full_path(new_path))
-        {
-            ret = FALSE;
-            break;
-        }
-        new_path[len] = '\\';
+        /* Truncate path, create intermediate directory, and restore path */
+        *p = 0;
+        rv = CreateDirectoryW(path, NULL);
+        *p = '\\';
+        if (!rv && GetLastError() != ERROR_ALREADY_EXISTS)
+            return FALSE;
     }
-    HeapFree(GetProcessHeap(),0,new_path);
-    return ret;
+    /* notreached */
+    return FALSE;
 }
 
-void WCMD_create_dir (void) {
+void WCMD_create_dir (WCHAR *command) {
+    int   argno = 0;
+    WCHAR *argN = command;
 
     if (param1[0] == 0x00) {
         WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
         return;
     }
-    if (!create_full_path(param1)) WCMD_print_error ();
+    /* Loop through all args */
+    while (TRUE) {
+        WCHAR *thisArg = WCMD_parameter(command, argno++, &argN);
+        if (!argN) break;
+        if (!create_full_path(thisArg)) {
+            WCMD_print_error ();
+            errorlevel = 1;
+        }
+    }
 }
 
 /* Parse the /A options given by the user on the commandline
@@ -579,6 +580,228 @@ static void WCMD_delete_parse_attributes(DWORD *wantSet, DWORD *wantClear) {
     }
 }
 
+/* If filename part of parameter is * or *.*,
+ * and neither /Q nor /P options were given,
+ * prompt the user whether to proceed.
+ * Returns FALSE if user says no, TRUE otherwise.
+ * *pPrompted is set to TRUE if the user is prompted.
+ * (If /P supplied, del will prompt for individual files later.)
+ */
+static BOOL WCMD_delete_confirm_wildcard(WCHAR *filename, BOOL *pPrompted) {
+    static const WCHAR parmP[] = {'/','P','\0'};
+    static const WCHAR parmQ[] = {'/','Q','\0'};
+
+    if ((strstrW(quals, parmQ) == NULL) && (strstrW(quals, parmP) == NULL)) {
+        static const WCHAR anyExt[]= {'.','*','\0'};
+        WCHAR drive[10];
+        WCHAR dir[MAX_PATH];
+        WCHAR fname[MAX_PATH];
+        WCHAR ext[MAX_PATH];
+        WCHAR fpath[MAX_PATH];
+
+        /* Convert path into actual directory spec */
+        GetFullPathNameW(filename, sizeof(fpath)/sizeof(WCHAR), fpath, NULL);
+        WCMD_splitpath(fpath, drive, dir, fname, ext);
+
+        /* Only prompt for * and *.*, not *a, a*, *.a* etc */
+        if ((strcmpW(fname, starW) == 0) &&
+            (*ext == 0x00 || (strcmpW(ext, anyExt) == 0))) {
+
+            WCHAR question[MAXSTRING];
+            static const WCHAR fmt[] = {'%','s',' ','\0'};
+
+            /* Caller uses this to suppress "file not found" warning later */
+            *pPrompted = TRUE;
+
+            /* Ask for confirmation */
+            wsprintfW(question, fmt, fpath);
+            return WCMD_ask_confirm(question, TRUE, NULL);
+        }
+    }
+    /* No scary wildcard, or question suppressed, so it's ok to delete the file(s) */
+    return TRUE;
+}
+
+/* Helper function for WCMD_delete().
+ * Deletes a single file, directory, or wildcard.
+ * If /S was given, does it recursively.
+ * Returns TRUE if a file was deleted.
+ */
+static BOOL WCMD_delete_one (WCHAR *thisArg) {
+
+    static const WCHAR parmP[] = {'/','P','\0'};
+    static const WCHAR parmS[] = {'/','S','\0'};
+    static const WCHAR parmF[] = {'/','F','\0'};
+    DWORD wanted_attrs;
+    DWORD unwanted_attrs;
+    BOOL found = FALSE;
+    WCHAR argCopy[MAX_PATH];
+    WIN32_FIND_DATAW fd;
+    HANDLE hff;
+    WCHAR fpath[MAX_PATH];
+    WCHAR *p;
+    BOOL handleParm = TRUE;
+
+    WCMD_delete_parse_attributes(&wanted_attrs, &unwanted_attrs);
+
+    strcpyW(argCopy, thisArg);
+    WINE_TRACE("del: Processing arg %s (quals:%s)\n",
+               wine_dbgstr_w(argCopy), wine_dbgstr_w(quals));
+
+    if (!WCMD_delete_confirm_wildcard(argCopy, &found)) {
+        /* Skip this arg if user declines to delete *.* */
+        return FALSE;
+    }
+
+    /* First, try to delete in the current directory */
+    hff = FindFirstFileW(argCopy, &fd);
+    if (hff == INVALID_HANDLE_VALUE) {
+      handleParm = FALSE;
+    } else {
+      found = TRUE;
+    }
+
+    /* Support del <dirname> by just deleting all files dirname\* */
+    if (handleParm
+        && (strchrW(argCopy,'*') == NULL)
+        && (strchrW(argCopy,'?') == NULL)
+        && (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
+    {
+      WCHAR modifiedParm[MAX_PATH];
+      static const WCHAR slashStar[] = {'\\','*','\0'};
+
+      strcpyW(modifiedParm, argCopy);
+      strcatW(modifiedParm, slashStar);
+      FindClose(hff);
+      found = TRUE;
+      WCMD_delete_one(modifiedParm);
+
+    } else if (handleParm) {
+
+      /* Build the filename to delete as <supplied directory>\<findfirst filename> */
+      strcpyW (fpath, argCopy);
+      do {
+        p = strrchrW (fpath, '\\');
+        if (p != NULL) {
+          *++p = '\0';
+          strcatW (fpath, fd.cFileName);
+        }
+        else strcpyW (fpath, fd.cFileName);
+        if (!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
+          BOOL ok;
+
+          /* Handle attribute matching (/A) */
+          ok =  ((fd.dwFileAttributes & wanted_attrs) == wanted_attrs)
+             && ((fd.dwFileAttributes & unwanted_attrs) == 0);
+
+          /* /P means prompt for each file */
+          if (ok && strstrW (quals, parmP) != NULL) {
+            WCHAR  question[MAXSTRING];
+
+            /* Ask for confirmation */
+            wsprintfW(question, WCMD_LoadMessage(WCMD_DELPROMPT), fpath);
+            ok = WCMD_ask_confirm(question, FALSE, NULL);
+          }
+
+          /* Only proceed if ok to */
+          if (ok) {
+
+            /* If file is read only, and /A:r or /F supplied, delete it */
+            if (fd.dwFileAttributes & FILE_ATTRIBUTE_READONLY &&
+                ((wanted_attrs & FILE_ATTRIBUTE_READONLY) ||
+                strstrW (quals, parmF) != NULL)) {
+                SetFileAttributesW(fpath, fd.dwFileAttributes & ~FILE_ATTRIBUTE_READONLY);
+            }
+
+            /* Now do the delete */
+            if (!DeleteFileW(fpath)) WCMD_print_error ();
+          }
+
+        }
+      } while (FindNextFileW(hff, &fd) != 0);
+      FindClose (hff);
+    }
+
+    /* Now recurse into all subdirectories handling the parameter in the same way */
+    if (strstrW (quals, parmS) != NULL) {
+
+      WCHAR thisDir[MAX_PATH];
+      int cPos;
+
+      WCHAR drive[10];
+      WCHAR dir[MAX_PATH];
+      WCHAR fname[MAX_PATH];
+      WCHAR ext[MAX_PATH];
+
+      /* Convert path into actual directory spec */
+      GetFullPathNameW(argCopy, sizeof(thisDir)/sizeof(WCHAR), thisDir, NULL);
+      WCMD_splitpath(thisDir, drive, dir, fname, ext);
+
+      strcpyW(thisDir, drive);
+      strcatW(thisDir, dir);
+      cPos = strlenW(thisDir);
+
+      WINE_TRACE("Searching recursively in '%s'\n", wine_dbgstr_w(thisDir));
+
+      /* Append '*' to the directory */
+      thisDir[cPos] = '*';
+      thisDir[cPos+1] = 0x00;
+
+      hff = FindFirstFileW(thisDir, &fd);
+
+      /* Remove residual '*' */
+      thisDir[cPos] = 0x00;
+
+      if (hff != INVALID_HANDLE_VALUE) {
+        DIRECTORY_STACK *allDirs = NULL;
+        DIRECTORY_STACK *lastEntry = NULL;
+
+        do {
+          if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
+              (strcmpW(fd.cFileName, dotdotW) != 0) &&
+              (strcmpW(fd.cFileName, dotW) != 0)) {
+
+            DIRECTORY_STACK *nextDir;
+            WCHAR subParm[MAX_PATH];
+
+            /* Work out search parameter in sub dir */
+            strcpyW (subParm, thisDir);
+            strcatW (subParm, fd.cFileName);
+            strcatW (subParm, slashW);
+            strcatW (subParm, fname);
+            strcatW (subParm, ext);
+            WINE_TRACE("Recursive, Adding to search list '%s'\n", wine_dbgstr_w(subParm));
+
+            /* Allocate memory, add to list */
+            nextDir = HeapAlloc(GetProcessHeap(),0,sizeof(DIRECTORY_STACK));
+            if (allDirs == NULL) allDirs = nextDir;
+            if (lastEntry != NULL) lastEntry->next = nextDir;
+            lastEntry = nextDir;
+            nextDir->next = NULL;
+            nextDir->dirName = HeapAlloc(GetProcessHeap(),0,
+	 (strlenW(subParm)+1) * sizeof(WCHAR));
+            strcpyW(nextDir->dirName, subParm);
+          }
+        } while (FindNextFileW(hff, &fd) != 0);
+        FindClose (hff);
+
+        /* Go through each subdir doing the delete */
+        while (allDirs != NULL) {
+          DIRECTORY_STACK *tempDir;
+
+          tempDir = allDirs->next;
+          found |= WCMD_delete_one (allDirs->dirName);
+
+          HeapFree(GetProcessHeap(),0,allDirs->dirName);
+          HeapFree(GetProcessHeap(),0,allDirs);
+          allDirs = tempDir;
+        }
+      }
+    }
+
+    return found;
+}
+
 /****************************************************************************
  * WCMD_delete
  *
@@ -592,234 +815,37 @@ static void WCMD_delete_parse_attributes(DWORD *wantSet, DWORD *wantClear) {
  *         non-hidden files
  */
 
-BOOL WCMD_delete (WCHAR *command, BOOL expectDir) {
-
-    int   argno         = 0;
-    int   argsProcessed = 0;
-    WCHAR *argN          = command;
+BOOL WCMD_delete (WCHAR *command) {
+    int   argno;
+    WCHAR *argN;
+    BOOL  argsProcessed = FALSE;
     BOOL  foundAny      = FALSE;
-    static const WCHAR parmQ[] = {'/','Q','\0'};
-    static const WCHAR parmP[] = {'/','P','\0'};
-    static const WCHAR parmS[] = {'/','S','\0'};
-    static const WCHAR parmF[] = {'/','F','\0'};
-    DWORD wanted_attrs;
-    DWORD unwanted_attrs;
 
-    WCMD_delete_parse_attributes(&wanted_attrs, &unwanted_attrs);
+    errorlevel = 0;
 
-    /* If not recursing, clear error flag */
-    if (expectDir) errorlevel = 0;
+    for (argno=0; ; argno++) {
+        BOOL found;
+        WCHAR *thisArg;
 
-    /* Loop through all args */
-    while (argN) {
-      WCHAR *thisArg = WCMD_parameter (command, argno++, &argN);
-      WCHAR argCopy[MAX_PATH];
+        argN = NULL;
+        thisArg = WCMD_parameter (command, argno, &argN);
+        if (!argN)
+            break;       /* no more parameters */
+        if (argN[0] == '/')
+            continue;    /* skip options */
 
-      if (argN && argN[0] != '/') {
-
-        WIN32_FIND_DATAW fd;
-        HANDLE hff;
-        WCHAR fpath[MAX_PATH];
-        WCHAR *p;
-        BOOL handleParm = TRUE;
-        BOOL found = FALSE;
-        static const WCHAR anyExt[]= {'.','*','\0'};
-
-        strcpyW(argCopy, thisArg);
-        WINE_TRACE("del: Processing arg %s (quals:%s)\n",
-                   wine_dbgstr_w(argCopy), wine_dbgstr_w(quals));
-        argsProcessed++;
-
-        /* If filename part of parameter is * or *.*, prompt unless
-           /Q supplied.                                            */
-        if ((strstrW (quals, parmQ) == NULL) && (strstrW (quals, parmP) == NULL)) {
-
-          WCHAR drive[10];
-          WCHAR dir[MAX_PATH];
-          WCHAR fname[MAX_PATH];
-          WCHAR ext[MAX_PATH];
-
-          /* Convert path into actual directory spec */
-          GetFullPathNameW(argCopy, sizeof(fpath)/sizeof(WCHAR), fpath, NULL);
-          WCMD_splitpath(fpath, drive, dir, fname, ext);
-
-          /* Only prompt for * and *.*, not *a, a*, *.a* etc */
-          if ((strcmpW(fname, starW) == 0) &&
-              (*ext == 0x00 || (strcmpW(ext, anyExt) == 0))) {
-            BOOL  ok;
-            WCHAR  question[MAXSTRING];
-            static const WCHAR fmt[] = {'%','s',' ','\0'};
-
-            /* Note: Flag as found, to avoid file not found message */
-            found = TRUE;
-
-            /* Ask for confirmation */
-            wsprintfW(question, fmt, fpath);
-            ok = WCMD_ask_confirm(question, TRUE, NULL);
-
-            /* Abort if answer is 'N' */
-            if (!ok) continue;
-          }
-        }
-
-        /* First, try to delete in the current directory */
-        hff = FindFirstFileW(argCopy, &fd);
-        if (hff == INVALID_HANDLE_VALUE) {
-          handleParm = FALSE;
-        } else {
-          found = TRUE;
-        }
-
-        /* Support del <dirname> by just deleting all files dirname\* */
-        if (handleParm && (strchrW(argCopy,'*') == NULL) && (strchrW(argCopy,'?') == NULL)
-		&& (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
-          WCHAR modifiedParm[MAX_PATH];
-          static const WCHAR slashStar[] = {'\\','*','\0'};
-
-          strcpyW(modifiedParm, argCopy);
-          strcatW(modifiedParm, slashStar);
-          FindClose(hff);
-          found = TRUE;
-          WCMD_delete(modifiedParm, FALSE);
-
-        } else if (handleParm) {
-
-          /* Build the filename to delete as <supplied directory>\<findfirst filename> */
-          strcpyW (fpath, argCopy);
-          do {
-            p = strrchrW (fpath, '\\');
-            if (p != NULL) {
-              *++p = '\0';
-              strcatW (fpath, fd.cFileName);
-            }
-            else strcpyW (fpath, fd.cFileName);
-            if (!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
-              BOOL ok;
-
-              /* Handle attribute matching (/A) */
-              ok =  ((fd.dwFileAttributes & wanted_attrs) == wanted_attrs)
-                 && ((fd.dwFileAttributes & unwanted_attrs) == 0);
-
-              /* /P means prompt for each file */
-              if (ok && strstrW (quals, parmP) != NULL) {
-                WCHAR  question[MAXSTRING];
-
-                /* Ask for confirmation */
-                wsprintfW(question, WCMD_LoadMessage(WCMD_DELPROMPT), fpath);
-                ok = WCMD_ask_confirm(question, FALSE, NULL);
-              }
-
-              /* Only proceed if ok to */
-              if (ok) {
-
-                /* If file is read only, and /A:r or /F supplied, delete it */
-                if (fd.dwFileAttributes & FILE_ATTRIBUTE_READONLY &&
-                    ((wanted_attrs & FILE_ATTRIBUTE_READONLY) ||
-                    strstrW (quals, parmF) != NULL)) {
-                    SetFileAttributesW(fpath, fd.dwFileAttributes & ~FILE_ATTRIBUTE_READONLY);
-                }
-
-                /* Now do the delete */
-                if (!DeleteFileW(fpath)) WCMD_print_error ();
-              }
-
-            }
-          } while (FindNextFileW(hff, &fd) != 0);
-          FindClose (hff);
-        }
-
-        /* Now recurse into all subdirectories handling the parameter in the same way */
-        if (strstrW (quals, parmS) != NULL) {
-
-          WCHAR thisDir[MAX_PATH];
-          int cPos;
-
-          WCHAR drive[10];
-          WCHAR dir[MAX_PATH];
-          WCHAR fname[MAX_PATH];
-          WCHAR ext[MAX_PATH];
-
-          /* Convert path into actual directory spec */
-          GetFullPathNameW(argCopy, sizeof(thisDir)/sizeof(WCHAR), thisDir, NULL);
-          WCMD_splitpath(thisDir, drive, dir, fname, ext);
-
-          strcpyW(thisDir, drive);
-          strcatW(thisDir, dir);
-          cPos = strlenW(thisDir);
-
-          WINE_TRACE("Searching recursively in '%s'\n", wine_dbgstr_w(thisDir));
-
-          /* Append '*' to the directory */
-          thisDir[cPos] = '*';
-          thisDir[cPos+1] = 0x00;
-
-          hff = FindFirstFileW(thisDir, &fd);
-
-          /* Remove residual '*' */
-          thisDir[cPos] = 0x00;
-
-          if (hff != INVALID_HANDLE_VALUE) {
-            DIRECTORY_STACK *allDirs = NULL;
-            DIRECTORY_STACK *lastEntry = NULL;
-
-            do {
-              if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
-                  (strcmpW(fd.cFileName, dotdotW) != 0) &&
-                  (strcmpW(fd.cFileName, dotW) != 0)) {
-
-                DIRECTORY_STACK *nextDir;
-                WCHAR subParm[MAX_PATH];
-
-                /* Work out search parameter in sub dir */
-                strcpyW (subParm, thisDir);
-                strcatW (subParm, fd.cFileName);
-                strcatW (subParm, slashW);
-                strcatW (subParm, fname);
-                strcatW (subParm, ext);
-                WINE_TRACE("Recursive, Adding to search list '%s'\n", wine_dbgstr_w(subParm));
-
-                /* Allocate memory, add to list */
-                nextDir = HeapAlloc(GetProcessHeap(),0,sizeof(DIRECTORY_STACK));
-                if (allDirs == NULL) allDirs = nextDir;
-                if (lastEntry != NULL) lastEntry->next = nextDir;
-                lastEntry = nextDir;
-                nextDir->next = NULL;
-                nextDir->dirName = HeapAlloc(GetProcessHeap(),0,
-                                             (strlenW(subParm)+1) * sizeof(WCHAR));
-                strcpyW(nextDir->dirName, subParm);
-              }
-            } while (FindNextFileW(hff, &fd) != 0);
-            FindClose (hff);
-
-            /* Go through each subdir doing the delete */
-            while (allDirs != NULL) {
-              DIRECTORY_STACK *tempDir;
-
-              tempDir = allDirs->next;
-              found |= WCMD_delete (allDirs->dirName, FALSE);
-
-              HeapFree(GetProcessHeap(),0,allDirs->dirName);
-              HeapFree(GetProcessHeap(),0,allDirs);
-              allDirs = tempDir;
-            }
-          }
-        }
-        /* Keep running total to see if any found, and if not recursing
-           issue error message                                         */
-        if (expectDir) {
-          if (!found) {
+        argsProcessed = TRUE;
+        found = WCMD_delete_one(thisArg);
+        if (!found) {
             errorlevel = 1;
-            WCMD_output (WCMD_LoadMessage(WCMD_FILENOTFOUND), argCopy);
-          }
+            WCMD_output (WCMD_LoadMessage(WCMD_FILENOTFOUND), thisArg);
         }
         foundAny |= found;
-      }
     }
 
     /* Handle no valid args */
-    if (argsProcessed == 0) {
-      WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
-    }
+    if (!argsProcessed)
+        WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
 
     return foundAny;
 }
@@ -836,10 +862,10 @@ void WCMD_echo (const WCHAR *command) {
   int count;
   const WCHAR *origcommand = command;
 
-  if (command[0]==' ' || command[0]=='.')
+  if (command[0]==' ' || command[0]=='.' || command[0]==':')
     command++;
   count = strlenW(command);
-  if (count == 0 && origcommand[0]!='.') {
+  if (count == 0 && origcommand[0]!='.' && origcommand[0]!=':') {
     if (echo_mode) WCMD_output (WCMD_LoadMessage(WCMD_ECHOPROMPT), onW);
     else WCMD_output (WCMD_LoadMessage(WCMD_ECHOPROMPT), offW);
     return;
@@ -2507,8 +2533,6 @@ void WCMD_type (WCHAR *command) {
         WCMD_output_asis (buffer);
       }
       CloseHandle (h);
-      if (!writeHeaders)
-          WCMD_output_asis (newline);
     }
   }
 }
