@@ -36,7 +36,7 @@ typedef enum _CMDdelimiters {
   CMD_NONE,        /* End of line or single & */
   CMD_ONFAILURE,   /* ||                      */
   CMD_ONSUCCESS,   /* &&                      */
-  CMD_PIPE         /* Single                  */
+  CMD_PIPE         /* Single |                */
 } CMD_DELIMITERS;
 
 typedef struct _CMD_LIST {
@@ -52,11 +52,12 @@ void WCMD_assoc (WCHAR *, BOOL);
 void WCMD_batch (WCHAR *, WCHAR *, int, WCHAR *, HANDLE);
 void WCMD_call (WCHAR *command);
 void WCMD_change_tty (void);
+void WCMD_choice (WCHAR *);
 void WCMD_clear_screen (void);
 void WCMD_color (void);
 void WCMD_copy (void);
-void WCMD_create_dir (void);
-BOOL WCMD_delete (WCHAR *, BOOL);
+void WCMD_create_dir (WCHAR *);
+BOOL WCMD_delete (WCHAR *);
 void WCMD_directory (WCHAR *);
 void WCMD_echo (const WCHAR *);
 void WCMD_endlocal (void);
@@ -100,6 +101,7 @@ WCHAR *WCMD_strtrim_leading_spaces (WCHAR *string);
 void WCMD_HandleTildaModifiers(WCHAR **start, WCHAR *forVariable, WCHAR *forValue, BOOL justFors);
 
 void WCMD_splitpath(const WCHAR* path, WCHAR* drv, WCHAR* dir, WCHAR* name, WCHAR* ext);
+void WCMD_opt_s_strip_quotes(WCHAR *cmd);
 WCHAR *WCMD_LoadMessage(UINT id);
 WCHAR *WCMD_strdupW(WCHAR *input);
 void WCMD_strsubstW(WCHAR *start, WCHAR* next, WCHAR* insert, int len);
@@ -117,6 +119,7 @@ void      WCMD_execute (WCHAR *orig_command, WCHAR *redirects, WCHAR *parameter,
 typedef struct {
   WCHAR *command;	/* The command which invoked the batch file */
   HANDLE h;             /* Handle to the open batch file */
+  WCHAR *batchfileW;    /* Name of same */
   int shift_count[10];	/* Offset in terms of shifts for %0 - %9 */
   void *prev_context;	/* Pointer to the previous context block */
   BOOL  skip_rest;      /* Skip the rest of the batch program and exit */
@@ -148,7 +151,7 @@ typedef struct _DIRECTORY_STACK
 
 /*
  *	Serial nos of builtin commands. These constants must be in step with
- *	the list of strings defined in WCMD.C, and WCMD_EXIT *must* always be
+ *	the list of strings defined in wcmd.rc, and WCMD_EXIT *must* always be
  *	the last one.
  *
  *	Yes it *would* be nice to use an enumeration here, but the Resource
@@ -200,9 +203,10 @@ typedef struct _DIRECTORY_STACK
 #define WCMD_COLOR  41
 #define WCMD_FTYPE  42
 #define WCMD_MORE   43
+#define WCMD_CHOICE 44
 
 /* Must be last in list */
-#define WCMD_EXIT   44
+#define WCMD_EXIT   45
 
 /* Some standard messages */
 extern const WCHAR newline[];
