@@ -903,8 +903,8 @@ static void init_msvcrt_io_block(STARTUPINFOW* st)
     if (st_p.cbReserved2 && st_p.lpReserved2)
     {
         /* Override the entries for fd 0,1,2 if we happened
-         * to change those std handles (this depends on the way wcmd sets
-         * it's new input & output handles)
+         * to change those std handles (this depends on the way cmd sets
+         * its new input & output handles)
          */
         size_t sz = max(sizeof(unsigned) + (sizeof(char) + sizeof(HANDLE)) * 3, st_p.cbReserved2);
         BYTE* ptr = HeapAlloc(GetProcessHeap(), 0, sz);
@@ -1447,7 +1447,7 @@ void WCMD_execute (WCHAR *command, WCHAR *redirects,
 	break;
       case WCMD_DEL:
       case WCMD_ERASE:
-        WCMD_delete (p, TRUE);
+        WCMD_delete (p);
         break;
       case WCMD_DIR:
         WCMD_directory (p);
@@ -1472,7 +1472,7 @@ void WCMD_execute (WCHAR *command, WCHAR *redirects,
         break;
       case WCMD_MD:
       case WCMD_MKDIR:
-        WCMD_create_dir ();
+        WCMD_create_dir (p);
 	break;
       case WCMD_MOVE:
         WCMD_move ();
@@ -1519,6 +1519,7 @@ void WCMD_execute (WCHAR *command, WCHAR *redirects,
         WCMD_type (p);
 	break;
       case WCMD_VER:
+        WCMD_output(newline);
         WCMD_version ();
         break;
       case WCMD_VERIFY:
@@ -1660,7 +1661,7 @@ static void WCMD_addCommand(WCHAR *command, int *commandLen,
 /***************************************************************************
  * WCMD_IsEndQuote
  *
- *   Checks if the quote pointet to is the end-quote.
+ *   Checks if the quote pointed to is the end-quote.
  *
  *   Quotes end if:
  *
@@ -1715,7 +1716,7 @@ static BOOL WCMD_IsEndQuote(WCHAR *quote, int quoteIndex)
  *
  *   Either uses supplied input or
  *     Reads a file from the handle, and then...
- *   Parse the text buffer, spliting into separate commands
+ *   Parse the text buffer, splitting into separate commands
  *     - unquoted && strings split 2 commands but the 2nd is flagged as
  *            following an &&
  *     - ( as the first character just ups the bracket depth
@@ -1785,10 +1786,22 @@ WCHAR *WCMD_ReadAndParseLine(WCHAR *optionalcmd, CMD_LIST **output, HANDLE readF
     /* Show prompt before batch line IF echo is on and in batch program */
     if (context && echo_mode && extraSpace[0] && (extraSpace[0] != '@')) {
       const WCHAR spc[]={' ','\0'};
+      const WCHAR echoDot[] = {'e','c','h','o','.'};
+      const WCHAR echoCol[] = {'e','c','h','o',':'};
+      const DWORD len = sizeof(echoDot)/sizeof(echoDot[0]);
+      DWORD curr_size = strlenW(extraSpace);
+      DWORD min_len = (curr_size < len ? curr_size : len);
       WCMD_show_prompt();
       WCMD_output_asis(extraSpace);
       /* I don't know why Windows puts a space here but it does */
-      WCMD_output_asis(spc);
+      /* Except for lines starting with 'echo.' or 'echo:'. Ask MS why */
+      if (CompareStringW(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
+                         extraSpace, min_len, echoDot, len) != CSTR_EQUAL
+          && CompareStringW(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
+                         extraSpace, min_len, echoCol, len) != CSTR_EQUAL)
+      {
+          WCMD_output_asis(spc);
+      }
       WCMD_output_asis(newline);
     }
 
@@ -2421,7 +2434,7 @@ int wmain (int argc, WCHAR *argvW[])
   }
 
   if (opt_c) {
-      /* If we do a "wcmd /c command", we don't want to allocate a new
+      /* If we do a "cmd /c command", we don't want to allocate a new
        * console since the command returns immediately. Rather, we use
        * the currently allocated input and output handles. This allows
        * us to pipe to and read from the command interpreter.
@@ -2554,7 +2567,7 @@ int wmain (int argc, WCHAR *argvW[])
 
     /* Read until EOF (which for std input is never, but if redirect
        in place, may occur                                          */
-    WCMD_show_prompt ();
+    if (echo_mode) WCMD_show_prompt();
     if (WCMD_ReadAndParseLine(NULL, &toExecute,
                               GetStdHandle(STD_INPUT_HANDLE)) == NULL)
       break;
