@@ -28,5 +28,9 @@ ok(WScript === WSH, "WScript !== WSH");
 ok(WScript.Name === "Windows Script Host", "WScript.Name = " + WScript.Name);
 ok(typeof(WScript.Version) === "string", "typeof(WScript.Version) = " + typeof(WScript.Version));
 ok(typeof(WScript.BuildVersion) === "number", "typeof(WScript.BuldVersion) = " + typeof(WScript.BuldVersion));
+ok(WScript.FullName === winetest.wscriptFullName, "WScript.FullName = ", WScript.FullName);
+ok(WScript.Path === winetest.wscriptPath, "WScript.Path = ", WScript.Path);
+ok(WScript.ScriptName === winetest.wscriptScriptName, "WScript.ScriptName = " + WScript.ScriptName);
+ok(WScript.ScriptFullName === winetest.wscriptScriptFullName, "WScript.ScriptFullName = " + WScript.ScriptFullName);
 
 winetest.reportSuccess();
