@@ -70,11 +70,20 @@ static const SID interactive_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY },
 static const SID anonymous_logon_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_ANONYMOUS_LOGON_RID } };
 static const SID authenticated_user_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_AUTHENTICATED_USER_RID } };
 static const SID local_system_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_LOCAL_SYSTEM_RID } };
+static const struct /* same fields as struct SID */
+{
+    BYTE Revision;
+    BYTE SubAuthorityCount;
+    SID_IDENTIFIER_AUTHORITY IdentifierAuthority;
+    DWORD SubAuthority[5];
+} local_user_sid = { SID_REVISION, 5, { SECURITY_NT_AUTHORITY }, { SECURITY_NT_NON_UNIQUE, 0, 0, 0, 1000 } };
+
 const PSID security_world_sid = (PSID)&world_sid;
 static const PSID security_local_sid = (PSID)&local_sid;
-const PSID security_interactive_sid = (PSID)&interactive_sid;
+static const PSID security_interactive_sid = (PSID)&interactive_sid;
 static const PSID security_authenticated_user_sid = (PSID)&authenticated_user_sid;
 const PSID security_local_system_sid = (PSID)&local_system_sid;
+const PSID security_local_user_sid = (PSID)&local_user_sid;
 
 static luid_t prev_luid_value = { 1000, 0 };
 
@@ -194,7 +203,7 @@ const SID *security_unix_uid_to_sid( uid_t uid )
 {
     /* very simple mapping: either the current user or not the current user */
     if (uid == getuid())
-        return &interactive_sid;
+        return (const SID *)&local_user_sid;
     else
         return &anonymous_logon_sid;
 }
@@ -715,7 +724,7 @@ static unsigned int token_adjust_privileges( struct token *token, const LUID_AND
             continue;
         }
 
-        if (privs[i].Attributes & SE_PRIVILEGE_REMOVE)
+        if (privs[i].Attributes & SE_PRIVILEGE_REMOVED)
             privilege_remove( privilege );
         else
         {
@@ -776,7 +785,7 @@ int token_check_privileges( struct token *token, int all_required,
         return (enabled_count > 0);
 }
 
-static int token_sid_present( struct token *token, const SID *sid, int deny )
+int token_sid_present( struct token *token, const SID *sid, int deny )
 {
     struct group *group;
 
@@ -1225,27 +1234,51 @@ DECL_HANDLER(access_check)
 }
 
 /* retrieves the SID of the user that the token represents */
-DECL_HANDLER(get_token_user)
+DECL_HANDLER(get_token_sid)
 {
     struct token *token;
 
-    reply->user_len = 0;
+    reply->sid_len = 0;
 
     if ((token = (struct token *)get_handle_obj( current->process, req->handle,
                                                  TOKEN_QUERY,
                                                  &token_ops )))
     {
-        const SID *user = token->user;
+        const SID *sid = NULL;
 
-        reply->user_len = FIELD_OFFSET(SID, SubAuthority[user->SubAuthorityCount]);
-        if (reply->user_len <= get_reply_max_size())
+        switch (req->which_sid)
         {
-            SID *user_reply = set_reply_data_size( reply->user_len );
-            if (user_reply)
-                memcpy( user_reply, user, reply->user_len );
+        case TokenUser:
+            assert(token->user);
+            sid = token->user;
+            break;
+        case TokenPrimaryGroup:
+            sid = token->primary_group;
+            break;
+        case TokenOwner:
+        {
+            struct group *group;
+            LIST_FOR_EACH_ENTRY( group, &token->groups, struct group, entry )
+            {
+                if (group->owner)
+                {
+                    sid = &group->sid;
+                    break;
+                }
+            }
+            break;
+        }
+        default:
+            set_error( STATUS_INVALID_PARAMETER );
+            break;
         }
-        else set_error( STATUS_BUFFER_TOO_SMALL );
 
+        if (sid)
+        {
+            reply->sid_len = FIELD_OFFSET(SID, SubAuthority[sid->SubAuthorityCount]);
+            if (reply->sid_len <= get_reply_max_size()) set_reply_data( sid, reply->sid_len );
+            else set_error( STATUS_BUFFER_TOO_SMALL );
+        }
         release_object( token );
     }
 }
