@@ -55,8 +55,6 @@ enum target_cpu target_cpu = CPU_x86;
 enum target_cpu target_cpu = CPU_x86_64;
 #elif defined(__sparc__)
 enum target_cpu target_cpu = CPU_SPARC;
-#elif defined(__ALPHA__)
-enum target_cpu target_cpu = CPU_ALPHA;
 #elif defined(__powerpc__)
 enum target_cpu target_cpu = CPU_POWERPC;
 #elif defined(__arm__)
@@ -90,6 +88,7 @@ static int fake_module;
 char *as_command = NULL;
 char *ld_command = NULL;
 char *nm_command = NULL;
+char *cpu_option = NULL;
 
 static int nb_res_files;
 static char **res_files;
@@ -165,6 +164,7 @@ static void set_subsystem( const char *subsystem, DLLSPEC *spec )
     if (!strcmp( str, "native" )) spec->subsystem = IMAGE_SUBSYSTEM_NATIVE;
     else if (!strcmp( str, "windows" )) spec->subsystem = IMAGE_SUBSYSTEM_WINDOWS_GUI;
     else if (!strcmp( str, "console" )) spec->subsystem = IMAGE_SUBSYSTEM_WINDOWS_CUI;
+    else if (!strcmp( str, "wince" ))   spec->subsystem = IMAGE_SUBSYSTEM_WINDOWS_CE_GUI;
     else if (!strcmp( str, "win16" )) spec->type = SPEC_WIN16;
     else fatal_error( "Invalid subsystem name '%s'\n", subsystem );
     if (major)
@@ -258,7 +258,7 @@ static const char usage_str[] =
 "   -o, --output=NAME         Set the output file name (default: stdout)\n"
 "   -r, --res=RSRC.RES        Load resources from RSRC.RES\n"
 "       --save-temps          Do not delete the generated intermediate files\n"
-"       --subsystem=SUBSYS    Set the subsystem (one of native, windows, console)\n"
+"       --subsystem=SUBSYS    Set the subsystem (one of native, windows, console, wince)\n"
 "   -u, --undefined=SYMBOL    Add an undefined reference to SYMBOL when linking\n"
 "   -v, --verbose             Display the programs invoked\n"
 "       --version             Print the version and exit\n"
@@ -380,11 +380,11 @@ static char **parse_options( int argc, char **argv, DLLSPEC *spec )
             lib_path[nb_lib_paths++] = xstrdup( optarg );
             break;
         case 'm':
-            if (strcmp( optarg, "16" ) && strcmp( optarg, "32" ) && strcmp( optarg, "64" ))
-                fatal_error( "Invalid -m option '%s', expected -m16, -m32 or -m64\n", optarg );
             if (!strcmp( optarg, "16" )) spec->type = SPEC_WIN16;
             else if (!strcmp( optarg, "32" )) force_pointer_size = 4;
-            else force_pointer_size = 8;
+            else if (!strcmp( optarg, "64" )) force_pointer_size = 8;
+            else if (!strncmp( optarg, "cpu=", 4 )) cpu_option = xstrdup( optarg + 4 );
+            else fatal_error( "Invalid -m option '%s', expected -m16, -m32, -m64 or -mcpu\n", optarg );
             break;
         case 'M':
             spec->main_module = xstrdup( optarg );
