@@ -159,7 +159,6 @@ static const struct
     { "amd64",   CPU_x86_64 },
     { "x86_64",  CPU_x86_64 },
     { "sparc",   CPU_SPARC },
-    { "alpha",   CPU_ALPHA },
     { "powerpc", CPU_POWERPC },
     { "arm",     CPU_ARM }
 };
@@ -219,8 +218,6 @@ static const enum target_cpu build_cpu = CPU_x86;
 static const enum target_cpu build_cpu = CPU_x86_64;
 #elif defined(__sparc__)
 static const enum target_cpu build_cpu = CPU_SPARC;
-#elif defined(__ALPHA__)
-static const enum target_cpu build_cpu = CPU_ALPHA;
 #elif defined(__powerpc__)
 static const enum target_cpu build_cpu = CPU_POWERPC;
 #elif defined(__arm__)
@@ -1255,7 +1252,7 @@ int main(int argc, char **argv)
     {
         if (argv[i][0] == '-')  /* option */
 	{
-	    /* determine if tihs switch is followed by a separate argument */
+	    /* determine if this switch is followed by a separate argument */
 	    next_is_arg = 0;
 	    option_arg = 0;
 	    switch(argv[i][1])
@@ -1394,6 +1391,8 @@ int main(int argc, char **argv)
                         opts.force_pointer_size = 8;
 			raw_linker_arg = 1;
                     }
+                    else if (strncmp("-mcpu=", argv[i], 6) == 0)
+                        strarray_add(opts.winebuild_args, argv[i]);
 		    break;
                 case 'n':
                     if (strcmp("-nostdinc", argv[i]) == 0)
